@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -17,6 +17,17 @@
 #include "ProData.h" // for onelab
 #include "ProParser.h" // for onelab
 
+#if defined(HAVE_ONELAB2)
+#include "NetworkUtils.h"
+#include "OnelabNetworkClient.h"
+#ifndef WIN32
+#include <pthread.h>
+#include <signal.h>
+#else
+#include <wspiapi.h>
+#endif
+#endif
+
 #if !defined(WIN32) || defined(__CYGWIN__)
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -41,6 +52,19 @@
 #include <gmsh/GmshMessage.h>
 #endif
 
+#if defined(HAVE_OCTAVE)
+#undef _D1
+#undef _D2
+#undef HAVE_ARPACK
+#include <octave/oct.h>
+#include <octave/octave.h>
+#include <octave/toplev.h>
+#endif
+
+#if defined(HAVE_PYTHON)
+#include <Python.h>
+#endif
+
 int Message::_commRank = 0;
 int Message::_commSize = 1;
 int Message::_isCommWorld = 1; // is the communicator set to WORLD (==1) or SELF (!=1)
@@ -51,9 +75,14 @@ bool Message::_operatingInTimeLoopAdaptive = false;
 int Message::_verbosity = 5;
 int Message::_progressMeterStep = 10;
 int Message::_progressMeterCurrent = 0;
+double Message::_startTime = 0.;
 std::map<std::string, double> Message::_timers;
 GmshClient* Message::_client = 0;
+#ifdef HAVE_ONELAB2
+OnelabNetworkClient* Message::_onelabClient = 0;
+#else
 onelab::client* Message::_onelabClient = 0;
+#endif
 #if !defined(HAVE_ONELAB) // if Gmsh is compiled without onelab
 onelab::server *onelab::server::_server = 0;
 #endif
@@ -84,6 +113,7 @@ static void gslErrorHandler(const char *reason, const char *file, int line,
 
 void Message::Initialize(int argc, char **argv)
 {
+  _startTime = GetTimeOfDay();
   _errorCount = 0;
 #if defined(HAVE_PETSC)
   MPI_Init(&argc, &argv);
@@ -94,23 +124,49 @@ void Message::Initialize(int argc, char **argv)
 #if defined(HAVE_GSL)
   gsl_set_error_handler(&gslErrorHandler);
 #endif
+#if defined(HAVE_OCTAVE)
+  string_vector oargv(2);
+  oargv(0) = "embedded";
+  oargv(1) = "-q";
+  octave_main(2, oargv.c_str_vec(), 1);
+#endif
+#if defined(HAVE_PYTHON)
+  Py_SetProgramName(argv[0]);
+  Py_InitializeEx(0); // Py_Initialize() handles signals
+  PySys_SetArgv(argc, argv);
+#endif
 }
 
 void Message::Finalize()
 {
 #if defined(HAVE_PETSC)
-  int flag;
-  MPI_Initialized(&flag);
-  if(flag) MPI_Finalize();
+  int initialized, finalized;
+  MPI_Initialized(&initialized);
+  MPI_Finalized(&finalized);
+  if(initialized && !finalized)
+    MPI_Finalize();
 #endif
   FinalizeSocket();
   FinalizeOnelab();
+#if defined(HAVE_PYTHON)
+  Py_Finalize();
+#endif
 }
 
 void Message::Exit(int level)
 {
+#if defined(HAVE_PETSC)
+  if(level && _commSize > 1){
+    // abort (and not finalize) in order to terminate the full job *now*
+    MPI_Abort(MPI_COMM_WORLD, level);
+  }
+#endif
   Finalize();
+#if defined(HAVE_OCTAVE)
+  clean_up_and_exit(level);
+#else
   exit(level);
+#endif
 }
 
 static int streamIsFile(FILE* stream)
@@ -258,7 +314,7 @@ void Message::Info(const char *fmt, ...)
 
 void Message::Info(int level, const char *fmt, ...)
 {
-  if((_commRank && _isCommWorld) || _verbosity < level) return;
+  if((_commRank && _isCommWorld && level > 0) || _verbosity < abs(level)) return;
   char str[1024];
   va_list args;
   va_start(args, fmt);
@@ -272,7 +328,7 @@ void Message::Info(int level, const char *fmt, ...)
     _onelabClient->sendInfo(str);
   }
   else{
-    if(_isCommWorld)
+    if(_isCommWorld && (_commSize == 1 || level > 0))
       fprintf(stdout, "Info    : %s\n", str);
     else
       fprintf(stdout, "Info    : [rank %3d] %s\n", _commRank, str);
@@ -294,7 +350,7 @@ void Message::Direct(const char *fmt, ...)
 
 void Message::Direct(int level, const char *fmt, ...)
 {
-  if((_commRank && _isCommWorld) || _verbosity < level) return;
+  if((_commRank && _isCommWorld && level > 0) || _verbosity < abs(level)) return;
   va_list args;
   va_start(args, fmt);
   char str[1024];
@@ -309,13 +365,13 @@ void Message::Direct(int level, const char *fmt, ...)
   }
   else{
     const char *c0 = "", *c1 = "";
-    if(level == 3 && !streamIsFile(stdout) && streamIsVT100(stdout)){
+    if(abs(level) == 3 && !streamIsFile(stdout) && streamIsVT100(stdout)){
       c0 = "\33[34m"; c1 = "\33[0m";  // blue
     }
-    if(_isCommWorld)
+    if(_isCommWorld && (_commSize == 1 || level > 0))
       fprintf(stdout, "%s%s%s\n", c0, str, c1);
     else
-      fprintf(stdout, "%s [rank %3d] %s%s\n", c0, _commRank, str, c1);
+      fprintf(stdout, "%s[rank %3d] %s%s\n", c0, _commRank, str, c1);
     fflush(stdout);
   }
 }
@@ -365,41 +421,98 @@ void Message::Debug(const char *fmt, ...)
   }
 }
 
+double Message::GetWallClockTime()
+{
+  return GetTimeOfDay() - _startTime;
+}
+
 void Message::Cpu(const char *fmt, ...)
 {
-  if((_commRank && _isCommWorld) || _verbosity < 5) return;
+  if(_verbosity < 5) return;
+
   va_list args;
   va_start(args, fmt);
   char str[1024];
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
 
-  Cpu(5, str);
+  Cpu(5, false, true, true, true, str);
 }
 
-void Message::Cpu(int level, const char *fmt, ...)
+void Message::Cpu(int level, bool printDate, bool printWallTime, bool printCpu,
+                  bool printMem, const char *fmt, ...)
 {
-  if((_commRank && _isCommWorld) || _verbosity < level) return;
+  if(_verbosity < abs(level)) return;
+
   double s = 0.;
   long mem = 0;
   GetResources(&s, &mem);
+  double val[2] = {s, (double)mem / 1024. / 1024.};
+  double min[2] = {val[0], val[1]};
+  double max[2] = {val[0], val[1]};
+  double sum[2] = {val[0], val[1]};
 
-  char str[1024], str2[256];
+#if defined(HAVE_PETSC)
+  if(_commSize > 1 && _isCommWorld){
+    MPI_Reduce(val, min, 2, MPI_DOUBLE, MPI_MIN, 0, MPI_COMM_WORLD);
+    MPI_Reduce(val, max, 2, MPI_DOUBLE, MPI_MAX, 0, MPI_COMM_WORLD);
+    MPI_Reduce(val, sum, 2, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
+  }
+#endif
+
+  if(_commRank && _isCommWorld && level > 0) return;
+
+  char str[1024];
   va_list args;
   va_start(args, fmt);
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
   if(strlen(fmt)) strcat(str, " ");
 
-  time_t now;
-  time(&now);
-  std::string currtime(ctime(&now));
-  currtime.resize(currtime.size() - 1);
+  std::string pdate = "";
+  if(printDate){
+    time_t now;
+    time(&now);
+    pdate = ctime(&now);
+    pdate.resize(pdate.size() - 1);
+    if(printWallTime || printCpu || (printMem && mem))
+      pdate += ", ";
+  }
 
-  if(mem)
-    sprintf(str2, "(%s, CPU = %gs, Mem = %ldMb)", currtime.c_str(), s, mem / 1024 / 1024);
-  else
-    sprintf(str2, "(%s, CPU = %gs)", currtime.c_str(), s);
+  std::string pwall = "";
+  if(printWallTime){
+    char tmp[128];
+    sprintf(tmp, "Wall = %gs", GetWallClockTime());
+    pwall = tmp;
+    if(printCpu || (printMem && mem))
+      pwall += ", ";
+  }
+
+  std::string pcpu = "";
+  if(printCpu){
+    char tmp[128];
+    if(_commSize > 1 && _isCommWorld)
+      sprintf(tmp, "CPU = %gs [%gs,%gs]", sum[0], min[0], max[0]);
+    else
+      sprintf(tmp, "CPU = %gs", max[0]);
+    pcpu = tmp;
+    if(printMem && mem)
+      pcpu += ", ";
+  }
+
+  std::string pmem = "";
+  if(mem && printMem){
+    char tmp[128];
+    if(_commSize > 1 && _isCommWorld)
+      sprintf(tmp, "Mem = %gMb [%gMb,%gMb]", sum[1], min[1], max[1]);
+    else
+      sprintf(tmp, "Mem = %gMb", max[1]);
+    pmem = tmp;
+  }
+
+  char str2[256] = "";
+  if(pdate.size() || pwall.size() || pcpu.size() || pmem.size())
+    sprintf(str2, "(%s%s%s%s)", pdate.c_str(), pwall.c_str(), pcpu.c_str(), pmem.c_str());
   strcat(str, str2);
 
   if(_client){
@@ -409,7 +522,7 @@ void Message::Cpu(int level, const char *fmt, ...)
     _onelabClient->sendInfo(str);
   }
   else{
-    if(_isCommWorld)
+    if(_isCommWorld && (_commSize == 1 || level > 0))
       fprintf(stdout, "Info    : %s\n", str);
     else
       fprintf(stdout, "Info    : [rank %3d] %s\n", _commRank, str);
@@ -432,9 +545,11 @@ void Message::ProgressMeter(int n, int N, const char *fmt, ...)
     vsnprintf(str, sizeof(str), fmt, args);
     va_end(args);
     sprintf(str2, "%3d%%    : %s", _progressMeterCurrent, str);
+#ifndef HAVE_ONELAB2
     if(_onelabClient && _onelabClient->getName() == "GetDP"){
       _onelabClient->sendProgress(str);
     }
+#endif
 
     if(N <= 0){
       if(_onelabClient && _onelabClient->getName() != "GetDPServer"){
@@ -504,8 +619,8 @@ void Message::PrintErrorCounter(const char *title)
 {
   if(!_errorCount || _commRank || _verbosity < 1) return;
 
-  Error("%s encountered %d error%s - check the log for details",
-        title, _errorCount, (_errorCount > 1) ? "s" : "");
+  Message::Error("%s encountered %d error%s - check the log for details",
+                 title, _errorCount, (_errorCount > 1) ? "s" : "");
 }
 
 void Message::InitializeSocket(std::string sockname)
@@ -529,7 +644,11 @@ void Message::SendMergeFileRequest(const std::string &filename)
     _client->MergeFile((char*)filename.c_str());
   }
   else if(_onelabClient && _onelabClient->getName() != "GetDPServer"){
+#ifdef HAVE_ONELAB2
+    _onelabClient->mergeFile(filename);
+#else
     _onelabClient->sendMergeFileRequest(filename);
+#endif
   }
 }
 
@@ -579,11 +698,52 @@ public:
 
 void Message::InitializeOnelab(std::string name, std::string sockname)
 {
+#ifdef HAVE_ONELAB2
+  if(sockname.size()){
+    UInt32 address = 0;
+    UInt16 port = 1148;
+    size_t colon = sockname.find(':');
+    if(colon != std::string::npos) {
+      address = ip4_inet_pton(sockname.substr(0,colon).c_str());
+      port = atoi(sockname.substr(colon+1).c_str());
+    }
+    OnelabNetworkClient *c = new OnelabNetworkClient(name, address, port);
+    if(!c->connect()) {
+        Error("Could not connect to ONELAB server");
+        delete c;
+    }
+    else {
+      _onelabClient = c;
+
+      onelab::string o(name + "/FileExtension", ".pro");
+      o.setVisible(false);
+      o.setAttribute("Persistent", "1");
+      _onelabClient->set(o);
+      onelab::number o2(name + "/UseCommandLine", 1.);
+      o2.setVisible(false);
+      o2.setAttribute("Persistent", "1");
+      _onelabClient->set(o2);
+      onelab::number o3(name + "/GuessModelName", 1.);
+      o3.setVisible(false);
+      o3.setAttribute("Persistent", "1");
+      _onelabClient->set(o3);
+      std::vector<onelab::string> ps;
+      _onelabClient->get(ps, name + "/Action", true);
+      if(ps.size()){
+        Info("Performing ONELAB '%s'", ps[0].getValue().c_str());
+        if(ps[0].getValue() == "initialize") Exit(0);
+      }
+      else {
+        Warning("No action for GetDP in ONELAB database");
+      }
+    }
+  }
+#else
   if(sockname.size()){
     // getdp is called by a distant onelab server
     onelab::remoteNetworkClient *c = new onelab::remoteNetworkClient(name, sockname);
     if(!c->getGmshClient()){
-      Error("Could not connect to OneLab server");
+      Message::Error("Could not connect to ONELAB server");
       delete c;
     }
     else{
@@ -592,17 +752,20 @@ void Message::InitializeOnelab(std::string name, std::string sockname)
       // so that they are also sent e.g. when the database is reset
       onelab::string o(name + "/FileExtension", ".pro");
       o.setVisible(false);
+      o.setAttribute("Persistent", "1");
       _onelabClient->set(o);
       onelab::number o2(name + "/UseCommandLine", 1.);
       o2.setVisible(false);
+      o2.setAttribute("Persistent", "1");
       _onelabClient->set(o2);
       onelab::number o3(name + "/GuessModelName", 1.);
       o3.setVisible(false);
+      o3.setAttribute("Persistent", "1");
       _onelabClient->set(o3);
       std::vector<onelab::string> ps;
       _onelabClient->get(ps, name + "/Action");
       if(ps.size()){
-        Info("Performing OneLab '%s'", ps[0].getValue().c_str());
+        Info("Performing ONELAB '%s'", ps[0].getValue().c_str());
         if(ps[0].getValue() == "initialize") Exit(0);
       }
     }
@@ -619,7 +782,7 @@ void Message::InitializeOnelab(std::string name, std::string sockname)
       _onelabClient = new onelab::localClient("GetDPServer");
       FILE *fp = FOpen(name.c_str(), "rb");
       if(fp){
-        Message::Info("Reading OneLab database '%s'", name.c_str());
+        Message::Info("Reading ONELAB database '%s'", name.c_str());
         _onelabClient->fromFile(fp);
         fclose(fp);
       }
@@ -627,6 +790,7 @@ void Message::InitializeOnelab(std::string name, std::string sockname)
         Message::Error("Could not open file '%s'", name.c_str());
     }
   }
+#endif
 }
 
 void Message::AddOnelabNumberChoice(std::string name, double val, const char *color)
@@ -635,6 +799,9 @@ void Message::AddOnelabNumberChoice(std::string name, double val, const char *co
     std::vector<double> choices;
     std::vector<onelab::number> ps;
     _onelabClient->get(ps, name);
+#ifdef HAVE_ONELAB2
+    _onelabClient->recvfrom();
+#endif
     if(ps.size()){
       choices = ps[0].getChoices();
     }
@@ -649,10 +816,12 @@ void Message::AddOnelabNumberChoice(std::string name, double val, const char *co
     ps[0].setChoices(choices);
     _onelabClient->set(ps[0]);
 
+#if !defined(BUILD_ANDROID) // FIXME: understand why this leads to crashes
     // ask Gmsh to refresh
     onelab::string o("Gmsh/Action", "refresh");
     o.setVisible(false);
     _onelabClient->set(o);
+#endif
   }
 }
 
@@ -663,6 +832,9 @@ void Message::AddOnelabStringChoice(std::string name, std::string kind,
     std::vector<std::string> choices;
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, name);
+#ifdef HAVE_ONELAB2
+    _onelabClient->recvfrom();
+#endif
     if(ps.size()){
       choices = ps[0].getChoices();
       if(std::find(choices.begin(), choices.end(), value) == choices.end())
@@ -680,11 +852,57 @@ void Message::AddOnelabStringChoice(std::string name, std::string kind,
   }
 }
 
+void Message::SetOnelabNumber(std::string name, double val, bool visible)
+{
+  if(_onelabClient){
+    std::vector<onelab::number> numbers;
+#if defined(HAVE_ONELAB2)
+    _onelabClient->get(numbers, name, "GetDP");
+#else
+    _onelabClient->get(numbers, name);
+#endif
+    if(numbers.empty()){
+      numbers.resize(1);
+      numbers[0].setName(name);
+    }
+    numbers[0].setValue(val);
+    numbers[0].setVisible(visible);
+#if defined(HAVE_ONELAB2)
+    _onelabClient->set(numbers[0], "GetDP");
+#else
+    _onelabClient->set(numbers[0]);
+#endif
+  }
+}
+
+double Message::GetOnelabNumber(std::string name)
+{
+  if(_onelabClient){
+    std::vector<onelab::number> numbers;
+#if defined(HAVE_ONELAB2)
+    _onelabClient->get(numbers, name, "GetDP");
+#else
+    _onelabClient->get(numbers, name);
+#endif
+    if(numbers.empty()){
+      Message::Error("Unknown ONELAB number parameter '%s'", name.c_str());
+      return 0.;
+    }
+    else
+      return numbers[0].getValue();
+  }
+  Message::Warning("GetNumber requires a ONELAB client");
+  return 0.;
+}
+
 void Message::GetOnelabString(std::string name, char **val)
 {
   if(_onelabClient){
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, name);
+#ifdef HAVE_ONELAB2
+    _onelabClient->recvfrom();
+#endif
     if(ps.size() && ps[0].getValue().size()){
       *val = strSave(ps[0].getValue().c_str());
       return;
@@ -698,6 +916,9 @@ std::string Message::GetOnelabAction()
   if(_onelabClient){
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, _onelabClient->getName() + "/Action");
+#ifdef HAVE_ONELAB2
+    _onelabClient->recvfrom();
+#endif
     if(ps.size()) return ps[0].getValue();
   }
   return "";
@@ -742,6 +963,7 @@ static void _setStandardOptions(onelab::parameter *p, Message::fmap &fopt,
   // numbers
   if(fopt.count("Visible")) p->setVisible(fopt["Visible"][0] ? true : false);
   if(fopt.count("ReadOnly")) p->setReadOnly(fopt["ReadOnly"][0] ? true : false);
+  if(fopt.count("NeverChanged")) p->setNeverChanged(fopt["NeverChanged"][0] ? true : false);
   if(fopt.count("ReadOnlyRange"))
     p->setAttribute("ReadOnlyRange", fopt["ReadOnlyRange"][0] ? "1" : "0");
   if(fopt.count("AutoCheck"))
@@ -752,7 +974,17 @@ void Message::ExchangeOnelabParameter(Constant *c, fmap &fopt, cmap &copt)
 {
   if(!_onelabClient) return;
 
-  std::string name = _getParameterName(c->Name, copt);
+  std::string name;
+  if(copt.count("Name"))
+    name = copt["Name"][0];
+
+  if(name.empty()){
+    if(copt.size() || fopt.size())
+      Message::Error("From now on you need to use the `Name' attribute to create a "
+                     "ONELAB parameter: `Name \"%s\"'",
+                     _getParameterName(c->Name, copt).c_str());
+    return;
+  }
 
   if(c->Type == VAR_FLOAT){
     std::vector<onelab::number> ps;
@@ -763,7 +995,7 @@ void Message::ExchangeOnelabParameter(Constant *c, fmap &fopt, cmap &copt)
       if(fopt.count("ReadOnly") && fopt["ReadOnly"][0])
         ps[0].setValue(c->Value.Float); // use local value
       else
-	c->Value.Float = ps[0].getValue(); // use value from server
+        c->Value.Float = ps[0].getValue(); // use value from server
       // keep track of these attributes, which can be changed server-side
       // (unless they are not visible, or, for the range/choices, when
       // explicitely setting these attributes as ReadOnly)
@@ -879,7 +1111,17 @@ void Message::ExchangeOnelabParameter(Group *g, fmap &fopt, cmap &copt)
 {
   if(!_onelabClient) return;
 
-  std::string name = _getParameterName(g->Name, copt);
+  std::string name;
+  if(copt.count("Name"))
+    name = copt["Name"][0];
+
+  if(name.empty()){
+    if(copt.size() || fopt.size())
+      Message::Error("From now on you need to use the `Name' attribute to create a "
+                     "ONELAB parameter: `Name \"%s\"'",
+                     _getParameterName(g->Name, copt).c_str());
+    return;
+  }
 
   std::vector<onelab::region> ps;
   _onelabClient->get(ps, name);
@@ -924,9 +1166,7 @@ void Message::ExchangeOnelabParameter(Expression *e, fmap &fopt, cmap &copt)
 {
   if(!_onelabClient) return;
 
-  std::string name = _getParameterName(e->Name, copt);
-
-  printf("exchanging function %s with OneLab!\n", name.c_str());
+  Message::Error("Exchanging functions with ONELAB in not implemented yet");
 }
 
 void Message::UndefineOnelabParameter(const std::string &name)
@@ -934,6 +1174,7 @@ void Message::UndefineOnelabParameter(const std::string &name)
   if(!_onelabClient) return;
 
   bool found = false;
+#ifndef HAVE_ONELAB2
   {
     // try to clear number with short name == name
     std::vector<onelab::number> ps;
@@ -957,6 +1198,8 @@ void Message::UndefineOnelabParameter(const std::string &name)
       }
     }
   }
+  // TODO for ONELAB2 (search only in local parameter ?)
+#endif
 
   if(!found)
     _onelabClient->clear(name);
@@ -964,6 +1207,35 @@ void Message::UndefineOnelabParameter(const std::string &name)
 
 void Message::FinalizeOnelab()
 {
+#ifdef HAVE_ONELAB2
+  if(_onelabClient){
+    // add default computation modes
+    std::string name = ((VirtualClient *)_onelabClient)->getName();
+    std::vector<onelab::string> ps;
+    _onelabClient->get(ps, name + "/Action");
+    if(ps.size()){
+      if(ps[0].getValue() != "initialize"){
+        _onelabClient->get(ps, name + "/9ComputeCommand");
+        if(ps.empty()){ // only change value if none exists
+          ps.resize(1);
+          ps[0].setName(name + "/9ComputeCommand");
+          ps[0].setValue("-solve -pos");
+        }
+        ps[0].setLabel("Compute command");
+        std::vector<std::string> choices;
+        choices.push_back("-pre");
+        choices.push_back("-cal");
+        choices.push_back("-pos");
+        choices.push_back("-solve");
+        choices.push_back("-solve -pos");
+        ps[0].setChoices(choices);
+        _onelabClient->set(ps[0]);
+      }
+    }
+    _onelabClient->disconnect(true);
+    delete _onelabClient;
+    _onelabClient = 0;
+#else
   if(_onelabClient){
     // add default computation modes
     std::string name = _onelabClient->getName();
@@ -990,6 +1262,7 @@ void Message::FinalizeOnelab()
     }
     delete _onelabClient;
     _onelabClient = 0;
+#endif
   }
 }
 
