@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -40,19 +40,20 @@
 
 #define MAX_DIM  9   /* second-rank tensor of order 3 : 3^2 = 9  */
 
-/* Please set NBR_MAX_HARMONIC to the lowest possible value for common
-   getdp versions, until we do dynamic allocation. Otherwise, some
-   postprocessing operations become almost impossible to perform in
-   3D.
-*/
+// - NBR_MAX_HARMONIC controls the size of a 'Value' ; keep this small by
+// default for efficient operations in recursive expression evaluation and
+// post-processing
+//
+// - MAX_STACK_SIZE controls the size of the stack used in the evaluation of
+// expressions ; keep this large enough by default to allow for complex expressions
 
-#if defined(HAVE_MULTIHARMONIC)
-#define NBR_MAX_HARMONIC    40
-#else
+#if !defined(HAVE_MULTIHARMONIC)
 #define NBR_MAX_HARMONIC    2
+#else
+#define NBR_MAX_HARMONIC  100
 #endif
 
-#define MAX_STACK_SIZE      40
+#define MAX_STACK_SIZE     40
 
 struct Value {
   int     Type;
@@ -115,6 +116,7 @@ struct MovingBand2D {
 #define GROUPSOFEDGESOF           12
 #define GROUPSOFFACETSOF          13
 #define GROUPSOFEDGESONNODESOF    14
+#define GROUPOFREGIONSOF          15
 
 #define EDGESOFTREEIN             21
 #define FACETSOFTREEIN            22
@@ -188,6 +190,7 @@ struct ConstraintPerRegion {
       int  RegionRefIndex, SubRegionRefIndex;
       int  FilterIndex, CoefIndex, FunctionIndex;
       int  FilterIndex2, CoefIndex2, FunctionIndex2;
+      double ToleranceFactor;
     } Link;
   } Case;
 };
@@ -333,6 +336,7 @@ struct BasisFunction {
   void    (*Function)();
   void    (*dFunction)();
   void    (*dInvFunction)();
+  void    (*dPlusFunction)();
   List_T  *SubFunction, *SubdFunction;
   int     SupportIndex, EntityIndex;
   double  Order;
@@ -417,6 +421,7 @@ struct Function {
 			same name as enclosing class */
   int     TypeOfValue, NbrArguments, NbrParameters;
   double *Para;
+  char   *String;
   struct FunctionActive *Active;
 };
 
@@ -433,6 +438,9 @@ struct FunctionActive {
     struct {
       double  Value;
     } GetVolume;
+    struct {
+      int  Value;
+    } GetNumElements;
     struct {
       List_T *Table;
     } ValueFromIndex;
@@ -507,8 +515,8 @@ struct DefineQuantity {
   /* for subspaces */
   List_T  *IndexInFunctionSpace;
 
-  /* for MH calculation */
-  List_T  *DummyFrequency;
+  /* for MH calculation - possibly reduced frequency spectrum for some quantities*/
+  List_T  *FrequencySpectrum;
 
   /* for multiple DofData vs. one FunctionSpace */
   int      DofDataIndex;
@@ -679,15 +687,18 @@ struct GlobalEquationTerm {
 #define DERHAM          4
 
 /* Term.TypeOfTimeDerivative */
-#define NODT_        0
-#define DT_          1
-#define DTDOF_       2
-#define DTDT_        3
-#define DTDTDOF_     4
-#define JACNL_      10
-#define NEVERDT_    11
-#define DTNL_       12
-#define DTDOFJACNL_ 13
+#define NODT_           0
+#define DT_             1
+#define DTDOF_          2
+#define DTDT_           3
+#define DTDTDOF_        4
+#define DTDTDTDOF_      5
+#define DTDTDTDTDOF_    6
+#define DTDTDTDTDTDOF_  7
+#define JACNL_         10
+#define NEVERDT_       11
+#define DTNL_          12
+#define DTDOFJACNL_    13
 
 /* Term.TypeOperator */
 #define NOOP       0
@@ -721,6 +732,7 @@ struct GlobalEquationTerm {
 
 #define _D1       21
 #define _D2       22
+#define _D3       23
 
 
 /* CanonicalWholeQuantity */
@@ -763,7 +775,13 @@ struct WholeQuantity {
     struct { int  Index, NbrArguments; }                         Expression;
     struct { List_T *WholeQuantity; }                            TimeDerivative;
     struct { List_T *WholeQuantity; int TimeStep; }              AtAnteriorTimeStep;
+    struct { List_T *WholeQuantity;
+             double TimeInit; double TimeFinal; }                MaxOverTime;
+    struct { List_T *WholeQuantity;
+             double TimeInit; double TimeFinal; int NbrFrequency;
+             double Exponent_f; double Exponent_b; }             FourierSteinmetz;
     struct { double *Value; }                                    CurrentValue;
+    struct { char *Name; }                                       NamedValue;
     struct { int  Index; }                                       Argument;
     struct { List_T *WholeQuantity_True, *WholeQuantity_False; } Test;
     struct { int  Index; }                                       SaveValue;
@@ -779,7 +797,8 @@ struct WholeQuantity {
              int DofNumber; }                                    DofValue;
     struct { List_T *WholeQuantity;
              int Index, NbrPoints; }                             MHTransform;
-    struct { int Index, NbrPoints, FreqOffSet; }                 MHJacNL;
+    struct { List_T *WholeQuantity;
+             int Index, NbrArguments, NbrPoints, FreqOffSet; }   MHJacNL;
   } Case;
 
 };
@@ -804,13 +823,17 @@ struct WholeQuantity {
 #define WQ_TRACE                   17
 #define WQ_ORDER                   18
 #define WQ_MHTIMEINTEGRATION       19
-#define WQ_MHTRANSFORM             199
-#define WQ_SHOWVALUE               20
-#define WQ_MHTIMEEVAL              211
-#define WQ_MHJACNL                 212
-#define WQ_POSTSAVE                214
-#define WQ_ATANTERIORTIMESTEP      22
-#define WQ_CHANGECURRENTPOSITION   30
+#define WQ_MHTRANSFORM             20
+#define WQ_SHOWVALUE               21
+#define WQ_MHTIMEEVAL              22
+#define WQ_MHJACNL                 23
+#define WQ_POSTSAVE                24
+#define WQ_ATANTERIORTIMESTEP      25
+#define WQ_CHANGECURRENTPOSITION   26
+#define WQ_MAXOVERTIME             27
+#define WQ_FOURIERSTEINMETZ        28
+#define WQ_SAVENAMEDVALUE          29
+#define WQ_NAMEDVALUESAVED         30
 
 /* TypeOperator */
 #define OP_PLUS           1
@@ -846,6 +869,7 @@ struct WholeQuantity {
 
 struct Resolution {
   char    *Name;
+  bool     Hidden;
   List_T  *DefineSystem, *Operation;
 };
 
@@ -866,7 +890,7 @@ struct DefineSystem {
 #define VAL_COMPLEX  2
 
 struct Operation {
-  int  Type, DefineSystemIndex, Rank;
+  int  Type, DefineSystemIndex, Flag;
 
   union {
     struct {
@@ -878,6 +902,9 @@ struct Operation {
     struct {
       char *String;
     } SystemCommand;
+    struct {
+      char *String;
+    } Error;
     struct {
       char *FileName;
       int ViewTag;
@@ -885,18 +912,31 @@ struct Operation {
     struct {
       char *FileName;
     } DeleteFile;
+    struct {
+      char *OldFileName, *NewFileName;
+    } RenameFile;
     struct {
       char *DirName;
     } CreateDir;
     struct {
       int     ExpressionIndex;
     } SetTime;
+    struct {
+      int     ExpressionIndex;
+    } Sleep;
     struct {
       int     ExpressionIndex;
     } Update;
     struct {
       int     GroupIndex, Type;
     } UpdateConstraint;
+    struct {
+      char *VariableName;
+      int NormType;
+    } GetResidual;
+    struct {
+      int CopyFromTimeStep;
+    } CreateSolution;
     struct {
       int     ExpressionIndex;
     } SetFrequency;
@@ -917,9 +957,10 @@ struct Operation {
     struct {
       int     NumEigenvalues;
       double  Shift_r, Shift_i;
+      int     FilterExpressionIndex;
     } EigenSolve;
     struct {
-      int     ExpressionIndex;
+      List_T  *Expressions;
     } Evaluate;
     struct {
       int     Iteration ;
@@ -990,8 +1031,12 @@ struct Operation {
       List_T  *Operation_True, *Operation_False;
     } Test;
     struct {
-      List_T  *DofNumber, *TimeStep, *Expression;
-      char    *FileOut;
+      int     ExpressionIndex;
+      List_T  *Operation;
+    } While;
+    struct {
+      List_T  *DofNumber, *TimeStep, *Expressions;
+      char    *FileOut, *FormatString;
     } Print;
     struct {
       int     GroupIndex, ExpressionIndex;
@@ -1034,9 +1079,6 @@ struct Operation {
       int     NbrStep;
       List_T  *Operation;
     } Generate_MH_Moving_S;
-    struct {
-      double  dummy;
-    } Add_MH_Moving;
     struct {
       int     GroupIndex;
     } Generate;
@@ -1050,14 +1092,14 @@ struct Operation {
       char    *Name_MshFile;
       int     GeoDataIndex;
       double  Factor;
+      int     GroupIndex;
     } DeformeMesh;
-    // FIXME: Roman
     struct {
-      List_T *SystemIndex, *ExpectationIndex;
-      List_T *LocalMatrixIndex;
-      List_T *ExpansionCoef;
-    } TensorProductSolve;
-
+      char    *String;
+    } SetGlobalSolverOptions;
+    struct {
+      List_T  *FieldsToSkip;
+    } BroadcastFields;
   } Case;
 
 };
@@ -1102,87 +1144,98 @@ struct IterativeLoopSystem {
 /* Operation.Type */
 #define OPERATION_NONE                      0
 
-#define OPERATION_GENERATE                  1
-#define OPERATION_SOLVE                     2
-#define OPERATION_SOLVEAGAIN              223
-#define OPERATION_SOLVEAGAINWITHOTHER     225
-#define OPERATION_SOLVENL                 233
-#define OPERATION_GENERATEJAC               3
-#define OPERATION_GENERATERHS             100
-#define OPERATION_GENERATEONLY            101
-#define OPERATION_GENERATEONLYJAC         103
-#define OPERATION_SOLVEJAC                  4
-#define OPERATION_SOLVEJACAGAIN           224
-#define OPERATION_SOLVEJACADAPTRELAX      888
-#define OPERATION_GENERATESEPARATE          5
-#define OPERATION_UPDATE                    6
-#define OPERATION_UPDATECONSTRAINT          7
-#define OPERATION_LANCZOS                   8
-#define OPERATION_PERTURBATION              9
-#define OPERATION_EIGENSOLVE               16
-#define OPERATION_EIGENSOLVEJAC            17
-#define OPERATION_EVALUATE                 18
-#define OPERATION_SELECTCORRECTION         80
-#define OPERATION_ADDCORRECTION            81
-#define OPERATION_INITCORRECTION           82
-#define OPERATION_MULTIPLYSOLUTION         83
-#define OPERATION_ADDOPPOSITEFULLSOLUTION  84
-#define OPERATION_TENSORPRODUCTSOLVE       85
-
-#define OPERATION_SAVESOLUTION             10
-#define OPERATION_SAVESOLUTIONS            11
-#define OPERATION_SAVESOLUTION_WITH_ENTITY_NUM 121
-#define OPERATION_SAVESOLUTIONEXTENDEDMH  111
-#define OPERATION_SAVESOLUTIONMHTOTIME    131
-#define OPERATION_INIT_MOVINGBAND2D       444
-#define OPERATION_MESH_MOVINGBAND2D       222
-#define OPERATION_GENERATE_MH_MOVING      999
-#define OPERATION_GENERATE_MH_MOVING_S   9991
-#define OPERATION_ADD_MH_MOVING          9992
-#define OPERATION_DUMMYDOFS              9993
-
-#define OPERATION_SAVEMESH                333
-#define OPERATION_DEFORMEMESH             334
-#define OPERATION_READSOLUTION             12
-#define OPERATION_TRANSFERSOLUTION         13
-#define OPERATION_INITSOLUTION             15
-#define OPERATION_INITSOLUTION1           115
-#define OPERATION_SETCURRENTSYSTEM         70
-
-#define OPERATION_SETTIME                  20
-#define OPERATION_SETFREQUENCY             21
-#define OPERATION_TEST                     22
-#define OPERATION_FOURIERTRANSFORM         23
-#define OPERATION_FOURIERTRANSFORM2       777
-#define OPERATION_BREAK                    24
-#define OPERATION_PRINT                    25
-#define OPERATION_WRITE                    26
-#define OPERATION_SCAN                     27
-#define OPERATION_READ                     28
-
-#define OPERATION_TIMELOOPTHETA            30
-#define OPERATION_TIMELOOPNEWMARK          31
-#define OPERATION_ITERATIVELOOP            32
-#define OPERATION_ITERATIVETIMEREDUCTION   33
-#define OPERATION_TIMELOOPRUNGEKUTTA       34
-#define OPERATION_ITERATIVELINEARSOLVER    35
-#define OPERATION_TIMELOOPADAPTIVE         36
-#define OPERATION_ITERATIVELOOPN           37
-
-#define OPERATION_CHANGEOFCOORDINATES      40
-#define OPERATION_CHANGEOFCOORDINATES2    400
-
-#define OPERATION_SYSTEMCOMMAND            50
-
-#define OPERATION_POSTOPERATION            60
-#define OPERATION_GMSHREAD                 61
-#define OPERATION_GMSHCLEARALL             62
-#define OPERATION_DELETEFILE               71
-#define OPERATION_CREATEDIR                72
-
-#define OPERATION_SETCOMMSELF              63
-#define OPERATION_SETCOMMWORLD             64
-#define OPERATION_BARRIER                  65
+#define OPERATION_ADDCORRECTION             1
+#define OPERATION_ADDOPPOSITEFULLSOLUTION   2
+#define OPERATION_ADDMHMOVING               3
+#define OPERATION_APPLY                     4
+#define OPERATION_BARRIER                   5
+#define OPERATION_BREAK                     6
+#define OPERATION_BROADCASTFIELDS           7
+#define OPERATION_CHANGEOFCOORDINATES       8
+#define OPERATION_CHANGEOFCOORDINATES2      9
+#define OPERATION_CREATEDIR                10
+#define OPERATION_DEFORMEMESH              11
+#define OPERATION_DELETEFILE               12
+#define OPERATION_DOFSFREQUENCYSPECTRUM    13
+#define OPERATION_EIGENSOLVE               14
+#define OPERATION_EIGENSOLVEJAC            15
+#define OPERATION_EVALUATE                 16
+#define OPERATION_FOURIERTRANSFORM         17
+#define OPERATION_FOURIERTRANSFORM2        18
+#define OPERATION_GENERATE                 19
+#define OPERATION_GENERATEJAC              20
+#define OPERATION_GENERATEJAC_CUMULATIVE   21
+#define OPERATION_GENERATEONLY             22
+#define OPERATION_GENERATEONLYJAC          23
+#define OPERATION_GENERATERHS              24
+#define OPERATION_GENERATERHS_CUMULATIVE   25
+#define OPERATION_GENERATESEPARATE         26
+#define OPERATION_GENERATE_CUMULATIVE      27
+#define OPERATION_GENERATE_MH_MOVING       28
+#define OPERATION_GENERATE_MH_MOVING_S     29
+#define OPERATION_GMSHCLEARALL             30
+#define OPERATION_GMSHMERGE                31
+#define OPERATION_GMSHOPEN                 32
+#define OPERATION_GMSHREAD                 33
+#define OPERATION_GMSHWRITE                34
+#define OPERATION_INITCORRECTION           35
+#define OPERATION_INITSOLUTION             36
+#define OPERATION_INITSOLUTION1            37
+#define OPERATION_INIT_MOVINGBAND2D        38
+#define OPERATION_ITERATIVELINEARSOLVER    39
+#define OPERATION_ITERATIVELOOP            40
+#define OPERATION_ITERATIVELOOPN           41
+#define OPERATION_ITERATIVETIMEREDUCTION   42
+#define OPERATION_LANCZOS                  43
+#define OPERATION_MESH_MOVINGBAND2D        44
+#define OPERATION_MULTIPLYSOLUTION         45
+#define OPERATION_PERTURBATION             46
+#define OPERATION_POSTOPERATION            47
+#define OPERATION_PRINT                    48
+#define OPERATION_READ                     49
+#define OPERATION_READSOLUTION             50
+#define OPERATION_SAVEMESH                 51
+#define OPERATION_SAVESOLUTION             52
+#define OPERATION_SAVESOLUTIONEXTENDEDMH   53
+#define OPERATION_SAVESOLUTIONMHTOTIME     54
+#define OPERATION_SAVESOLUTIONS            55
+#define OPERATION_SAVESOLUTION_WITH_ENTITY_NUM 56
+#define OPERATION_SCAN                     57
+#define OPERATION_SELECTCORRECTION         58
+#define OPERATION_SETCOMMSELF              59
+#define OPERATION_SETCOMMWORLD             60
+#define OPERATION_SETCURRENTSYSTEM         61
+#define OPERATION_SETFREQUENCY             62
+#define OPERATION_SETGLOBALSOLVEROPTIONS   63
+#define OPERATION_SETRHSASSOLUTION         64
+#define OPERATION_SETSOLUTIONASRHS         65
+#define OPERATION_SETTIME                  66
+#define OPERATION_SOLVE                    67
+#define OPERATION_SOLVEAGAIN               68
+#define OPERATION_SOLVEAGAINWITHOTHER      69
+#define OPERATION_SOLVEJAC                 70
+#define OPERATION_SOLVEJACADAPTRELAX       71
+#define OPERATION_SOLVEJACAGAIN            72
+#define OPERATION_SOLVENL                  73
+#define OPERATION_SYSTEMCOMMAND            74
+#define OPERATION_TEST                     75
+#define OPERATION_TIMELOOPADAPTIVE         76
+#define OPERATION_TIMELOOPNEWMARK          77
+#define OPERATION_TIMELOOPRUNGEKUTTA       78
+#define OPERATION_TIMELOOPTHETA            79
+#define OPERATION_TRANSFERSOLUTION         80
+#define OPERATION_UPDATE                   81
+#define OPERATION_UPDATECONSTRAINT         82
+#define OPERATION_WRITE                    83
+#define OPERATION_GETRESIDUAL              84
+#define OPERATION_RENAMEFILE               85
+#define OPERATION_WHILE                    86
+#define OPERATION_SETTIMESTEP              87
+#define OPERATION_ERROR                    88
+#define OPERATION_SLEEP                    89
+#define OPERATION_SWAPSOLUTIONANDRHS       90
+#define OPERATION_SWAPSOLUTIONANDRESIDUAL  91
+#define OPERATION_CREATESOLUTION           92
 
 /* ChangeOfState.Type */
 #define CHANGEOFSTATE_NOCHANGE              0
@@ -1261,32 +1314,42 @@ struct PostQuantityTerm {
 
 struct PostOperation {
   char    *Name, *AppendString;
+  bool     Hidden;
   int      PostProcessingIndex, Format;
   List_T  *PostSubOperation;
   int      Rank;
   double   ResampleTimeStart, ResampleTimeStop, ResampleTimeStep;
   bool     ResampleTime;
+  List_T  *TimeValue_L, *TimeImagValue_L;
+  int      LastTimeStepOnly, OverrideTimeStepValue, NoMesh, CatFile;
 };
 
 struct PostSubOperation {
   int    PostQuantityIndex[2], PostQuantitySupport[2];
   int    Type, SubType, CombinationType;
   int    Depth, Skin, Smoothing, Dimension, Comma, HarmonicToTime, CatFile;
+  int    FourierTransform;
   int    Format, Adapt, Sort, Iso, NoNewLine, NoTitle, DecomposeInSimplex;
   int    NewCoordinates;
   char  *NewCoordinatesFile;
   int    ValueIndex;
   int    ChangeOfCoordinates[3], LastTimeStepOnly, AppendTimeStepToFileName;
+  int    AppendExpressionToFileName;
+  char  *AppendStringToFileName, *AppendExpressionFormat;
   int    OverrideTimeStepValue, NoMesh;
-  int    StoreInRegister;
+  char  *StoreInVariable;
+  int    StoreInRegister, StoreMinInRegister, StoreMinXinRegister;
+  int    StoreMinYinRegister, StoreMinZinRegister, StoreMaxInRegister;
+  int    StoreMaxXinRegister, StoreMaxYinRegister, StoreMaxZinRegister;
   char  *SendToServer, *Color;
-  int    StoreInField;
+  int    StoreInField, StoreInMeshBasedField;
   int    Legend, FrozenTimeStepList;
   double LegendPosition[3];
   double Target;
   char   *ValueName, *Label;
   char   *FileOut;
   List_T *TimeStep_L, *Value_L, *Iso_L, *Frequency_L;
+  List_T *TimeValue_L, *TimeImagValue_L;
   List_T *ChangeOfValues;
   List_T *EvaluationPoints;
   union {
@@ -1310,6 +1373,7 @@ struct PostOpSolutions {
 #define POP_PRINT         1
 #define POP_GROUP         2
 #define POP_EXPRESSION    4
+#define POP_MERGE         5
 
 /* PostOperation.SubType */
 #define PRINT_ONREGION        1
@@ -1387,7 +1451,7 @@ struct PostOpSolutions {
 struct CurrentData {
   char   *Name;
 
-  int     NbrSystem, NbrCpu, RankCpu;
+  int     NbrSystem;
   struct DefineSystem   *DefineSystem_P ;
   struct DofData  *DofData_P0;
 
@@ -1429,6 +1493,8 @@ struct CurrentData {
 
   double  Val[NBR_MAX_HARMONIC * MAX_DIM];
 
+  double  QuadraturePointIndex;
+
   // For TimeLoopTheta and TimeLoopNewmark
   double  Time, TimeImag, TimeStep, DTime;
   double  Theta, Beta, Gamma;
@@ -1441,6 +1507,9 @@ struct CurrentData {
   // For IterativeLoop
   double  Iteration, RelativeDifference, RelativeDifferenceOld;
   double  RelaxationFactor;
+
+  // Iterative linear system solvers
+  double  KSPIts;
 };
 
 /* ------------------------------------------------------------------------ */
@@ -1579,7 +1648,7 @@ void Finalize_ProblemStructure();
 void Print_ProblemStructure();
 void Free_ProblemStructure();
 
-std::string Get_AbsolutePath(const char *fileName);
+std::string Fix_RelativePath(const char *fileName, const char *reference=0);
 
 void Print_ListResolution(int choice, int flag_lres, char **name);
 void Print_ListPostOperation(int choice, int flag_lpos, char *name[NBR_MAX_POS]);
