@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -44,6 +44,7 @@ struct StringXDefine  FunctionForGroup_Type[] = {
   {"GroupsOfEdgesOnNodesOf"   , GROUPSOFEDGESONNODESOF},
   {"GroupsOfEdgesOf"          , GROUPSOFEDGESOF},
   {"GroupsOfFacetsOf"         , GROUPSOFFACETSOF},
+  {"GroupOfRegionsOf"         , GROUPOFREGIONSOF},
   {"EdgesOfTreeIn"            , EDGESOFTREEIN},
   {"FacetsOfTreeIn"           , FACETSOFTREEIN},
   {"DualNodesOf"              , DUALNODESOF},
@@ -106,19 +107,19 @@ struct StringXDefine  Integration_SubType[] = {
 } ;
 
 struct StringXDefine  Element_Type[] = {
-  {"Point"       , POINT},
-  {"Line"        , LINE},
-  {"Triangle"    , TRIANGLE},
-  {"Quadrangle"  , QUADRANGLE},
-  {"Tetrahedron" , TETRAHEDRON},
-  {"Hexahedron"  , HEXAHEDRON},
-  {"Prism"       , PRISM},
-  {"Pyramid"     , PYRAMID},
-  {"Line2"       , LINE_2},
-  {"Triangle2"   , TRIANGLE_2},
-  {"Quadrangle2" , QUADRANGLE_2},
-  {"Quadrangle2_8N", QUADRANGLE_2_8N},
-  {NULL          , TRIANGLE}
+  {"Point"          , POINT},
+  {"Line"           , LINE},
+  {"Triangle"       , TRIANGLE},
+  {"Quadrangle"     , QUADRANGLE},
+  {"Tetrahedron"    , TETRAHEDRON},
+  {"Hexahedron"     , HEXAHEDRON},
+  {"Prism"          , PRISM},
+  {"Pyramid"        , PYRAMID},
+  {"Line2"          , LINE_2},
+  {"Triangle2"      , TRIANGLE_2},
+  {"Quadrangle2"    , QUADRANGLE_2},
+  {"Quadrangle2_8N" , QUADRANGLE_2_8N},
+  {NULL             , TRIANGLE}
  } ;
 
 struct StringXDefine  GlobalQuantity_Type[] = {
@@ -175,6 +176,7 @@ struct StringXDefine  Operator_Type[] = {
 
   {"D1"      , _D1 } ,
   {"D2"      , _D2 } ,
+  {"D3"      , _D3 } ,
 
   {NULL      , NOOP }
 } ;
@@ -195,28 +197,37 @@ struct StringXDefine  DefineSystem_Type[] = {
 } ;
 
 struct StringXDefine  Operation_Type[] = {
+  {"Apply"                  , OPERATION_APPLY},
+  {"DofsFrequencySpectrum"  , OPERATION_DOFSFREQUENCYSPECTRUM},
   {"Generate"               , OPERATION_GENERATE},
+  {"GenerateCumulative"     , OPERATION_GENERATE_CUMULATIVE},
+  {"GenerateJac"            , OPERATION_GENERATEJAC},
+  {"GenerateJacCumulative"  , OPERATION_GENERATEJAC_CUMULATIVE},
   {"GenerateOnly"           , OPERATION_GENERATEONLY},
   {"GenerateOnlyJac"        , OPERATION_GENERATEONLYJAC},
-  {"Update"                 , OPERATION_UPDATE},
-  {"Solve"                  , OPERATION_SOLVE},
-  {"SolveAgain"             , OPERATION_SOLVEAGAIN},
-  {"GenerateJac"            , OPERATION_GENERATEJAC},
-  {"SolveJac"               , OPERATION_SOLVEJAC},
-  {"SolveJacAgain"          , OPERATION_SOLVEJACAGAIN},
   {"GenerateRHS"            , OPERATION_GENERATERHS},
-  {"SolveNL"                , OPERATION_SOLVENL},
+  {"GenerateRHSCumulative"  , OPERATION_GENERATERHS_CUMULATIVE},
   {"GenerateSeparate"       , OPERATION_GENERATESEPARATE},
+  {"InitCorrection"         , OPERATION_INITCORRECTION},
   {"InitSolution"           , OPERATION_INITSOLUTION},
   {"InitSolution1"          , OPERATION_INITSOLUTION1},
-  {"SaveSolution"           , OPERATION_SAVESOLUTION},
-  {"SaveSolutions"          , OPERATION_SAVESOLUTIONS},
   {"ReadSolution"           , OPERATION_READSOLUTION},
-  {"TransferSolution"       , OPERATION_TRANSFERSOLUTION},
-  {"SolveJac_AdaptRelax"    , OPERATION_SOLVEJACADAPTRELAX},
+  {"SaveSolution"           , OPERATION_SAVESOLUTION},
   {"SaveSolutionExtendedMH" , OPERATION_SAVESOLUTIONEXTENDEDMH},
-  {"DummyDofs"              , OPERATION_DUMMYDOFS},
-  {"InitCorrection"         , OPERATION_INITCORRECTION},
+  {"SaveSolutions"          , OPERATION_SAVESOLUTIONS},
+  {"SetCurrentSystem"       , OPERATION_SETCURRENTSYSTEM},
+  {"SetRHSAsSolution"       , OPERATION_SETRHSASSOLUTION},
+  {"SetSolutionAsRHS"       , OPERATION_SETSOLUTIONASRHS},
+  {"SwapSolutionAndRHS"     , OPERATION_SWAPSOLUTIONANDRHS},
+  {"SwapSolutionAndResidual", OPERATION_SWAPSOLUTIONANDRESIDUAL},
+  {"Solve"                  , OPERATION_SOLVE},
+  {"SolveAgain"             , OPERATION_SOLVEAGAIN},
+  {"SolveJac"               , OPERATION_SOLVEJAC},
+  {"SolveJacAgain"          , OPERATION_SOLVEJACAGAIN},
+  {"SolveJac_AdaptRelax"    , OPERATION_SOLVEJACADAPTRELAX},
+  {"SolveNL"                , OPERATION_SOLVENL},
+  {"TransferSolution"       , OPERATION_TRANSFERSOLUTION},
+  {"Update"                 , OPERATION_UPDATE},
   {NULL                     , OPERATION_NONE}
 } ;
 
@@ -246,11 +257,13 @@ struct StringXDefine  NormOf_Type[] = {
 
 struct StringXPointer  Current_Value[] = {
 #if defined(HAVE_LEGACY)
-  {"Time"     , &Current.Time},       {"DTime"    , &Current.DTime},
-  {"Theta"    , &Current.Theta},      {"TimeStep" , &Current.TimeStep},
-  {"Iteration", &Current.Iteration},  {"Iter"     , &Current.Iteration},
-  {"TimeImag" , &Current.TimeImag},   {"Eigenvalue" , &Current.Time},
+  {"Time"     , &Current.Time},       {"DTime"     , &Current.DTime},
+  {"Theta"    , &Current.Theta},      {"TimeStep"  , &Current.TimeStep},
+  {"Iteration", &Current.Iteration},  {"Iter"      , &Current.Iteration},
+  {"TimeImag" , &Current.TimeImag},   {"Eigenvalue", &Current.Time},
   {"EigenvalueReal" , &Current.Time}, {"EigenvalueImag" , &Current.TimeImag},
+  {"ReOmega"  , &Current.Time},       {"ImOmega"   , &Current.TimeImag},
+  {"wr"       , &Current.Time},       {"wi"        , &Current.TimeImag},
   {"Breakpoint" , &Current.Breakpoint},
 
   {"X"  , &Current.x}, {"Y"  , &Current.y},  {"Z"  , &Current.z},
@@ -265,6 +278,11 @@ struct StringXPointer  Current_Value[] = {
   {"Val0" , &Current.Val[0]}, {"Val1" , &Current.Val[1]}, {"Val2" , &Current.Val[2]},
   {"Val3" , &Current.Val[3]}, {"Val4" , &Current.Val[4]}, {"Val5" , &Current.Val[5]},
   {"Val6" , &Current.Val[6]}, {"Val7" , &Current.Val[7]}, {"Val8" , &Current.Val[8]},
+
+  {"QuadraturePointIndex",  &Current.QuadraturePointIndex},
+  {"QP",                    &Current.QuadraturePointIndex},
+
+  {"KSPIts", &Current.KSPIts},
 #endif
   {NULL       , NULL}
 } ;
@@ -411,6 +429,8 @@ struct StringX3Function3Nbr  BF_Function[] = {
   {"BF_Node_3V", (CAST)BF_Node_3V,
                  (CAST)BF_GradNode_3V, (CAST)BF_Zero, 3., HEX|PRI|PYR, 0 },
 
+  {"BF_GradNodeRealCoord",    (CAST)BF_GradNodeRealCoord,
+                     (CAST)BF_Zero, (CAST)BF_Node,    0., ALL, 0 },
   {"BF_GradNode",    (CAST)BF_GradNode,
                      (CAST)BF_Zero, (CAST)BF_Node,    0., ALL, 0 },
   {"BF_GradNode_1N", (CAST)BF_GradNode,
@@ -839,7 +859,8 @@ struct StringX3Function3Nbr  BF_Function[] = {
   // Special basis Functions
 
   {"BF_Zero", (CAST)BF_Zero, (CAST)BF_Zero, (CAST)BF_Zero, 0., ALL, 0 },
-  {"BF_One",  (CAST)BF_One,  (CAST)BF_One,  (CAST)BF_One,  0., ALL, 0 },
+  {"BF_One",  (CAST)BF_One,  (CAST)BF_Zero, (CAST)BF_One,  0., ALL, 0 },
+  {"BF_OneZ", (CAST)BF_OneZ, (CAST)BF_Zero, (CAST)BF_One,  0., ALL, 0 },
 
   {"BF_Region"  , (CAST)BF_Region   , (CAST)BF_dRegion  , (CAST)BF_Zero , 0. , ALL, 0 },
   {"BF_RegionX" , (CAST)BF_RegionX  , (CAST)BF_dRegionX , (CAST)BF_Zero , 0. , ALL, 0 },
@@ -907,8 +928,11 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"Unit"              , (CAST)F_Unit             ,   0,   1 },
   {"ScalarUnit"        , (CAST)F_ScalarUnit       ,   0,   1 },
   {"Normalized"        , (CAST)F_Unit             ,   0,   1 },
+  {"Cos_wt_p"          , (CAST)F_Cos_wt_p         ,   2,   0 },
+  {"Sin_wt_p"          , (CAST)F_Sin_wt_p         ,   2,   0 },
   {"F_Cos_wt_p"        , (CAST)F_Cos_wt_p         ,   2,   0 },
   {"F_Sin_wt_p"        , (CAST)F_Sin_wt_p         ,   2,   0 },
+  {"Period"            , (CAST)F_Period           ,   1,   1 },
   {"F_Period"          , (CAST)F_Period           ,   1,   1 },
   {"Interval"          , (CAST)F_Interval         ,   3,   3 },
   {"Complex"           , (CAST)F_Complex          ,   0,  -2 },
@@ -923,6 +947,7 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"TensorSym"         , (CAST)F_TensorSym        ,   0,   6 },
   {"TensorDiag"        , (CAST)F_TensorDiag       ,   0,   3 },
   {"SquDyadicProduct"  , (CAST)F_SquDyadicProduct ,   0,   1 },
+  {"Comp"              , (CAST)F_Comp             ,   1,   1 },
   {"CompX"             , (CAST)F_CompX            ,   0,   1 },
   {"CompY"             , (CAST)F_CompY            ,   0,   1 },
   {"CompZ"             , (CAST)F_CompZ            ,   0,   1 },
@@ -965,13 +990,47 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"ElementVol"        , (CAST)F_ElementVol       ,   0,   0 },
   {"SurfaceArea"       , (CAST)F_SurfaceArea      ,  -1,   0 },
   {"GetVolume"         , (CAST)F_GetVolume        ,  -1,   0 },
+  {"GetNumElements"    , (CAST)F_GetNumElements   ,  -1,   0 },
+  {"CellSize"          , (CAST)F_CellSize         ,  -1,   0 },
   {"ProjectPointOnEllipse", (CAST)F_ProjectPointOnEllipse,  2, 1 },
+  {"SquNormEdgeValues" , (CAST)F_SquNormEdgeValues,   2,   0 },
 
   // F_Misc
   {"Printf"            , (CAST)F_Printf           ,   0,   1 },
   {"Rand"              , (CAST)F_Rand             ,   0,   1 },
   {"CompElementNum"    , (CAST)F_CompElementNum   ,   0,   0 },
+  {"ElementNum"        , (CAST)F_ElementNum       ,   0,   0 },
+  {"QuadraturePointIndex" , (CAST)F_QuadraturePointIndex ,   0,   0 },
+  {"CpuTime"           , (CAST)F_GetCpuTime       ,   0,   0 },
+  {"GetCpuTime"        , (CAST)F_GetCpuTime       ,   0,   0 },
+  {"GetWallClockTime"  , (CAST)F_GetWallClockTime ,   0,   0 },
+  {"Memory"            , (CAST)F_GetMemory        ,   0,   0 },
+  {"GetMemory"         , (CAST)F_GetMemory        ,   0,   0 },
+  {"SetNumber"         , (CAST)F_SetNumber        ,   -1,   -1 },
+  {"GetNumber"         , (CAST)F_GetNumber        ,   -1,   -1 },
   {"VirtualWork"       , (CAST)F_VirtualWork      ,   0,   1 },
+  {"Felec"	       , (CAST)F_Felec      	  ,   0,   1 },
+
+  {"dFxdux"	       , (CAST)F_dFxdux      	  ,   0,   1 },
+  {"dFydux"	       , (CAST)F_dFydux      	  ,   0,   1 },
+  {"dFzdux"	       , (CAST)F_dFzdux      	  ,   0,   1 },
+  {"dFxduy"	       , (CAST)F_dFxduy      	  ,   0,   1 },
+  {"dFyduy"	       , (CAST)F_dFyduy      	  ,   0,   1 },
+  {"dFzduy"	       , (CAST)F_dFzduy      	  ,   0,   1 },
+  {"dFxduz"	       , (CAST)F_dFxduz      	  ,   0,   1 },
+  {"dFyduz"	       , (CAST)F_dFyduz      	  ,   0,   1 },
+  {"dFzduz"	       , (CAST)F_dFzduz      	  ,   0,   1 },
+
+  {"dFxdv"             , (CAST)F_dFxdv      	  ,   0,   1 },
+  {"dFydv"             , (CAST)F_dFydv      	  ,   0,   1 },
+  {"dFzdv"             , (CAST)F_dFzdv      	  ,   0,   1 },
+
+  {"dWedxdv"	       , (CAST)F_dWedxdv      	  ,   0,   1 },
+  {"dWedydv"	       , (CAST)F_dWedydv      	  ,   0,   1 },
+  {"dWedzdv"	       , (CAST)F_dWedzdv      	  ,   0,   1 },
+
+  {"NodeForceDensity"  , (CAST)F_NodeForceDensity ,   0,   1 },
+  {"AssDiag"           , (CAST)F_AssDiag          ,  -1,   0 },
 
   // F_Interpolation
   {"InterpolationLinear"   , (CAST)F_InterpolationLinear   ,  -2,   1 },
@@ -999,6 +1058,11 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"dbdh_Vinch"            , (CAST)F_dbdh_Vinch            ,  -1,   3 },
   {"Update_Jk"             , (CAST)F_Update_Jk             ,  -1,   6 },
   {"Update_Jk_sd"          , (CAST)F_Update_Jk_sd          ,  -1,   6 },
+  {"b_Vinch_K"             , (CAST)F_b_Vinch_K             ,  -1,   15 },  // 3+3*4=15
+  {"h_Vinch_K"             , (CAST)F_h_Vinch_K             ,   1,   17 },  // parameter is dimension {2},{3}, 5+3*4=17
+  {"dbdh_Vinch_K"          , (CAST)F_dbdh_Vinch_K          ,   1,   15 },  // parameter is dimension {2},{3}, 3+3*4=15
+  {"dhdb_Vinch_K"          , (CAST)F_dhdb_Vinch_K          ,   1,   15 },  // parameter is dimension {2},{3}, 3+3*4=15
+  {"nu_Vinch_K"            , (CAST)F_nu_Vinch_K            ,   1,   17 },  // parameter is dimension {2},{3}, 5+3*4=17
 
   // F_MultiHar
   {"MHToTime"          , (CAST)F_MHToTime         ,   0,   2 },
@@ -1032,6 +1096,7 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"DthetaAcousticFieldHardCylinder", (CAST)F_DthetaAcousticFieldHardCylinder, 2, 1 },
   {"RCSHardCylinder",                 (CAST)F_RCSHardCylinder,  2, 1 },
   {"OSRC_C0",   (CAST)F_OSRC_C0, 2, 0},
+  {"OSRC_R0",   (CAST)F_OSRC_R0, 2, 0},
   {"OSRC_Aj",   (CAST)F_OSRC_Aj, 3, 0},
   {"OSRC_Bj",   (CAST)F_OSRC_Bj, 3, 0},
 
@@ -1051,6 +1116,10 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"ComplexScalarField", (CAST)F_ComplexScalarField,   -1,   -1 },
   {"ComplexVectorField", (CAST)F_ComplexVectorField,   -1,   -1 },
   {"ComplexTensorField", (CAST)F_ComplexTensorField,   -1,   -1 },
+  {"GradScalarField",        (CAST)F_GradScalarField       ,   -1,   -1 },
+  {"GradVectorField",        (CAST)F_GradVectorField       ,   -1,   -1 },
+  {"GradComplexScalarField", (CAST)F_GradComplexScalarField,   -1,   -1 },
+  {"GradComplexVectorField", (CAST)F_GradComplexVectorField,   -1,   -1 },
 
   // F_DiffGeom
   {"Hodge",              (CAST)F_Hodge             ,    1,   2 },
@@ -1066,6 +1135,12 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"InvPullBack",        (CAST)F_InvPullBack       ,    1,   2 },
   {"InvPushForward",     (CAST)F_InvPushForward    ,    1,   2 },
 
+  // F_Octave
+  {"Octave",             (CAST)F_Octave            ,    -1,   -1 },
+
+  // F_Python
+  {"Python",             (CAST)F_Python            ,    -1,   -1 },
+
   // GF_XXX : Green Functions
   {"Laplace"           , (CAST)GF_Laplace          ,   1,   0 },
   {"GradLaplace"       , (CAST)GF_GradLaplace      ,   1,   0 },
