@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -9,6 +9,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <map>
+#include <vector>
 #include <string>
 #include "ListUtils.h"
 
@@ -27,13 +28,13 @@ struct Constant {
 #define VAR_CHAR          3
 
 extern FILE *getdp_yyin;
-extern char getdp_yyname[256];
+extern std::string getdp_yyname;
 extern char getdp_yyincludename[256];
 extern long int getdp_yylinenum;
 extern int getdp_yycolnum;
 extern int getdp_yyincludenum;
 extern int getdp_yyerrorlevel;
-extern std::map<std::string, double> CommandLineNumbers;
+extern std::map<std::string, std::vector<double> > CommandLineNumbers;
 extern std::map<std::string, std::string> CommandLineStrings;
 
 int getdp_yyparse();
@@ -44,6 +45,9 @@ void cStyleComments();
 void cxxStyleComments();
 void parseString(char endchar);
 void skipUntil(const char *skip, const char *until);
+void skipUntil_test(const char *skip, const char *until,
+                    const char *until2, int l_until2_sub, int *type_until2);
 void Print_Constants();
+int  Print_ListOfDouble(char *format, List_T *list, char *buffer);
 
 #endif
