@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -8,6 +8,7 @@
 //   Ruth Sabariego
 //
 
+#include <map>
 #include <math.h>
 #include "ProData.h"
 #include "ProDefine.h"
@@ -29,6 +30,9 @@
 extern struct Problem Problem_S ;
 extern struct CurrentData Current ;
 
+std::map<int, bool> assDiag_done;
+
+
 /* ------------------------------------------------------------------------ */
 /*  C a l _ I n i t G a l e r k i n T e r m O f F e m E q u a t i o n       */
 /* ------------------------------------------------------------------------ */
@@ -39,7 +43,8 @@ void Cal_InitGalerkinTermOfFemEquation(struct EquationTerm     * EquationTerm_P,
 				       struct Dof              * DofForNoDof_P)
 {
   struct FemLocalTermActive  * FI ;
-  extern int MH_Moving_Matrix_simple, MH_Moving_Matrix_probe, MH_Moving_Matrix_separate;
+  //extern int MH_Moving_Matrix_simple, MH_Moving_Matrix_probe, MH_Moving_Matrix_separate;
+  extern int MHMoving_assemblyType ;
 
   FI = EquationTerm_P->Case.LocalTerm.Active ;
 
@@ -69,6 +74,8 @@ void Cal_InitGalerkinTermOfFemEquation(struct EquationTerm     * EquationTerm_P,
     (EquationTerm_P->Case.LocalTerm.Term.TypeOperatorEqu ==
      EquationTerm_P->Case.LocalTerm.Term.TypeOperatorDof) ;
 
+  assDiag_done.clear();
+
   if(EquationTerm_P->Case.LocalTerm.Term.CanonicalWholeQuantity_Equ != CWQ_NONE)
     FI->SymmetricalMatrix = 0 ;
 
@@ -216,34 +223,39 @@ void Cal_InitGalerkinTermOfFemEquation(struct EquationTerm     * EquationTerm_P,
   /*  -------------------------------------------------  */
 
   switch (EquationTerm_P->Case.LocalTerm.Term.TypeTimeDerivative) {
-  case NODT_        : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NoDt   ; break;
-  case DT_          : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_Dt     ; break;
-  case DTDOF_       : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDof  ; break;
-  case DTDT_        : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDt   ; break;
-  case DTDTDOF_     : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDtDof; break;
-  case JACNL_       : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_JacNL  ; break;
-  case DTDOFJACNL_  : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDofJacNL; break;
-  case NEVERDT_     : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NeverDt; break;
-  case DTNL_        : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtNL   ; break;
-  default           :
+  case NODT_          : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NoDt         ; break;
+  case DT_            : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_Dt           ; break;
+  case DTDOF_         : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDof        ; break;
+  case DTDT_          : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDt         ; break;
+  case DTDTDOF_       : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDtDof      ; break;
+  case DTDTDTDOF_     : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDtDtDof    ; break;
+  case DTDTDTDTDOF_   : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDtDtDtDof  ; break;
+  case DTDTDTDTDTDOF_ : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDtDtDtDtDof; break;
+  case JACNL_         : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_JacNL        ; break;
+  case DTDOFJACNL_    : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDofJacNL   ; break;
+  case NEVERDT_       : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NeverDt      ; break;
+  case DTNL_          : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtNL         ; break;
+  default             :
     Message::Error("Unknown type of Operator for Galerkin term (%d)",
                    EquationTerm_P->Case.LocalTerm.Term.TypeTimeDerivative);
     FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NoDt   ; break;
   }
 
 
+  if(MHMoving_assemblyType)
+    FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_MHMoving;
+
+  /*
   if (MH_Moving_Matrix_simple) {
-    /* Message::Info("AssembleTerm_MH_Moving") ; */
     FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_MH_Moving_simple ;
   }
   if (MH_Moving_Matrix_probe) {
-    /* Message::Info("AssembleTerm_MH_Moving") ; */
     FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_MH_Moving_probe ;
   }
   if (MH_Moving_Matrix_separate) {
-    /* Message::Info("AssembleTerm_MH_Moving") ; */
     FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_MH_Moving_separate ;
   }
+*/
 
   /*  initialisation of MHJacNL-term (nonlinear multi-harmonics) if necessary */
   Cal_InitGalerkinTermOfFemEquation_MHJacNL(EquationTerm_P);
@@ -255,6 +267,17 @@ void Cal_InitGalerkinTermOfFemEquation(struct EquationTerm     * EquationTerm_P,
   }
 }
 
+
+/* ------------------------------------------------------------------------ */
+/*  C a l _ E n d G a l e r k i n T e r m O f F e m E q u a t i o n         */
+/* ------------------------------------------------------------------------ */
+
+void Cal_EndGalerkinTermOfFemEquation()
+{
+  assDiag_done.clear();
+}
+
+
 /* ------------------------------------------------------------------------ */
 /*  C a l _ a p p l y M e t r i c T e n s o r                               */
 /* ------------------------------------------------------------------------ */
@@ -366,8 +389,14 @@ void Cal_vBFxDof(struct EquationTerm       * EquationTerm_P,
     w = Current.w ;
   }
 
+  // initialize vBFxDof to zero; this allows to perform e.g. [0, {d u}] without
+  // having to explicitly use [Vector[0,0,0], {d u}] ; if this is too slow, we
+  // should check vBFxDof[j].Type against FI->Type_FormEqu before calling
+  // FI->Cal_Productx to report errors
+  for (j = 0 ; j < Nbr_Dof ; j++)
+    Cal_ZeroValue(&vBFxDof[j]);
 
-  /* shape functions, integral quantity or dummy */
+  // shape functions, integral quantity or dummy
 
   if (!FI->SymmetricalMatrix) {
 
@@ -401,9 +430,6 @@ void Cal_vBFxDof(struct EquationTerm       * EquationTerm_P,
 				       QuantityStorage_P0, QuantityStorageDof_P,
 				       FI->Type_DefineQuantityDof, Nbr_Dof,
 				       (void (**)())xFunctionBFDof, vBFxDof) ;
-
-
-
       FI->Type_ValueDof = FI->Type_FormDof = vBFxDof[0].Type; /* now this type is correct */
       break ;
     case NODOF :  /* Supprimer le DofForNoDof_P de la structure dans Data_Active.h */
@@ -676,6 +702,8 @@ void  Cal_GalerkinTermOfFemEquation(struct Element          * Element,
 
       for (i_IntPoint = 0 ; i_IntPoint < Nbr_IntPoints ; i_IntPoint++) {
 
+        Current.QuadraturePointIndex = i_IntPoint;
+
 	Get_IntPoint(Nbr_IntPoints, i_IntPoint,
 		     &Current.u, &Current.v, &Current.w, &weight) ;
 
@@ -878,12 +906,20 @@ void  Cal_GalerkinTermOfFemEquation(struct Element          * Element,
       for (i = 0 ; i < Nbr_Equ ; i++) {
 	/*      for (j = 0 ; j < Nbr_Dof ; j++)*/
 	j = i;
-	Ek[i][j][0] = -1.;
-	for (k = 1 ; k < Current.NbrHar ; k++)  Ek[i][j][k] = 0. ;
-	((void (*)(struct Dof*, struct Dof*, double*))
-	 FI->Function_AssembleTerm)
-	  (QuantityStorageEqu_P->BasisFunction[i].Dof,
-	   QuantityStorageDof_P->BasisFunction[j].Dof,  Ek[i][j]) ;
+        if (QuantityStorageEqu_P->BasisFunction[i].Dof->Type == DOF_UNKNOWN
+            &&
+            assDiag_done.find
+            (QuantityStorageEqu_P->BasisFunction[i].Dof->Case.Unknown.NumDof-1)
+            == assDiag_done.end()) {
+          assDiag_done
+            [QuantityStorageEqu_P->BasisFunction[i].Dof->Case.Unknown.NumDof-1] = true;
+          Ek[i][j][0] = 1.;
+          for (k = 1 ; k < Current.NbrHar ; k++)  Ek[i][j][k] = 0. ;
+          ((void (*)(struct Dof*, struct Dof*, double*))
+           FI->Function_AssembleTerm)
+            (QuantityStorageEqu_P->BasisFunction[i].Dof,
+             QuantityStorageDof_P->BasisFunction[j].Dof,  Ek[i][j]) ;
+        }
       }
     }
 
@@ -895,4 +931,5 @@ void  Cal_GalerkinTermOfFemEquation(struct Element          * Element,
   }  /* while (1) ... */
 
   Current.flagAssDiag = 0; /*+++prov*/
+
 }
