@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -17,9 +17,12 @@ void  Cal_GalerkinTermOfFemEquation(struct Element          * Element,
 				    struct EquationTerm     * EquationTerm_P,
 				    struct QuantityStorage  * QuantityStorage_P0);
 
+void Cal_EndGalerkinTermOfFemEquation();
+
 /* In F_MultiHar */
-void Cal_InitGalerkinTermOfFemEquation_MHJacNL(struct EquationTerm *);
-void Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element *, struct EquationTerm *,
-					   struct QuantityStorage *);
+void Cal_InitGalerkinTermOfFemEquation_MHJacNL(struct EquationTerm * EquationTerm_P);
+void Cal_GalerkinTermOfFemEquation_MHJacNL(struct Element * Element,
+                                           struct EquationTerm * EquationTerm_P,
+					   struct QuantityStorage *QuantityStorage_P0);
 
 #endif
