@@ -1,10 +1,11 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
 
 #include <string.h>
 #include <math.h>
+#include <map>
 #include "ProData.h"
 #include "GeoData.h"
 #include "DofData.h"
@@ -18,6 +19,8 @@
 #include "MallocUtils.h"
 #include "Message.h"
 
+#include "Pos_Format.h"
+
 extern struct Problem Problem_S ;
 extern struct CurrentData Current ;
 extern int TreatmentStatus ;
@@ -30,7 +33,8 @@ void  Get_ValueOfExpression(struct Expression * Expression_P,
 			    struct QuantityStorage * QuantityStorage_P0,
 			    double u, double v, double w,
 			    struct Value * Value,
-			    int NbrArguments)
+			    int    NbrArguments,
+			    char   *CallingExpressionName)
 {
   int k ;
   struct ExpressionPerRegion  * ExpressionPerRegion_P ;
@@ -55,7 +59,8 @@ void  Get_ValueOfExpression(struct Expression * Expression_P,
     Cal_WholeQuantity(Current.Element, QuantityStorage_P0,
 		      Expression_P->Case.WholeQuantity,
 		      u,v,w, -1, 0, Value, NbrArguments,
-		      Expression_P->Name) ;
+		      CallingExpressionName ?
+		          CallingExpressionName : Expression_P->Name) ;
     break ;
 
   case PIECEWISEFUNCTION :
@@ -79,7 +84,7 @@ void  Get_ValueOfExpression(struct Expression * Expression_P,
     }
     Get_ValueOfExpression
       (Expression_P->Case.PieceWiseFunction.ExpressionForLastRegion,
-       QuantityStorage_P0, u, v, w, Value, NbrArguments) ;
+       QuantityStorage_P0, u, v, w, Value, NbrArguments, Expression_P->Name) ;
     break ;
 
   case UNDEFINED_EXP :
@@ -234,26 +239,15 @@ void Cal_SolidAngle(int Source, struct Element *Element,
 /*  C a l _ W h o l e Q u a n t i t y                                       */
 /* ------------------------------------------------------------------------ */
 
-#define MAX_REGISTER_SIZE   100
-
 #define CAST3V    void(*)(struct Value*, struct Value*, struct Value*)
 #define CAST1V    void(*)(struct Value*)
 #define CASTF2V   void(*)(struct Function*, struct Value*, struct Value*)
 
-/* There can be at max one "Dof{op qty}" per WholeQuantity, but as
-   many {op qty} as you want.
+// There can be at max one "Dof{op qty}" per WholeQuantity, but as
+// many {op qty} as you want.
 
-   Note that Stack[8][MAX_STACK_SIZE] should actually be
-   Stack[NBR_MAX_BASISFUNCTION][MAX_STACK_SIZE]. But this tends to
-   overflow the stack when we don't use USE_GLOBAL_STACK. Also, 8 is
-   OK since the 'multi' feature is only used for SolidAngle
-   computations at the moment.
-
-   A better solution would be to build a single stack (instead of 8
-   stacks), where a Value could be multiple. But this requires to
-   change the way we deal with function arguments. */
-
-static struct Value ValueSaved[MAX_REGISTER_SIZE] ;
+static std::map<int, Value> ValueSaved ;
+static std::map<std::string, Value> NamedValueSaved ;
 
 void Cal_WholeQuantity(struct Element * Element,
 		       struct QuantityStorage * QuantityStorage_P0,
@@ -278,68 +272,10 @@ void Cal_WholeQuantity(struct Element * Element,
 
   struct Element* Save_CurrentElement ;
 
-//#define USE_GLOBAL_STACK
-
-#if defined(USE_GLOBAL_STACK)
-
-  /* Use a single global (static) stack for all quantity evaluations.
-
-     Stack size
-       = MAX_RECURSION * MAX_STACK_SIZE * 8 * sizeof(struct Value)
-       = 50 * 40 * 8 * (MAX_DIM * NBR_MAX_HARMONIC * sizeof(double))
-       = 50 * 40 * 8 * (9 * 2 * 8)
-      ~= 2 Mb
-
-     Beware that for NBR_MAX_HARMONIC=40, the size would grow to
-     40Mb... So let's define MAX_RECURSION as follows : */
-
-#if NBR_MAX_HARMONIC <= 10
-#define MAX_RECURSION 50
-#else
-#define MAX_RECURSION 10
-#endif
-
-  /* We need MAX_RECURSION sufficiently large for expressions like
-     (a?b:(c?d:(e?...))) with all n<MAX_RECURSION first tests
-     evaluating to false. This case happens quite often when
-     specifying piecewise defined physical characteristics.
-
-     If this poses problem in the future, we could still think about
-     reallocating ths stack as it grows. The same holds for
-     MAX_STACK_SIZE, of course. */
-
-  static struct Value ***StaticStack;
-  static int RecursionIndex = -1, first = 1;
-  struct Value **Stack;
-
-#else
-
-  /* Use a local stack: this can quickly overflow the stack on Windows
-     and Mac OS X */
+  // we could make this dynamic (with std::vector) to reduce stack usage, but
+  // the performance hit is important
   struct Value Stack[8][MAX_STACK_SIZE] ;
 
-#endif
-
-#if defined(USE_GLOBAL_STACK)
-  if(first){
-    StaticStack = (struct Value***)Malloc(MAX_RECURSION*sizeof(struct Value**));
-    for(j = 0; j < MAX_RECURSION; j++){
-      StaticStack[j] = (struct Value**)Malloc(8*sizeof(struct Value*));
-      for(k = 0; k < 8; k++){
-	StaticStack[j][k] = (struct Value*)Malloc(MAX_STACK_SIZE*sizeof(struct Value));
-      }
-    }
-    first = 0;
-  }
-  RecursionIndex++;
-  if(RecursionIndex < 0 || RecursionIndex >= MAX_RECURSION){
-    Message::Error("Recursion problem in Cal_WholeQuantity (%d outside [0,%d])",
-                   RecursionIndex, MAX_RECURSION);
-    return;
-  }
-  Stack = StaticStack[RecursionIndex];
-#endif
-
   WholeQuantity_P0 = (struct WholeQuantity*)List_Pointer(WholeQuantity_L, 0) ;
 
   Index     = 0 ;
@@ -672,7 +608,7 @@ void Cal_WholeQuantity(struct Element * Element,
 
     case WQ_ARGUMENT :
       if (WholeQuantity_P->Case.Argument.Index > NbrArguments){
-        Message::Error("Function %s called with too few arguments", ExpressionName);
+        Message::Error("Function %s called with too few arguments.", ExpressionName);
       }
       Cal_CopyValue(DofValue + WholeQuantity_P->Case.Argument.Index - 1,
 		    &Stack[0][Index]) ;
@@ -693,7 +629,8 @@ void Cal_WholeQuantity(struct Element * Element,
                           u, v, w, -1, 0, &Stack[0][Index], NbrArguments, ExpressionName);
 
         for (k = 0 ; k < Current.NbrSystem ; k++)
-          (Current.DofData_P0+k)->Save_CurrentSolution = (Current.DofData_P0+k)->CurrentSolution;
+          (Current.DofData_P0+k)->Save_CurrentSolution =
+            (Current.DofData_P0+k)->CurrentSolution;
         Save_TimeStep = Current.TimeStep ;
         Save_Time = Current.Time ;
         Save_TimeImag = Current.TimeImag ;
@@ -701,7 +638,8 @@ void Cal_WholeQuantity(struct Element * Element,
         for (int n = 0; n < Current.CorrOrder; n++) {
 
           for (k = 0 ; k < Current.NbrSystem ; k++){
-            Solution_P0 = (struct Solution*)List_Pointer((Current.DofData_P0+k)->Solutions, 0);
+            Solution_P0 = (struct Solution*)List_Pointer
+              ((Current.DofData_P0+k)->Solutions, 0);
             if(((Current.DofData_P0+k)->CurrentSolution - Solution_P0) > 0)
               ((Current.DofData_P0+k)->CurrentSolution) -= 1 ;
             else{
@@ -728,7 +666,8 @@ void Cal_WholeQuantity(struct Element * Element,
 
         }
 
-        Cal_MultValue(&Stack[0][Index], 1./(Current.bCorrCoeff*Current.DTime), &Stack[0][Index]);
+        Cal_MultValue(&Stack[0][Index], 1./(Current.bCorrCoeff*Current.DTime),
+                      &Stack[0][Index]);
 
         for (k = 0 ; k < Current.NbrSystem ; k++)
           (Current.DofData_P0+k)->CurrentSolution =
@@ -736,8 +675,6 @@ void Cal_WholeQuantity(struct Element * Element,
         Current.TimeStep = Save_TimeStep ;
         Current.Time = Save_Time ;
         Current.TimeImag = Save_TimeImag ;
-
-
       }
       else if (Current.NbrHar == 1) {
         for (j=0; j<NbrArguments; j++) {
@@ -755,22 +692,27 @@ void Cal_WholeQuantity(struct Element * Element,
 	   (Current.DofData_P0+k)->CurrentSolution;
 
 	  if(List_Nbr((Current.DofData_P0+k)->Solutions) > 1){
-	    Solution_P0 = (struct Solution*)List_Pointer((Current.DofData_P0+k)->Solutions, 0);
+	    Solution_P0 = (struct Solution*)List_Pointer
+              ((Current.DofData_P0+k)->Solutions, 0);
 	    if ((Current.DofData_P0+k)->CurrentSolution != Solution_P0)
 	      ((Current.DofData_P0+k)->CurrentSolution) -- ;
 	  }
 	  else {
 	    if (!Flag_WarningMissSolForDt) {
-	      Message::Warning("Missing solution for time derivative computation (Sys#%d/%d)",
-                               k, Current.NbrSystem);
+	      Message::Warning("Missing solution for time derivative computation "
+                               "(Sys#%d/%d)", k+1, Current.NbrSystem);
 	      Flag_WarningMissSolForDt = 1 ;
 	    }
 	  }
 	}
 
-	Save_Time = Current.Time ;
-	Current.Time = Current.DofData->CurrentSolution->Time ;
+        Save_TimeStep = Current.TimeStep ;
+        Save_Time = Current.Time ;
+        Save_TimeImag = Current.TimeImag ;
 
+        Current.TimeStep = Current.DofData->CurrentSolution->TimeStep ;
+        Current.Time = Current.DofData->CurrentSolution->Time ;
+        Current.TimeImag = Current.DofData->CurrentSolution->TimeImag ;
 	for (j=0; j<NbrArguments; j++) {
           Cal_CopyValue(DofValue + j, &Stack[0][Index+1]) ;
           Multi[Index+1] = 0 ;
@@ -791,8 +733,9 @@ void Cal_WholeQuantity(struct Element * Element,
 	for (k = 0 ; k < Current.NbrSystem ; k++)
 	  (Current.DofData_P0+k)->CurrentSolution =
 	    (Current.DofData_P0+k)->Save_CurrentSolution;
-
-	Current.Time = Save_Time ;
+        Current.TimeStep = Save_TimeStep ;
+        Current.Time = Save_Time ;
+        Current.TimeImag = Save_TimeImag ;
       }
       else {
         for (j=0; j<NbrArguments; j++) {
@@ -834,8 +777,8 @@ void Cal_WholeQuantity(struct Element * Element,
 	}
 	else {
 	  if (!Flag_WarningMissSolForTime_ntime) {
-	    Message::Warning("Missing solution for time step -%d computation (System #%d/%d)",
-                             ntime, k+1, Current.NbrSystem);
+	    Message::Warning("Missing solution for time step -%d computation "
+                             "(System #%d/%d)", ntime, k+1, Current.NbrSystem);
 	    Flag_WarningMissSolForTime_ntime = 1 ;
 	  }
 	}
@@ -874,9 +817,210 @@ void Cal_WholeQuantity(struct Element * Element,
       Index++ ;
       break ;
 
+
+    case WQ_MAXOVERTIME :
+      if (Current.NbrHar == 1) {
+        double time_init = WholeQuantity_P->Case.MaxOverTime.TimeInit;
+        double time_final = WholeQuantity_P->Case.MaxOverTime.TimeFinal;
+        /*
+	for (k = 0 ; k < Current.NbrSystem ; k++)
+	  (Current.DofData_P0+k)->Save_CurrentSolution =
+	   (Current.DofData_P0+k)->CurrentSolution;
+        */
+        Save_TimeStep = Current.TimeStep ;
+        Save_Time = Current.Time ;
+        Save_TimeImag = Current.TimeImag ;
+
+        for (j=0; j<NbrArguments; j++) {
+          Cal_CopyValue(DofValue + j, &Stack[0][Index]) ;
+          Multi[Index] = 0 ;
+          Index++ ;
+        }
+        Index -= NbrArguments ;
+
+        double val_maxInTime = -1.e99;
+
+        for (j=1; j<List_Nbr((Current.DofData)->Solutions); j++) {
+
+         Current.DofData->CurrentSolution =
+            (struct Solution*)List_Pointer((Current.DofData)->Solutions, j);
+
+          //++++ Add: also for other systems!
+
+          Current.TimeStep = Current.DofData->CurrentSolution->TimeStep ;
+          Current.Time = Current.DofData->CurrentSolution->Time ;
+          Current.TimeImag = Current.DofData->CurrentSolution->TimeImag ;
+
+          //++++ test to do more accurately!
+          if (Current.Time >= time_init && Current.Time <= time_final) {
+            Cal_WholeQuantity(Element, QuantityStorage_P0,
+                              WholeQuantity_P->Case.MaxOverTime.WholeQuantity,
+                              u, v, w, -1, 0, &Stack[0][Index], NbrArguments, ExpressionName) ;
+
+            if (Stack[0][Index].Type == SCALAR) {
+              if (Stack[0][Index].Val[0] > val_maxInTime){
+                val_maxInTime = Stack[0][Index].Val[0];
+              }
+            }
+            else {
+              Message::Error("MaxOverTime can only be applied on scalar values") ;
+            }
+          }
+        }
+        Stack[0][Index].Val[0] = val_maxInTime;
+        /*
+	for (k = 0 ; k < Current.NbrSystem ; k++)
+	  (Current.DofData_P0+k)->CurrentSolution =
+	    (Current.DofData_P0+k)->Save_CurrentSolution;
+        */
+        Current.TimeStep = Save_TimeStep ;
+        Current.Time = Save_Time ;
+        Current.TimeImag = Save_TimeImag ;
+
+        Multi[Index] = 0 ;
+        Index++ ;
+      }
+      else {
+        Message::Error("MaxOverTime can only be used in time domain") ;
+        break;
+      }
+      break ;
+
+    case WQ_FOURIERSTEINMETZ :
+      if (Current.NbrHar == 1) {
+        double time_init = WholeQuantity_P->Case.FourierSteinmetz.TimeInit;
+        double time_final = WholeQuantity_P->Case.FourierSteinmetz.TimeFinal;
+        int nbrFrequencyInFormula = WholeQuantity_P->Case.FourierSteinmetz.NbrFrequency;
+        double exponent_f = WholeQuantity_P->Case.FourierSteinmetz.Exponent_f;
+        double exponent_b = WholeQuantity_P->Case.FourierSteinmetz.Exponent_b;
+
+        /*
+	for (k = 0 ; k < Current.NbrSystem ; k++)
+	  (Current.DofData_P0+k)->Save_CurrentSolution =
+	   (Current.DofData_P0+k)->CurrentSolution;
+        */
+        Save_TimeStep = Current.TimeStep ;
+        Save_Time = Current.Time ;
+        Save_TimeImag = Current.TimeImag ;
+
+        for (j=0; j<NbrArguments; j++) {
+          Cal_CopyValue(DofValue + j, &Stack[0][Index]) ;
+          Multi[Index] = 0 ;
+          Index++ ;
+        }
+        Index -= NbrArguments ;
+
+        int i_Solution_init = -1, i_Solution_final = -1;
+        int NbrTimeStep, Size=-1;
+
+        for (j=0; j<List_Nbr((Current.DofData)->Solutions); j++) {
+          Current.DofData->CurrentSolution =
+            (struct Solution*)List_Pointer((Current.DofData)->Solutions, j);
+          Current.Time = Current.DofData->CurrentSolution->Time ;
+          if (Current.Time >= time_init && i_Solution_init < 0)
+            i_Solution_init = j;
+          if (Current.Time <= time_final) {
+            i_Solution_final = j;
+          }
+          if (Current.Time > time_final) {
+            break;
+          }
+        }
+        NbrTimeStep = i_Solution_final-i_Solution_init+1;
+        if (NbrTimeStep < 2)
+          Message::Error("Wrong time interval in Function FourierSteinmetz (%d,%d)",
+                         i_Solution_init, i_Solution_final) ;
+
+        double *Times = (double *)Malloc(NbrTimeStep*sizeof(double));
+        struct Value *TmpValues = (struct Value *)Malloc(NbrTimeStep*sizeof(struct Value));
+
+        for (j=0; j<NbrTimeStep; j++) {
+          Current.DofData->CurrentSolution =
+            (struct Solution*)List_Pointer((Current.DofData)->Solutions,
+                                           i_Solution_init+j);
+
+          //++++ Add: also for other systems!
+
+          Current.TimeStep = Current.DofData->CurrentSolution->TimeStep ;
+          Current.Time = Current.DofData->CurrentSolution->Time ;
+          Current.TimeImag = Current.DofData->CurrentSolution->TimeImag ;
+
+          Cal_WholeQuantity(Element, QuantityStorage_P0,
+                            WholeQuantity_P->Case.MaxOverTime.WholeQuantity,
+                            u, v, w, -1, 0, &Stack[0][Index], NbrArguments, ExpressionName) ;
+          Times[j] = Current.Time ;
+          Cal_CopyValue(&Stack[0][Index], &TmpValues[j]) ;
+          if (Stack[0][Index].Type == SCALAR)
+            Size = 1;
+          else if (Stack[0][Index].Type == VECTOR)
+            Size = 3;
+          else
+            Message::Error("FourierSteinmetz can only be applied on scalar or vector values") ;
+        }
+
+        // FourierTransform
+        int NbrFreq ;
+        double *Frequencies;
+        struct Value *FourierValues;
+
+        Pos_FourierTransform(NbrTimeStep, 1, Times, TmpValues, Size,
+                             2, &NbrFreq, &Frequencies, &FourierValues);
+
+        /*
+          we calculate the Sum for all frequencies of frequency_i^exponent_f *
+          b_i^exponent_b
+        */
+
+        if (nbrFrequencyInFormula > NbrFreq-1)
+          Message::Error("FourierSteinmetz: too many frequencies asked "
+                         "(%d asked and only %d available)",
+                         nbrFrequencyInFormula, NbrFreq-1) ;
+
+        double val=0.;
+        for (j=0; j<nbrFrequencyInFormula; j++) {
+          if (Size==1) {
+            val += pow(Frequencies[j+1], exponent_f) *
+              pow(FourierValues[j+1].Val[0], exponent_b);
+          }
+          else {
+            val +=
+              pow(Frequencies[j+1], exponent_f) *
+              pow(sqrt(FourierValues[j+1].Val[0]*FourierValues[j+1].Val[0]
+                       + FourierValues[j+1].Val[1]*FourierValues[j+1].Val[1]
+                       + FourierValues[j+1].Val[2]*FourierValues[j+1].Val[2]),
+                  exponent_b);
+          }
+        }
+        Stack[0][Index].Type = SCALAR;
+        Stack[0][Index].Val[0] = val;
+
+        Free(Frequencies);
+        Free(FourierValues);
+
+        Free(Times); Free(TmpValues) ;
+
+        /*
+	for (k = 0 ; k < Current.NbrSystem ; k++)
+	  (Current.DofData_P0+k)->CurrentSolution =
+	    (Current.DofData_P0+k)->Save_CurrentSolution;
+        */
+        Current.TimeStep = Save_TimeStep ;
+        Current.Time = Save_Time ;
+        Current.TimeImag = Save_TimeImag ;
+
+        Multi[Index] = 0 ;
+        Index++ ;
+      }
+      else {
+        Message::Error("FourierSteinmetz can only be used in time domain") ;
+        break;
+      }
+      break ;
+
     case WQ_MHTRANSFORM :
       if(Current.NbrHar == 1){
-	Message::Error("MHTransform can only be used in complex (multi-harmonic) calculations") ;
+	Message::Error("MHTransform can only be used in complex (multi-harmonic)"
+                       " calculations") ;
         break;
       }
 
@@ -885,6 +1029,7 @@ void Cal_WholeQuantity(struct Element * Element,
         Multi[Index] = 0 ;
         Index++ ;
       }
+
       Index -= NbrArguments ;
       Cal_WholeQuantity(Element, QuantityStorage_P0,
 			WholeQuantity_P->Case.MHTransform.WholeQuantity,
@@ -956,22 +1101,41 @@ void Cal_WholeQuantity(struct Element * Element,
       break ;
 
     case WQ_SAVEVALUE :
-      if(WholeQuantity_P->Case.SaveValue.Index > MAX_REGISTER_SIZE-1){
-	Message::Error("Register Size Exceeded (%d)", MAX_REGISTER_SIZE);
-        break;
-      }
-      /* if (WholeQuantity_P->Case.SaveValue.Index >= 0) */
       Cal_CopyValue(&Stack[0][Index-1],
-		    ValueSaved + WholeQuantity_P->Case.SaveValue.Index) ;
+		    &ValueSaved[WholeQuantity_P->Case.SaveValue.Index]) ;
       break ;
 
     case WQ_VALUESAVED :
-      if(WholeQuantity_P->Case.ValueSaved.Index > MAX_REGISTER_SIZE-1){
-	Message::Error("Register size exceeded (%d)", MAX_REGISTER_SIZE);
-        break;
+      if(ValueSaved.count(WholeQuantity_P->Case.ValueSaved.Index))
+        Cal_CopyValue(&ValueSaved[WholeQuantity_P->Case.ValueSaved.Index],
+                      &Stack[0][Index]) ;
+      else{
+        if(TreatmentStatus != _PRE)
+          Message::Warning("Empty register %d: assuming zero value",
+                           WholeQuantity_P->Case.ValueSaved.Index);
+        Cal_ZeroValue(&Stack[0][Index]);
+        Stack[0][Index].Type = SCALAR ;
+      }
+      Multi[Index] = 0 ;
+      Index++ ;
+      break ;
+
+    case WQ_SAVENAMEDVALUE :
+      Cal_CopyValue(&Stack[0][Index-1],
+		    &NamedValueSaved[WholeQuantity_P->Case.NamedValue.Name]) ;
+      break ;
+
+    case WQ_NAMEDVALUESAVED :
+      if(NamedValueSaved.count(WholeQuantity_P->Case.NamedValue.Name))
+        Cal_CopyValue(&NamedValueSaved[WholeQuantity_P->Case.NamedValue.Name],
+                      &Stack[0][Index]) ;
+      else{
+        if(TreatmentStatus != _PRE)
+          Message::Warning("Unknown current value '$%s': assuming zero value",
+                           WholeQuantity_P->Case.NamedValue.Name);
+        Cal_ZeroValue(&Stack[0][Index]);
+        Stack[0][Index].Type = SCALAR ;
       }
-      Cal_CopyValue(ValueSaved + WholeQuantity_P->Case.ValueSaved.Index,
-		    &Stack[0][Index]) ;
       Multi[Index] = 0 ;
       Index++ ;
       break ;
@@ -996,23 +1160,6 @@ void Cal_WholeQuantity(struct Element * Element,
   }
 
   if (DofIndexInWholeQuantity < 0) Cal_CopyValue(&Stack[0][0], &DofValue[0]) ;
-
-#if defined(USE_GLOBAL_STACK)
-  RecursionIndex--;
-#endif
-}
-
-/* ------------------------------------------------------------------------ */
-/*  P u r i f y _ W h o l e Q u a n t i t y                                 */
-/* ------------------------------------------------------------------------ */
-
-List_T * Purify_WholeQuantity(List_T * WQ_L)
-{
-  /* It would be nice to pre-compute all trivial sequences in a list
-     of WholeQuantities. For example, when all the quantties are
-     constants, it is pretty stupid to recompute everything
-     everytime using the stack... */
-  return NULL ;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1021,5 +1168,14 @@ List_T * Purify_WholeQuantity(List_T * WQ_L)
 
 void Cal_StoreInRegister(struct  Value  *Value, int RegisterIndex)
 {
-  Cal_CopyValue(Value, ValueSaved + RegisterIndex) ;
+  Cal_CopyValue(Value, &ValueSaved[RegisterIndex]) ;
+}
+
+/* ------------------------------------------------------------------------ */
+/*  C a l _ S t o r e I n V a r i a b l e                                   */
+/* ------------------------------------------------------------------------ */
+
+void Cal_StoreInVariable(struct  Value  *Value, const char *name)
+{
+  Cal_CopyValue(Value, &NamedValueSaved[name]) ;
 }
