@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -227,14 +227,15 @@ void Cal_AddValueArray2DoubleArray(struct Value *V1, double *R, int Nbr_Values)
    R <- 0
    ------------------------------------------------------------------------ */
 
-static double VALUE_ZERO [NBR_MAX_HARMONIC * MAX_DIM] =
-{0.,0.,0., 0.,0.,0.,
- 0.,0.,0., 0.,0.,0.,
- 0.,0.,0., 0.,0.,0.} ;
+//static double VALUE_ZERO [NBR_MAX_HARMONIC * MAX_DIM] =
+// {0.,0.,0., 0.,0.,0.,
+//  0.,0.,0., 0.,0.,0.,
+//  0.,0.,0., 0.,0.,0.} ;
 
 void Cal_ZeroValue(struct Value * R)
 {
-  memcpy(R->Val, VALUE_ZERO, Current.NbrHar*MAX_DIM*sizeof(double));
+  //memcpy(R->Val, VALUE_ZERO, Current.NbrHar*MAX_DIM*sizeof(double));
+  for(int i = 0; i < Current.NbrHar*MAX_DIM; i++) R->Val[i] = 0.;
 }
 
 /* ------------------------------------------------------------------------
@@ -1116,7 +1117,7 @@ void Cal_ProductValue(struct Value * V1, struct Value * V2, struct Value * R)
   else if (V1->Type == TENSOR_SYM && V2->Type == TENSOR_SYM) {
     if (Current.NbrHar == 1) {
       a1[0] = V1->Val[0]*V2->Val[0] + V1->Val[1]*V2->Val[1] + V1->Val[2]*V2->Val[2];
-      a1[1] = V1->Val[0]*V2->Val[1] + V1->Val[1]*V2->Val[2] + V1->Val[2]*V2->Val[4];
+      a1[1] = V1->Val[0]*V2->Val[1] + V1->Val[1]*V2->Val[3] + V1->Val[2]*V2->Val[4];
       a1[2] = V1->Val[0]*V2->Val[2] + V1->Val[1]*V2->Val[4] + V1->Val[2]*V2->Val[5];
       a1[3] = V1->Val[1]*V2->Val[0] + V1->Val[3]*V2->Val[1] + V1->Val[4]*V2->Val[2];
       a1[4] = V1->Val[1]*V2->Val[1] + V1->Val[3]*V2->Val[3] + V1->Val[4]*V2->Val[4];
@@ -1131,7 +1132,7 @@ void Cal_ProductValue(struct Value * V1, struct Value * V2, struct Value * R)
     else {
       for (k = 0 ; k < Current.NbrHar ; k += 2) {
 	CMULT(0,0,0);  CMULT(1,1,1);  CMULT(2,2,2);
-	CMULT(0,1,3);  CMULT(1,2,4);  CMULT(2,4,5);
+	CMULT(0,1,3);  CMULT(1,3,4);  CMULT(2,4,5);
 	CMULT(0,2,6);  CMULT(1,4,7);  CMULT(2,5,8);
 	CMULT(1,0,9);  CMULT(3,1,10); CMULT(4,2,11);
 	CMULT(1,1,12); CMULT(3,3,13); CMULT(4,4,14);
@@ -1146,7 +1147,6 @@ void Cal_ProductValue(struct Value * V1, struct Value * V2, struct Value * R)
     }
     R->Type = TENSOR;
   }
-
   else if (V1->Type == TENSOR && V2->Type == TENSOR) {
     if (Current.NbrHar == 1) {
       a1[0] = V1->Val[0]*V2->Val[0] + V1->Val[1]*V2->Val[3] + V1->Val[2]*V2->Val[6];
@@ -1180,6 +1180,72 @@ void Cal_ProductValue(struct Value * V1, struct Value * V2, struct Value * R)
     }
     R->Type = TENSOR;
   }
+  else if (V1->Type == TENSOR_SYM && V2->Type == TENSOR) {
+    if (Current.NbrHar == 1) {
+      a1[0] = V1->Val[0]*V2->Val[0] + V1->Val[1]*V2->Val[3] + V1->Val[2]*V2->Val[6];
+      a1[1] = V1->Val[0]*V2->Val[1] + V1->Val[1]*V2->Val[4] + V1->Val[2]*V2->Val[7];
+      a1[2] = V1->Val[0]*V2->Val[2] + V1->Val[1]*V2->Val[5] + V1->Val[2]*V2->Val[8];
+      a1[3] = V1->Val[1]*V2->Val[0] + V1->Val[3]*V2->Val[3] + V1->Val[4]*V2->Val[6];
+      a1[4] = V1->Val[1]*V2->Val[1] + V1->Val[3]*V2->Val[4] + V1->Val[4]*V2->Val[7];
+      a1[5] = V1->Val[1]*V2->Val[2] + V1->Val[3]*V2->Val[5] + V1->Val[4]*V2->Val[8];
+      a1[6] = V1->Val[2]*V2->Val[0] + V1->Val[4]*V2->Val[3] + V1->Val[5]*V2->Val[6];
+      a1[7] = V1->Val[2]*V2->Val[1] + V1->Val[4]*V2->Val[4] + V1->Val[5]*V2->Val[7];
+      a1[8] = V1->Val[2]*V2->Val[2] + V1->Val[4]*V2->Val[5] + V1->Val[5]*V2->Val[8];
+      R->Val[0] = a1[0];  R->Val[1] = a1[1];  R->Val[2] = a1[2];
+      R->Val[3] = a1[3];  R->Val[4] = a1[4];  R->Val[5] = a1[5];
+      R->Val[6] = a1[6];  R->Val[7] = a1[7];  R->Val[8] = a1[8];
+    }
+    else {
+      for (k = 0 ; k < Current.NbrHar ; k += 2) {
+        CMULT(0,0,0);  CMULT(1,3,1);  CMULT(2,6,2);
+        CMULT(0,1,3);  CMULT(1,4,4);  CMULT(2,7,5);
+        CMULT(0,2,6);  CMULT(1,5,7);  CMULT(2,8,8);
+        CMULT(1,0,9);  CMULT(2,3,10); CMULT(4,6,11);
+        CMULT(1,1,12); CMULT(2,4,13); CMULT(4,7,14);
+        CMULT(1,2,15); CMULT(2,5,16); CMULT(4,8,17);
+        CMULT(3,0,18); CMULT(4,3,19); CMULT(5,6,20);
+        CMULT(3,1,21); CMULT(4,4,22); CMULT(5,7,23);
+        CMULT(3,2,24); CMULT(4,5,25); CMULT(5,8,26);
+        CPUT3(0,1,2,0);    CPUT3(3,4,5,1);    CPUT3(6,7,8,2);
+        CPUT3(9,10,11,3);  CPUT3(12,13,14,4); CPUT3(15,16,17,5);
+        CPUT3(18,19,20,6); CPUT3(21,22,23,7); CPUT3(24,25,26,8);
+        }
+    }
+    R->Type = TENSOR;
+  }
+  else if (V1->Type == TENSOR && V2->Type == TENSOR_SYM) {
+    if (Current.NbrHar == 1) {
+      a1[0] = V1->Val[0]*V2->Val[0] + V1->Val[1]*V2->Val[1] + V1->Val[2]*V2->Val[2];
+      a1[1] = V1->Val[0]*V2->Val[1] + V1->Val[1]*V2->Val[3] + V1->Val[2]*V2->Val[4];
+      a1[2] = V1->Val[0]*V2->Val[2] + V1->Val[1]*V2->Val[4] + V1->Val[2]*V2->Val[5];
+      a1[3] = V1->Val[3]*V2->Val[0] + V1->Val[4]*V2->Val[1] + V1->Val[5]*V2->Val[2];
+      a1[4] = V1->Val[3]*V2->Val[1] + V1->Val[4]*V2->Val[3] + V1->Val[5]*V2->Val[4];
+      a1[5] = V1->Val[3]*V2->Val[2] + V1->Val[4]*V2->Val[4] + V1->Val[5]*V2->Val[5];
+      a1[6] = V1->Val[6]*V2->Val[0] + V1->Val[7]*V2->Val[1] + V1->Val[8]*V2->Val[2];
+      a1[7] = V1->Val[6]*V2->Val[1] + V1->Val[7]*V2->Val[3] + V1->Val[8]*V2->Val[4];
+      a1[8] = V1->Val[6]*V2->Val[2] + V1->Val[7]*V2->Val[4] + V1->Val[8]*V2->Val[5];
+      R->Val[0] = a1[0];  R->Val[1] = a1[1];  R->Val[2] = a1[2];
+      R->Val[3] = a1[3];  R->Val[4] = a1[4];  R->Val[5] = a1[5];
+      R->Val[6] = a1[6];  R->Val[7] = a1[7];  R->Val[8] = a1[8];
+    }
+    else {
+      for (k = 0 ; k < Current.NbrHar ; k += 2) {
+        CMULT(0,0,0);  CMULT(1,1,1);  CMULT(2,2,2);
+        CMULT(0,1,3);  CMULT(1,3,4);  CMULT(2,4,5);
+        CMULT(0,2,6);  CMULT(1,4,7);  CMULT(2,5,8);
+        CMULT(3,0,9);  CMULT(4,1,10); CMULT(5,2,11);
+        CMULT(3,1,12); CMULT(4,3,13); CMULT(5,4,14);
+        CMULT(3,2,15); CMULT(4,4,16); CMULT(5,5,17);
+        CMULT(6,0,18); CMULT(7,1,19); CMULT(8,2,20);
+        CMULT(6,1,21); CMULT(7,3,22); CMULT(8,4,23);
+        CMULT(6,2,24); CMULT(7,4,25); CMULT(8,5,26);
+        CPUT3(0,1,2,0);    CPUT3(3,4,5,1);    CPUT3(6,7,8,2);
+        CPUT3(9,10,11,3);  CPUT3(12,13,14,4); CPUT3(15,16,17,5);
+        CPUT3(18,19,20,6); CPUT3(21,22,23,7); CPUT3(24,25,26,8);
+     }
+    }
+    R->Type = TENSOR;
+  }
 
   /* a faire: differents tenseurs entre eux */
 
@@ -2468,8 +2534,8 @@ std::string Print_Value_ToString(struct Value *A)
 
   switch(A->Type){
 
-  case SCALAR ://***Ruth: Provisional: incompatibility with gnuplot
-    //if(Current.NbrHar>1) sstream << "(";
+  case SCALAR :
+    if(Current.NbrHar>1) sstream << "(";
     for (k = 0 ; k < Current.NbrHar ; k++) {
       if(k) sstream << ",";
       sstream << A->Val[MAX_DIM*k];
@@ -2477,18 +2543,18 @@ std::string Print_Value_ToString(struct Value *A)
     if(Current.NbrHar>1) sstream << ")";
     break;
 
-  case VECTOR ://***Ruth
-    //sstream << "[";
+  case VECTOR :
+    sstream << "[";
     for (i = 0 ; i < 3 ; i++) {
       if(i) sstream << " ";
-      //if(Current.NbrHar>1) sstream << "(";
+      if(Current.NbrHar>1) sstream << "(";
       for (k = 0 ; k < Current.NbrHar ; k++) {
 	if(k) sstream << ",";
 	sstream << A->Val[MAX_DIM*k+i];
       }
-      //if(Current.NbrHar>1) sstream << ")";
+      if(Current.NbrHar>1) sstream << ")";
     }
-    //sstream << "]";
+    sstream << "]";
     break;
 
   case TENSOR_DIAG :
@@ -2527,9 +2593,13 @@ std::string Print_Value_ToString(struct Value *A)
   return ret;
 }
 
-void Print_Value(struct Value *A)
+void Print_Value(struct Value *A, FILE *fp)
 {
-  Message::Direct("%s", Print_Value_ToString(A).c_str());
+  std::string s = Print_Value_ToString(A);
+  if(fp && fp != stdout && fp != stderr)
+    fprintf(fp, "%s\n", s.c_str());
+  else
+    Message::Direct("%s", s.c_str());
 }
 
 /* ------------------------------------------------------------------------
