@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -74,6 +74,9 @@ void Dof_InitDofData(struct DofData * DofData_P, int Num,
   DofData_P->Flag_Init[1] = 0 ;
   DofData_P->Flag_Init[2] = 0 ;
   DofData_P->Flag_Init[3] = 0 ;
+  DofData_P->Flag_Init[4] = 0 ;
+  DofData_P->Flag_Init[5] = 0 ;
+  DofData_P->Flag_Init[6] = 0 ;
 
   DofData_P->Flag_Only = 0 ;
   DofData_P->Flag_InitOnly[0] = 0 ;
@@ -124,12 +127,12 @@ void Dof_FreeDofData(struct DofData * DofData_P)
   if(DofData_P->Flag_Init[0] == 1 || DofData_P->Flag_Init[0] == 2){
     LinAlg_DestroyMatrix(&DofData_P->A);
     LinAlg_DestroyVector(&DofData_P->b);
+    LinAlg_DestroyVector(&DofData_P->res);
     LinAlg_DestroySolver(&DofData_P->Solver);
   }
 
   if(DofData_P->Flag_Init[0] == 2){
     LinAlg_DestroyMatrix(&DofData_P->Jac);
-    LinAlg_DestroyVector(&DofData_P->res);
     LinAlg_DestroyVector(&DofData_P->dx);
   }
 
@@ -157,6 +160,30 @@ void Dof_FreeDofData(struct DofData * DofData_P)
     List_Delete(DofData_P->m3s);
   }
 
+  if(DofData_P->Flag_Init[4] == 1){
+    LinAlg_DestroyMatrix(&DofData_P->M4);
+    LinAlg_DestroyVector(&DofData_P->m4);
+    for(int i = 0; i < List_Nbr(DofData_P->m4s); i++)
+       LinAlg_DestroyVector((gVector*)List_Pointer(DofData_P->m4s, i));
+    List_Delete(DofData_P->m4s);
+  }
+
+  if(DofData_P->Flag_Init[5] == 1){
+    LinAlg_DestroyMatrix(&DofData_P->M5);
+    LinAlg_DestroyVector(&DofData_P->m5);
+    for(int i = 0; i < List_Nbr(DofData_P->m5s); i++)
+       LinAlg_DestroyVector((gVector*)List_Pointer(DofData_P->m5s, i));
+    List_Delete(DofData_P->m5s);
+  }
+
+  if(DofData_P->Flag_Init[6] == 1){
+    LinAlg_DestroyMatrix(&DofData_P->M6);
+    LinAlg_DestroyVector(&DofData_P->m6);
+    for(int i = 0; i < List_Nbr(DofData_P->m6s); i++)
+       LinAlg_DestroyVector((gVector*)List_Pointer(DofData_P->m6s, i));
+    List_Delete(DofData_P->m6s);
+  }
+
   if(DofData_P->Flag_Only){
     if(DofData_P->Flag_InitOnly[0] == 1){
       LinAlg_DestroyMatrix(&DofData_P->A1);
@@ -525,6 +552,11 @@ void Dof_WriteFileRES_ExtendMH(char * Name_File, struct DofData * DofData_P,
 {
   if(Message::GetIsCommWorld() && Message::GetCommRank()) return;
 
+  if(!DofData_P->CurrentSolution){
+    Message::Warning("No solution to save");
+    return;
+  }
+
   gVector x;
   double d;
   int i, inew;
@@ -560,6 +592,11 @@ void Dof_WriteFileRES_MHtoTime(char * Name_File, struct DofData * DofData_P,
 {
   if(Message::GetIsCommWorld() && Message::GetCommRank()) return;
 
+  if(!DofData_P->CurrentSolution){
+    Message::Warning("No solution to save");
+    return;
+  }
+
   gVector x;
   double Time, d1, d2, d, *Pulsation;
   int iT, i, j, k;
@@ -613,6 +650,11 @@ void Dof_WriteFileRES(char * Name_File, struct DofData * DofData_P, int Format,
 {
   if(Message::GetIsCommWorld() && Message::GetCommRank()) return;
 
+  if(!DofData_P->CurrentSolution){
+    Message::Warning("No solution to save");
+    return;
+  }
+
   Dof_OpenFile(DOF_RES, Name_File, (char*)(Format ? "ab" : "a")) ;
 
   fprintf(File_RES, "$Solution  /* DofData #%d */\n", DofData_P->Num) ;
@@ -638,6 +680,11 @@ void Dof_WriteFileRES_WithEntityNum(char * Name_File, struct DofData * DofData_P
 {
   if(Message::GetIsCommWorld() && Message::GetCommRank()) return;
 
+  if(!DofData_P->CurrentSolution){
+    Message::Warning("No solution to save");
+    return;
+  }
+
   char    FileCplx[256] ;
   char    FileRe[256] ;
   char    FileIm[256] ;
@@ -1105,7 +1152,7 @@ void Dof_DefineUnknownDof(int D1, int D2, int NbrHar, bool NonLocal)
   }
 }
 
-void NumberUnknownDof (void *a, void *b)
+static void NumberUnknownDof (void *a, void *b)
 {
   struct Dof * Dof_P ;
 
@@ -1189,23 +1236,6 @@ void Dof_DefineAssociateDof(int E1, int E2, int D1, int D2, int NbrHar,
   }
 }
 
-void Dof_GetDof_Four(struct DofData * DofData_P, struct DofData * DofData2_P)
-{
-  int NbrHar2, NbrDof2, i ;
-  struct Dof * Dof_P ;
-
-  NbrHar2 = DofData2_P->NbrHar ;
-  NbrDof2 = List_Nbr(DofData2_P->DofList) ;
-
-  for (i=0 ; i<NbrDof2 ; i+=NbrHar2) {
-
-    Dof_P = (struct Dof *) List_Pointer(DofData2_P->DofList, i) ;
-
-    printf("i %d, dof %d \n", i/NbrHar2, List_ISearch(DofData_P->DofList, Dof_P, fcmp_Dof)) ;
-
-  }
-}
-
 /* ------------------------------------------------------------------------ */
 /*  P R O C E S S I N G   ( C o d e s   i n   L i s t )                     */
 /* ------------------------------------------------------------------------ */
@@ -1227,12 +1257,20 @@ struct Dof *Dof_GetDofStruct(struct DofData * DofData_P, int D1, int D2, int D3)
 /*  D o f _ U p d a t e A s s i g n F i x e d D o f                         */
 /* ------------------------------------------------------------------------ */
 
-void Dof_UpdateAssignFixedDof(struct Dof *Dof_P, int NbrHar, double *Val)
+void Dof_UpdateAssignFixedDof(int D1, int D2, int NbrHar, double *Val)
 {
-  int k ;
+  struct Dof  Dof, * Dof_P ;
+  int         k ;
+
+  Dof.NumType = D1 ;  Dof.Entity = D2 ;
 
   for(k=0 ; k<NbrHar ; k+=gSCALAR_SIZE){
-    LinAlg_SetScalar(&Dof_P->Val, &Val[k]) ;
+    Dof.Harmonic = k ;
+    if(CurrentDofData->DofTree)
+      Dof_P = (struct Dof *)Tree_PQuery(CurrentDofData->DofTree, &Dof);
+    else
+      Dof_P = (struct Dof *)List_PQuery(CurrentDofData->DofList, &Dof, fcmp_Dof);
+    LinAlg_SetScalar(&Dof_P->Val, &Val[Dof_P->Harmonic]) ;
   }
 }
 
@@ -1240,24 +1278,30 @@ void Dof_UpdateAssignFixedDof(struct Dof *Dof_P, int NbrHar, double *Val)
 /*  D o f _ U p d a t e L i n k D o f                                       */
 /* ------------------------------------------------------------------------ */
 
-void Dof_UpdateLinkDof(struct Dof *Dof_P, int NbrHar, double Value[], int D2_Link)
+void Dof_UpdateLinkDof(int D1, int D2, int NbrHar, double Value[], int D2_Link)
 {
+  struct Dof  Dof, * Dof_P ;
   int         k ;
 
-  if (Dof_P->Type == DOF_LINK || Dof_P->Type == DOF_LINKCPLX) {
-    /*
-    fprintf(stderr,"===> %d %d %.16g\n", Dof_P->NumType, Dof_P->Entity, Value[0]) ;
-    */
-    for(k=0 ; k<NbrHar ; k+=gSCALAR_SIZE){
-      Dof_P->Case.Link.Coef = Value[0] ;
+  Dof.NumType = D1 ;  Dof.Entity = D2 ;
 
+  for(k=0 ; k<NbrHar ; k+=gSCALAR_SIZE){
+    Dof.Harmonic = k ;
+    if(CurrentDofData->DofTree)
+      Dof_P = (struct Dof *)Tree_PQuery(CurrentDofData->DofTree, &Dof);
+    else
+      Dof_P = (struct Dof *)List_PQuery(CurrentDofData->DofList, &Dof, fcmp_Dof);
+
+    if (Dof_P->Type == DOF_LINK || Dof_P->Type == DOF_LINKCPLX) {
+      /*
+        fprintf(stderr,"===> %d %d %.16g\n", Dof_P->NumType, Dof_P->Entity, Value[0]) ;
+      */
+      Dof_P->Case.Link.Coef = Value[0] ;
       if (Dof_P->Type == DOF_LINKCPLX)
 	Dof_P->Case.Link.Coef2 = Value[1] ;
-
       Dof_P->Case.Link.EntityRef = D2_Link ;
       Dof_P->Case.Link.Dof = NULL ;
     }
-
   }
 }
 
@@ -1814,10 +1858,10 @@ void Dof_GetDummies(struct DefineSystem * DefineSystem_P, struct DofData * DofDa
   struct GlobalQuantity   * GlobalQuantity_P ;
   struct Dof              * Dof_P ;
 
-  int i, j, k, l, iDof, ii,iit, iNum, iHar;
+  int i, j, k, l, iDof, ii, iit, iNum, iHar;
   int Nbr_Formulation, Index_Formulation ;
-  int * DummyDof;
-  double DummyFrequency, * Val_Pulsation;
+  int *DummyDof;
+  double FrequencySpectrum, *Val_Pulsation;
 
   if (!(Val_Pulsation = Current.DofData->Val_Pulsation)){
     Message::Error("Dof_GetDummies can only be used for harmonic problems");
@@ -1836,12 +1880,12 @@ void Dof_GetDummies(struct DefineSystem * DefineSystem_P, struct DofData * DofDa
     for (j = 0 ; j < List_Nbr(Formulation_P->DefineQuantity) ; j++) {
       DefineQuantity_P = (struct DefineQuantity*)
 	List_Pointer(Formulation_P->DefineQuantity, j) ;
-      for (l = 0 ; l < List_Nbr(DefineQuantity_P->DummyFrequency) ; l++) {
-	DummyFrequency = *(double *)List_Pointer(DefineQuantity_P->DummyFrequency, l) ;
+      for (l = 0 ; l < List_Nbr(DefineQuantity_P->FrequencySpectrum) ; l++) {
+	FrequencySpectrum = *(double *)List_Pointer(DefineQuantity_P->FrequencySpectrum, l) ;
 
 	iHar=-1;
 	for (k = 0 ; k < Current.NbrHar/2 ; k++)
-	  if (fabs (Val_Pulsation[k]-TWO_PI*DummyFrequency) <= 1e-10 * Val_Pulsation[k]) {
+	  if (fabs(Val_Pulsation[k]-TWO_PI*FrequencySpectrum) <= 1e-10*Val_Pulsation[k]) {
 	    iHar = 2*k; break;
 	  }
 	if(iHar>=0) {
@@ -1862,9 +1906,11 @@ void Dof_GetDummies(struct DefineSystem * DefineSystem_P, struct DofData * DofDa
 		}
 	      }
 	    }
-	    Message::Info("Freq %e (%d/%d) Form %d  Quant %d  Basis %d  #dummies %d/%d",
-                          Val_Pulsation[iHar/2]/TWO_PI, iHar/2, Current.NbrHar/2,
-                          i, j, ((struct BasisFunction *)BasisFunction_P)->Num, ii, iit) ;
+            if(ii)
+              Message::Info("Freq %4lg (%d/%d) Formulation %s Quantity %s (BF %d)  #DofsFreqSpectrum %d/%d",
+                            Val_Pulsation[iHar/2]/TWO_PI, iHar/2, Current.NbrHar/2,
+                            Formulation_P->Name, DefineQuantity_P->Name,
+                            ((struct BasisFunction *)BasisFunction_P)->Num, ii, iit) ;
 	  }
 
 	  for (k = 0 ; k < List_Nbr(FunctionSpace_P->GlobalQuantity) ; k++) {
@@ -1881,25 +1927,28 @@ void Dof_GetDummies(struct DefineSystem * DefineSystem_P, struct DofData * DofDa
 		}
 	      }
 	    }
-	    Message::Info("Freq %e (%d/%d) Form %d  Quant %d  Global %d  #dummies %d/%d",
+            if(ii)
+	    Message::Info("Freq %4lg (%d/%d) Formulation %s  GlobalQuantity %s  (BF %d)  #DofsWithSpectrum %d/%d",
                           Val_Pulsation[iHar/2]/TWO_PI, iHar/2, Current.NbrHar/2,
-                          i, j, ((struct GlobalQuantity *)GlobalQuantity_P)->Num, ii, iit) ;
+                          Formulation_P->Name, GlobalQuantity_P->Name,
+                          ((struct GlobalQuantity *)GlobalQuantity_P)->Num, ii, iit) ;
 	  }
 
-	}   /*  end DummyFrequency in DofData */
-      }   /* end DummyFrequency in Quantity */
+	}   /*  end FrequencySpectrum in DofData */
+      }   /* end FrequencySpectrum in Quantity */
     }   /* end Quantity */
   }   /* end Formulation */
 
   i=0;
   for (iDof = 0 ; iDof < DofData_P->NbrDof ; iDof++) {
     if(DummyDof[iDof]) i++;
-    /*
-    Dof_P = (struct Dof *)List_Pointer(DofData_P->DofList, iDof) ;
-    Message::Info("Dof Num iHar, Entity %d %d %d",
-	          iDof, Dof_P->NumType, Dof_P->Harmonic, Dof_P->Entity);
-    */
+
+    if(Message::GetVerbosity() == 99){
+      Dof_P = (struct Dof *)List_Pointer(DofData_P->DofList, iDof) ;
+      Message::Debug("Dof Num iHar, Entity %d %d %d",
+                     iDof, Dof_P->NumType, Dof_P->Harmonic, Dof_P->Entity);
+     }
   }
 
-  Message::Info("Total %d Dummies %d", DofData_P->NbrDof,i) ;
+  Message::Info("N: %d - N with FrequencySpectrum: %d", DofData_P->NbrDof, i) ;
 }
