@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -13,7 +13,7 @@
 #endif
 
 void EigenSolve(struct DofData * DofData_P, int NumEigenvalues,
-		double shift_r, double shift_i)
+		double shift_r, double shift_i, int FilterExpressionIndex)
 {
 #if defined(HAVE_ARPACK) && defined(HAVE_SLEPC)
   // if both Arpack and SLEPC are available, use Arpack by default
@@ -21,13 +21,17 @@ void EigenSolve(struct DofData * DofData_P, int NumEigenvalues,
   PetscTruth slepc = PETSC_FALSE, set;
   PetscOptionsGetTruth(PETSC_NULL, "-slepc", &slepc, &set);
   if(slepc)
-    EigenSolve_SLEPC(DofData_P, NumEigenvalues, shift_r, shift_i);
+    EigenSolve_SLEPC(DofData_P, NumEigenvalues, shift_r, shift_i,
+                     FilterExpressionIndex);
   else
-    EigenSolve_ARPACK(DofData_P, NumEigenvalues, shift_r, shift_i);
+    EigenSolve_ARPACK(DofData_P, NumEigenvalues, shift_r, shift_i,
+                      FilterExpressionIndex);
 #elif defined(HAVE_ARPACK)
-  EigenSolve_ARPACK(DofData_P, NumEigenvalues, shift_r, shift_i);
+  EigenSolve_ARPACK(DofData_P, NumEigenvalues, shift_r, shift_i,
+                    FilterExpressionIndex);
 #elif defined(HAVE_SLEPC)
-  EigenSolve_SLEPC(DofData_P, NumEigenvalues, shift_r, shift_i);
+  EigenSolve_SLEPC(DofData_P, NumEigenvalues, shift_r, shift_i,
+                   FilterExpressionIndex);
 #else
   Message::Error("EigenSolve not available without SLEPC or ARPACK");
 #endif
