@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -92,6 +92,7 @@ void F_Trace(F_ARG)
 
 void F_RotateXYZ(F_ARG)
 {
+  // Apply a (X_1 Y_2 Z_3) rotation matrix using Euler (Tait-Bryan) angles
   double  ca, sa, cb, sb, cc, sc ;
   struct  Value Rot ;
 
@@ -988,6 +989,15 @@ void F_CompX(F_ARG){ get_comp_vector(0, "CompX") }
 void F_CompY(F_ARG){ get_comp_vector(1, "CompY") }
 void F_CompZ(F_ARG){ get_comp_vector(2, "CompZ") }
 
+void F_Comp(F_ARG){
+  if (Fct->NbrParameters != 1)
+    Message::Error("Function 'Comp': one parameter needed to define component index");
+  if ((int)(Fct->Para[0]) < 0 || (int)(Fct->Para[0]) > 2)
+    Message::Error("Function 'Comp': parameter (%g) out of range (must be 0, 1 or 2)", Fct->Para[0]);
+
+  get_comp_vector((int)(Fct->Para[0]), "Comp")
+}
+
 #undef get_comp_vector
 
 /* ------------------------------------------------------------------------ */
