@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -51,7 +51,7 @@ int Geo_AddGeoData(List_T * GeoData_L,
     Message::Info("Loading Geometric data '%s'", Name_MshFile) ;
     i = List_Nbr(GeoData_L) ;
     Geo_InitGeoData(&GeoData_S, i, Name_MshFile) ;
-    Geo_OpenFile(Name_MshFile, "r") ;
+    Geo_OpenFile(Name_MshFile, "rb") ;
     Geo_ReadFile(&GeoData_S) ;
     Geo_CloseFile() ;
 
@@ -178,29 +178,30 @@ int Geo_GetElementType(int Format, int Type)
   switch(Format){
   case FORMAT_GMSH :
     switch(Type){
-    case 15 : return(POINT) ;
-    case 1  : return(LINE) ;
-    case 2  : return(TRIANGLE) ;
-    case 3  : return(QUADRANGLE) ;
-    case 4  : return(TETRAHEDRON) ;
-    case 5  : return(HEXAHEDRON) ;
-    case 6  : return(PRISM) ;
-    case 7  : return(PYRAMID) ;
-    case 8  : return(LINE_2) ;
-    case 9  : return(TRIANGLE_2) ;
-    case 10 : return(QUADRANGLE_2) ;
-    case 11 : return(TETRAHEDRON_2) ;
-    case 12 : return(HEXAHEDRON_2) ;
-    case 13 : return(PRISM_2) ;
-    case 14 : return(PYRAMID_2) ;
-    case 16 : return(QUADRANGLE_2_8N) ;
-    default : Message::Error("Unkown type of Element in Gmsh format (%d)",
-                             FORMAT_GMSH) ; return(-1) ;
+    case 15 : return POINT;
+    case 1  : return LINE;
+    case 2  : return TRIANGLE;
+    case 3  : return QUADRANGLE;
+    case 4  : return TETRAHEDRON;
+    case 5  : return HEXAHEDRON;
+    case 6  : return PRISM;
+    case 7  : return PYRAMID;
+    case 8  : return LINE_2;
+    case 9  : return TRIANGLE_2;
+    case 10 : return QUADRANGLE_2;
+    case 11 : return TETRAHEDRON_2;
+    case 12 : return HEXAHEDRON_2;
+    case 13 : return PRISM_2;
+    case 14 : return PYRAMID_2;
+    case 16 : return QUADRANGLE_2_8N;
+    default :
+      Message::Error("Unknown type of Element in Gmsh format (%d)", FORMAT_GMSH);
+      return -1;
     }
     break ;
   default :
-    Message::Error("Unkown mesh format (%d)", Format) ;
-    return(-1) ;
+    Message::Error("Unknown mesh format (%d)", Format);
+    return -1;
   }
 }
 
@@ -209,52 +210,125 @@ int Geo_GetElementTypeInv(int Format, int Type)
   switch(Format){
   case FORMAT_GMSH :
     switch(Type){
-    case POINT         : return(15) ;
-    case LINE          : return(1) ;
-    case TRIANGLE      : return(2) ;
-    case QUADRANGLE    : return(3) ;
-    case TETRAHEDRON   : return(4) ;
-    case HEXAHEDRON    : return(5) ;
-    case PRISM         : return(6) ;
-    case PYRAMID       : return(7) ;
-    case LINE_2        : return(8) ;
-    case TRIANGLE_2    : return(9) ;
-    case QUADRANGLE_2  : return(10) ;
-    case TETRAHEDRON_2 : return(11) ;
-    case HEXAHEDRON_2  : return(12) ;
-    case PRISM_2       : return(13) ;
-    case PYRAMID_2     : return(14) ;
-    case QUADRANGLE_2_8N: return(16) ;
-    default : Message::Error("Unkown type of Element in Gmsh format") ; return(-1) ;
+    case POINT         : return 15;
+    case LINE          : return 1;
+    case TRIANGLE      : return 2;
+    case QUADRANGLE    : return 3;
+    case TETRAHEDRON   : return 4;
+    case HEXAHEDRON    : return 5;
+    case PRISM         : return 6;
+    case PYRAMID       : return 7;
+    case LINE_2        : return 8;
+    case TRIANGLE_2    : return 9;
+    case QUADRANGLE_2  : return 10;
+    case TETRAHEDRON_2 : return 11;
+    case HEXAHEDRON_2  : return 12;
+    case PRISM_2       : return 13;
+    case PYRAMID_2     : return 14;
+    case QUADRANGLE_2_8N: return 16;
+    default :
+      Message::Error("Unknown type of Element in Gmsh format");
+      return -1;
     }
     break ;
   default :
-    Message::Error("Unkown mesh format") ;
-    return(-1) ;
+    Message::Error("Unknown mesh format");
+    return -1;
   }
-
 }
 
 int Geo_GetNbNodesPerElement(int Type)
 {
   switch(Type){
-  case POINT         : return(1) ;
-  case LINE          : return(2) ;
-  case TRIANGLE      : return(3) ;
-  case QUADRANGLE    : return(4) ;
-  case TETRAHEDRON   : return(4) ;
-  case HEXAHEDRON    : return(8) ;
-  case PRISM         : return(6) ;
-  case PYRAMID       : return(5) ;
-  case LINE_2        : return(3) ;
-  case TRIANGLE_2    : return(6) ;
-  case QUADRANGLE_2  : return(9) ;
-  case TETRAHEDRON_2 : return(10) ;
-  case HEXAHEDRON_2  : return(20) ;
-  case PRISM_2       : return(15) ;
-  case PYRAMID_2     : return(13) ;
-  case QUADRANGLE_2_8N: return(8) ;
-  default : Message::Error("Unkown type of Element") ; return(-1) ;
+  case POINT         : return 1;
+  case LINE          : return 2;
+  case TRIANGLE      : return 3;
+  case QUADRANGLE    : return 4;
+  case TETRAHEDRON   : return 4;
+  case HEXAHEDRON    : return 8;
+  case PRISM         : return 6;
+  case PYRAMID       : return 5;
+  case LINE_2        : return 3;
+  case TRIANGLE_2    : return 6;
+  case QUADRANGLE_2  : return 9;
+  case TETRAHEDRON_2 : return 10;
+  case HEXAHEDRON_2  : return 20;
+  case PRISM_2       : return 15;
+  case PYRAMID_2     : return 13;
+  case QUADRANGLE_2_8N: return 8;
+  default :
+    Message::Error("Unknown type of Element");
+    return -1;
+  }
+}
+
+int Geo_GetDimOfElement(int Type)
+{
+  switch(Type){
+  case POINT         : return 0;
+  case LINE          : return 1;
+  case TRIANGLE      : return 2;
+  case QUADRANGLE    : return 2;
+  case TETRAHEDRON   : return 3;
+  case HEXAHEDRON    : return 3;
+  case PRISM         : return 3;
+  case PYRAMID       : return 3;
+  case LINE_2        : return 1;
+  case TRIANGLE_2    : return 2;
+  case QUADRANGLE_2  : return 2;
+  case TETRAHEDRON_2 : return 3;
+  case HEXAHEDRON_2  : return 3;
+  case PRISM_2       : return 3;
+  case PYRAMID_2     : return 3;
+  case QUADRANGLE_2_8N: return 2;
+  default :
+    Message::Error("Unknown type of Element");
+    return -1;
+  }
+}
+
+void Geo_ReverseElement(Geo_Element *Geo_Element)
+{
+  int tmp;
+  switch(Geo_Element->Type){
+  case LINE :
+  case LINE_2 :
+    tmp = Geo_Element->NumNodes[0];
+    Geo_Element->NumNodes[0] = Geo_Element->NumNodes[1];
+    Geo_Element->NumNodes[1] = tmp;
+    break;
+  case TRIANGLE :
+    tmp = Geo_Element->NumNodes[1];
+    Geo_Element->NumNodes[1] = Geo_Element->NumNodes[2];
+    Geo_Element->NumNodes[2] = tmp;
+    break;
+  case TRIANGLE_2 :
+    tmp = Geo_Element->NumNodes[1];
+    Geo_Element->NumNodes[1] = Geo_Element->NumNodes[2];
+    Geo_Element->NumNodes[2] = tmp;
+    tmp = Geo_Element->NumNodes[3+0];
+    Geo_Element->NumNodes[3] = Geo_Element->NumNodes[5];
+    Geo_Element->NumNodes[5] = tmp;
+    break;
+  case QUADRANGLE :
+    tmp = Geo_Element->NumNodes[1];
+    Geo_Element->NumNodes[1] = Geo_Element->NumNodes[3];
+    Geo_Element->NumNodes[3] = tmp;
+    break;
+  case QUADRANGLE_2 :
+  case QUADRANGLE_2_8N:
+    tmp = Geo_Element->NumNodes[1];
+    Geo_Element->NumNodes[1] = Geo_Element->NumNodes[3];
+    Geo_Element->NumNodes[3] = tmp;
+    tmp = Geo_Element->NumNodes[4+0];
+    Geo_Element->NumNodes[4] = Geo_Element->NumNodes[7];
+    Geo_Element->NumNodes[7] = tmp;
+    tmp = Geo_Element->NumNodes[5];
+    Geo_Element->NumNodes[5] = Geo_Element->NumNodes[6];
+    Geo_Element->NumNodes[6] = tmp;
+    break;
+  default :
+    break;
   }
 }
 
@@ -315,33 +389,46 @@ void Geo_SaveMesh(struct GeoData * GeoData_P, List_T * InitialList, char * FileN
   List_Delete(GeoData.Elements);
 }
 
+static std::string ExtractDoubleQuotedString(const char *str, int len)
+{
+  char *c = strstr((char*)str, "\"");
+  if(!c) return "";
+  std::string ret;
+  for(int i = 1; i < len; i++) {
+    if(c[i] == '"' || c[i] == EOF || c[i] == '\n' || c[i] == '\r') break;
+    ret.push_back(c[i]);
+  }
+  return ret;
+}
+
 void Geo_ReadFile(struct GeoData * GeoData_P)
 {
   int                 Nbr, i, j, Type, iDummy, Format, Size, NbTags ;
   double              Version = 1.0 ;
   struct Geo_Node     Geo_Node ;
   struct Geo_Element  Geo_Element ;
-  char                String[256] ;
+  char                String[256] = "" ;
   int                 binary = 0, swap = 0;
+  std::map<int, std::vector<int> > entities[4];
 
   while (1) {
 
     do {
-      fgets(String, sizeof(String), File_GEO) ;
-      if (feof(File_GEO))  break ;
+      if(!fgets(String, sizeof(String), File_GEO) || feof(File_GEO))
+        break;
     } while (String[0] != '$') ;
 
-    if (feof(File_GEO))  break ;
+    if (feof(File_GEO))
+      break ;
 
     /*  F O R M A T  */
 
     if(!strncmp(&String[1], "MeshFormat", 10)) {
 
-      fgets(String, sizeof(String), File_GEO) ;
-      if(sscanf(String, "%lf %d %d", &Version, &Format, &Size) != 3)
-        return;
-      if(Version < 2.0 || Version >= 3.0){
-	Message::Error("Unknown mesh file version (%g)", Version);
+      if(!fgets(String, sizeof(String), File_GEO)) return;
+      if(sscanf(String, "%lf %d %d", &Version, &Format, &Size) != 3) return;
+      if(Version < 2.0 || Version >= 3.1){
+	Message::Error("Unsupported or unknown mesh file version (%g)", Version);
 	return;
       }
 
@@ -363,7 +450,7 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 
       // GetDP does not currently use the name information
 
-      fgets(String, sizeof(String), File_GEO) ;
+      if(!fgets(String, sizeof(String), File_GEO)) return ;
       int numNames;
       if(sscanf(String, "%d", &numNames) != 1) return ;
       for(int i = 0; i < numNames; i++) {
@@ -372,7 +459,45 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
           if(fscanf(File_GEO, "%d", &dim) != 1) return ;
         }
         if(fscanf(File_GEO, "%d", &num) != 1) return ;
-        fgets(String, sizeof(String), File_GEO) ;
+        if(!fgets(String, sizeof(String), File_GEO)) return ;
+        std::string name = ExtractDoubleQuotedString(String, 256);
+        Message::Debug("Physical group %d (dim %d) has name %s",
+                       num, dim, name.c_str());
+      }
+    }
+
+    /* E N T I T I E S */
+
+    else if(!strncmp(&String[1], "Entities", 8)) {
+      if(!fgets(String, sizeof(String), File_GEO)) return;
+      int num[4];
+      if(sscanf(String, "%d %d %d %d", &num[0], &num[1], &num[2], &num[3]) != 4) return;
+      for(int dim = 0; dim < 4; dim++) {
+        for(int j = 0; j < num[dim]; j++) {
+          int num;
+          if(fscanf(File_GEO, "%d", &num) != 1) return;
+          int nbound = 0;
+          if(dim > 0){
+            if(fscanf(File_GEO, "%d", &nbound) != 1) return;
+            for(int k = 0; k < nbound; k++){
+              int dummy;
+              if(fscanf(File_GEO, "%d", &dummy) != 1) return;
+            }
+          }
+          int nphys;
+          if(fscanf(File_GEO, "%d", &nphys) != 1) return;
+          std::vector<int> physicals(nphys);
+          for(int k = 0; k < nphys; k++){
+            if(fscanf(File_GEO, "%d", &physicals[k]) != 1) return;
+          }
+          entities[dim][num] = physicals;
+          if(nphys > 1){
+            Message::Error("GetDP does not support multiple physical groups per element:"
+                           " elementary entity %d belongs to %d physical groups",
+                           num, nphys);
+            return;
+          }
+        }
       }
     }
 
@@ -380,9 +505,13 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 
     else if (!strncmp(&String[1], "NOE", 3) ||
 	     !strncmp(&String[1], "NOD", 3) ||
-	     !strncmp(&String[1], "Nodes", 5)) {
+	     !strncmp(&String[1], "Nodes", 5) ||
+             !strncmp(&String[1], "ParametricNodes", 15)) {
 
-      fgets(String, sizeof(String), File_GEO) ;
+      bool parametric =
+        !strncmp(&String[1], "ParametricNodes", 15) || (Version >= 3.0);
+
+      if(!fgets(String, sizeof(String), File_GEO)) return;
       if(sscanf(String, "%d", &Nbr) != 1) return;
       Message::Debug("%d nodes", Nbr);
 
@@ -390,20 +519,94 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 	GeoData_P->Nodes = List_Create(Nbr, 1000, sizeof(struct Geo_Node)) ;
 
       for (i = 0 ; i < Nbr ; i++) {
-	if (!binary){
-	  fscanf(File_GEO, "%d %lf %lf %lf",
-		 &Geo_Node.Num, &Geo_Node.x, &Geo_Node.y, &Geo_Node.z) ;
-	}
-	else {
-	  if(fread(&Geo_Node.Num, sizeof(int), 1, File_GEO) != 1) return;
-	  if(swap) swapBytes((char*)&Geo_Node.Num, sizeof(int), 1);
-	  double xyz[3];
-	  if(fread(xyz, sizeof(double), 3, File_GEO) != 3) return;
-	  if(swap) swapBytes((char*)xyz, sizeof(double), 3);
-	  Geo_Node.x = xyz[0];
-	  Geo_Node.y = xyz[1];
-	  Geo_Node.z = xyz[2];
-	}
+        if(!parametric){
+          if(!binary){
+            if(fscanf(File_GEO, "%d %lf %lf %lf",
+                      &Geo_Node.Num, &Geo_Node.x, &Geo_Node.y, &Geo_Node.z) != 4) return;
+          }
+          else {
+            if(fread(&Geo_Node.Num, sizeof(int), 1, File_GEO) != 1) return;
+            if(swap) swapBytes((char*)&Geo_Node.Num, sizeof(int), 1);
+            double xyz[3];
+            if(fread(xyz, sizeof(double), 3, File_GEO) != 3) return;
+            if(swap) swapBytes((char*)xyz, sizeof(double), 3);
+            Geo_Node.x = xyz[0];
+            Geo_Node.y = xyz[1];
+            Geo_Node.z = xyz[2];
+          }
+        }
+        else{
+          int dim = -1, entity;
+          if(!binary){
+            if(Version < 3.0){
+              if(fscanf(File_GEO, "%d %lf %lf %lf %d %d",
+                        &Geo_Node.Num, &Geo_Node.x, &Geo_Node.y, &Geo_Node.z,
+                        &dim, &entity) != 6) return;
+            }
+            else{
+              if(fscanf(File_GEO, "%d %lf %lf %lf %d",
+                        &Geo_Node.Num, &Geo_Node.x, &Geo_Node.y, &Geo_Node.z,
+                        &entity) != 5) return;
+              if(entity){
+                if(fscanf(File_GEO, "%d", &dim) != 1) return;
+              }
+            }
+          }
+          else{
+            if(fread(&Geo_Node.Num, sizeof(int), 1, File_GEO) != 1) return;
+            if(swap) swapBytes((char*)&Geo_Node.Num, sizeof(int), 1);
+            double xyz[3];
+            if(fread(xyz, sizeof(double), 3, File_GEO) != 3) return;
+            if(swap) swapBytes((char*)xyz, sizeof(double), 3);
+            if(Version < 3.0){
+              if(fread(&dim, sizeof(int), 1, File_GEO) != 1) return;
+              if(swap) swapBytes((char*)&dim, sizeof(int), 1);
+              if(fread(&entity, sizeof(int), 1, File_GEO) != 1) return;
+              if(swap) swapBytes((char*)&entity, sizeof(int), 1);
+            }
+            else{
+              if(fread(&entity, sizeof(int), 1, File_GEO) != 1) return;
+              if(swap) swapBytes((char*)&entity, sizeof(int), 1);
+              if(entity){
+                if(fread(&dim, sizeof(int), 1, File_GEO) != 1) return;
+                if(swap) swapBytes((char*)&dim, sizeof(int), 1);
+              }
+            }
+            Geo_Node.x = xyz[0];
+            Geo_Node.y = xyz[1];
+            Geo_Node.z = xyz[2];
+          }
+          if(dim == 1 && (Version < 3.0 || entity)){
+            double u[1];
+            if(!binary){
+              if(fscanf(File_GEO, "%lf", &u[0]) != 1) return;
+            }
+            else{
+              if(fread(u, sizeof(double), 1, File_GEO) != 1) return;
+              if(swap) swapBytes((char*)u, sizeof(double), 1);
+            }
+          }
+          else if(dim == 2 && (Version < 3.0 || entity)){
+            double uv[2];
+            if(!binary){
+              if(fscanf(File_GEO, "%lf %lf", &uv[0], &uv[1]) != 2) return;
+            }
+            else{
+              if(fread(uv, sizeof(double), 2, File_GEO) != 2) return;
+              if(swap) swapBytes((char*)uv, sizeof(double), 2);
+            }
+          }
+          else if(dim == 3 && Version >= 3.0 && entity){
+            double uvw[3];
+            if(!binary){
+              if(fscanf(File_GEO, "%lf %lf %lf", &uvw[0], &uvw[1], &uvw[2]) != 3) return;
+            }
+            else{
+              if(fread(uvw, sizeof(double), 3, File_GEO) != 2) return;
+              if(swap) swapBytes((char*)uvw, sizeof(double), 3);
+            }
+          }
+        }
 	List_Add(GeoData_P->Nodes, &Geo_Node) ;
 	if(!i){
 	  GeoData_P->Xmin = GeoData_P->Xmax = Geo_Node.x;
@@ -447,19 +650,12 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 	GeoData_P->CharacteristicLength = 1.;
     }
 
-    else if (!strncmp(&String[1], "ParametricNodes", 15)) {
-
-      Message::Error("Cannot read parametric nodes: please save mesh file with standard "
-                     "nodes instead!");
-
-    }
-
     /*  E L E M E N T S  */
 
     else if (!strncmp(&String[1], "ELM", 3) ||
 	     !strncmp(&String[1], "Elements", 8)) {
 
-      fgets(String, sizeof(String), File_GEO) ;
+      if(!fgets(String, sizeof(String), File_GEO)) return;
       if(sscanf(String, "%d", &Nbr) != 1) return;
       Message::Debug("%d elements", Nbr);
 
@@ -473,16 +669,18 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
       if (!binary){
 	for (i = 0 ; i < Nbr ; i++) {
 	  if(Version == 1.0){
-	    fscanf(File_GEO, "%d %d %d %d %d",
-		   &Geo_Element.Num, &Type, &Geo_Element.Region,
-		   &Geo_Element.ElementaryRegion, &Geo_Element.NbrNodes) ;
+	    if(fscanf(File_GEO, "%d %d %d %d %d",
+                      &Geo_Element.Num, &Type, &Geo_Element.Region,
+                      &Geo_Element.ElementaryRegion, &Geo_Element.NbrNodes) != 5)
+              return;
 	    Geo_Element.Type = Geo_GetElementType(FORMAT_GMSH, Type) ;
 	  }
-	  else{
-	    fscanf(File_GEO, "%d %d %d", &Geo_Element.Num, &Type, &NbTags);
+	  else if(Version < 3.0){
+	    if(fscanf(File_GEO, "%d %d %d", &Geo_Element.Num, &Type, &NbTags) != 3)
+              return;
 	    Geo_Element.Region = Geo_Element.ElementaryRegion = 1;
 	    for(j = 0; j < NbTags; j++){
-	      fscanf(File_GEO, "%d", &iDummy);
+	      if(fscanf(File_GEO, "%d", &iDummy) != 1) return;
 	      if(j == 0)
 		Geo_Element.Region = iDummy;
               else if(j == 1)
@@ -492,40 +690,117 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 	    Geo_Element.Type = Geo_GetElementType(FORMAT_GMSH, Type) ;
 	    Geo_Element.NbrNodes = Geo_GetNbNodesPerElement(Geo_Element.Type);
 	  }
+          else{
+            int numData;
+	    if(fscanf(File_GEO, "%d %d %d %d", &Geo_Element.Num, &Type,
+                      &Geo_Element.ElementaryRegion, &numData) != 4) return;
+	    Geo_Element.Type = Geo_GetElementType(FORMAT_GMSH, Type) ;
+	    Geo_Element.NbrNodes = Geo_GetNbNodesPerElement(Geo_Element.Type);
+            std::vector<int> phys = entities[Geo_GetDimOfElement(Geo_Element.Type)]
+              [Geo_Element.ElementaryRegion];
+            if(phys.empty()){
+              Message::Error("No physical group provided for element %d",
+                             Geo_Element.Num);
+              return;
+            }
+            else
+              Geo_Element.Region = phys[0];
+
+            /* ignore any other tags for now */
+            for(j = 0; j < numData - Geo_Element.NbrNodes; j++){
+              if(fscanf(File_GEO, "%d", &iDummy) != 1) return;
+            }
+          }
+
 	  Geo_Element.NumNodes = (int *)Malloc(Geo_Element.NbrNodes * sizeof(int)) ;
 	  for (j = 0 ; j < Geo_Element.NbrNodes ; j++)
 	    fscanf(File_GEO, "%d", &Geo_Element.NumNodes[j]) ;
 
+          if(Geo_Element.Region < 0){
+            Geo_ReverseElement(&Geo_Element);
+            Geo_Element.Region = -Geo_Element.Region;
+          }
+
 	  List_Add(GeoData_P->Elements, &Geo_Element) ;
 	}
       }
       else {
-	int numElementsPartial = 0;
-	while(numElementsPartial < Nbr){
-	  int header[3];
-	  if(fread(header, sizeof(int), 3, File_GEO) != 3) return;
-	  if(swap) swapBytes((char*)header, sizeof(int), 3);
-	  Type = header[0];
-	  int numElms = header[1];
-	  int numTags = header[2];
-	  Geo_Element.Type = Geo_GetElementType(FORMAT_GMSH, Type) ;
-	  Geo_Element.NbrNodes = Geo_GetNbNodesPerElement(Geo_Element.Type);
-	  unsigned int n = 1 + numTags + Geo_Element.NbrNodes;
-	  int *data = (int *)Malloc(n * sizeof(int)) ;
-	  for(i = 0; i < numElms; i++) {
-	    if(fread(data, sizeof(int), n, File_GEO) != n) return;
-	    if(swap) swapBytes((char*)data, sizeof(int), n);
-	    Geo_Element.Num = data[0];
-	    Geo_Element.Region = (numTags > 0) ? data[1] : 0;
-	    Geo_Element.ElementaryRegion = (numTags > 1) ? data[2] : 0;
-	    Geo_Element.NumNodes = (int *)Malloc(Geo_Element.NbrNodes * sizeof(int)) ;
-	    for (j = 0 ; j < Geo_Element.NbrNodes ; j++)
-	      Geo_Element.NumNodes[j] = data[numTags + 1 + j] ;
-	    List_Add(GeoData_P->Elements, &Geo_Element) ;
-	  }
-	  Free(data);
-	  numElementsPartial += numElms;
-	}
+        if(Version < 3.0){
+          int numElementsPartial = 0;
+          while(numElementsPartial < Nbr){
+            int header[3];
+            if(fread(header, sizeof(int), 3, File_GEO) != 3) return;
+            if(swap) swapBytes((char*)header, sizeof(int), 3);
+            Type = header[0];
+            int numElms = header[1];
+            int numTags = header[2];
+            Geo_Element.Type = Geo_GetElementType(FORMAT_GMSH, Type) ;
+            Geo_Element.NbrNodes = Geo_GetNbNodesPerElement(Geo_Element.Type);
+            unsigned int n = 1 + numTags + Geo_Element.NbrNodes;
+            int *data = (int *)Malloc(n * sizeof(int)) ;
+            for(i = 0; i < numElms; i++) {
+              if(fread(data, sizeof(int), n, File_GEO) != n) return;
+              if(swap) swapBytes((char*)data, sizeof(int), n);
+              Geo_Element.Num = data[0];
+              Geo_Element.Region = (numTags > 0) ? data[1] : 0;
+              Geo_Element.ElementaryRegion = (numTags > 1) ? data[2] : 0;
+              Geo_Element.NumNodes = (int *)Malloc(Geo_Element.NbrNodes * sizeof(int)) ;
+              for (j = 0 ; j < Geo_Element.NbrNodes ; j++)
+                Geo_Element.NumNodes[j] = data[numTags + 1 + j] ;
+              List_Add(GeoData_P->Elements, &Geo_Element) ;
+            }
+            Free(data);
+            numElementsPartial += numElms;
+          }
+        }
+        else{
+          for (i = 0 ; i < Nbr ; i++) {
+            int numData;
+            if(fread(&Geo_Element.Num, sizeof(int), 1, File_GEO) != 1) return;
+            if(swap) swapBytes((char*)&Geo_Element.Num, sizeof(int), 1);
+            if(fread(&Type, sizeof(int), 1, File_GEO) != 1) return;
+            if(swap) swapBytes((char*)&Type, sizeof(int), 1);
+            if(fread(&Geo_Element.ElementaryRegion, sizeof(int), 1, File_GEO) != 1) return;
+            if(swap) swapBytes((char*)&Geo_Element.ElementaryRegion, sizeof(int), 1);
+            if(fread(&numData, sizeof(int), 1, File_GEO) != 1) return;
+            if(swap) swapBytes((char*)&numData, sizeof(int), 1);
+            std::vector<int> data;
+            if(numData > 0){
+              data.resize(numData);
+              if((int)fread(&data[0], sizeof(int), numData, File_GEO) != numData) return;
+              if(swap) swapBytes((char*)&data[0], sizeof(int), numData);
+            }
+            Geo_Element.Type = Geo_GetElementType(FORMAT_GMSH, Type) ;
+            Geo_Element.NbrNodes = Geo_GetNbNodesPerElement(Geo_Element.Type);
+            Geo_Element.NumNodes = (int *)Malloc(Geo_Element.NbrNodes * sizeof(int)) ;
+            if((int)data.size() >= Geo_Element.NbrNodes){
+              for (j = 0 ; j < Geo_Element.NbrNodes ; j++){
+                Geo_Element.NumNodes[j] = data[numData - Geo_Element.NbrNodes + j] ;
+              }
+            }
+            else{
+              Message::Error("Missing node tags in element %d",
+                             Geo_Element.Num);
+              return;
+            }
+            std::vector<int> phys = entities[Geo_GetDimOfElement(Geo_Element.Type)]
+              [Geo_Element.ElementaryRegion];
+            if(phys.empty()){
+              Message::Error("No physical group provided for element %d",
+                             Geo_Element.Num);
+              return;
+            }
+            else
+              Geo_Element.Region = phys[0];
+
+            if(Geo_Element.Region < 0){
+              Geo_ReverseElement(&Geo_Element);
+              Geo_Element.Region = -Geo_Element.Region;
+            }
+
+            List_Add(GeoData_P->Elements, &Geo_Element) ;
+          }
+        }
       }
 
       List_Sort(GeoData_P->Elements, fcmp_Elm) ;
@@ -533,8 +808,8 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
     }
 
     do {
-      fgets(String, sizeof(String), File_GEO) ;
-      if (feof(File_GEO)){ Message::Error("Prematured end of file"); return; }
+      if(!fgets(String, sizeof(String), File_GEO) || feof(File_GEO))
+        break;
     } while (String[0] != '$') ;
 
   }   /* while 1 ... */
@@ -562,8 +837,8 @@ void Geo_ReadFileAdapt(struct GeoData * GeoData_P)
   while (1) {
 
     do {
-      fgets(String, sizeof(String), File_GEO) ;
-      if (feof(File_GEO))  break ;
+      if(!fgets(String, sizeof(String), File_GEO) || feof(File_GEO))
+        break ;
     } while (String[0] != '$') ;
 
     if (feof(File_GEO))  break ;
@@ -585,8 +860,8 @@ void Geo_ReadFileAdapt(struct GeoData * GeoData_P)
     }
 
     do {
-      fgets(String, sizeof(String), File_GEO) ;
-      if (feof(File_GEO)){ Message::Error("Prematured end of file"); break; }
+      if(!fgets(String, sizeof(String), File_GEO) || feof(File_GEO))
+        break;
     } while (String[0] != '$') ;
 
   }   /* while 1 ... */
@@ -657,7 +932,8 @@ struct Geo_Element *Geo_GetGeoElementOfNum(int Num_Element)
 /*  G e o _ G e t N b r G e o N o d e s                                     */
 /* ------------------------------------------------------------------------ */
 
-int Geo_GetNbrGeoNodes(void){
+int Geo_GetNbrGeoNodes(void)
+{
   return(List_Nbr(CurrentGeoData->Nodes)) ;
 }
 
