@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -56,6 +56,7 @@ void  Treatment_ConstraintForElement(struct FunctionSpace    * FunctionSpace_P,
 
     case ASSIGN :                case INIT :
     case ASSIGNFROMRESOLUTION :  case INITFROMRESOLUTION :
+    case ASSIGN_LOCALPROJ :      case INIT_LOCALPROJ :
     case CST_LINK : case CST_LINKCPLX :
 
       switch(Constraint_P->QuantityType) {
@@ -83,6 +84,9 @@ void  Treatment_ConstraintForElement(struct FunctionSpace    * FunctionSpace_P,
 		Geo_GetNodesCoordinates(1, &Current.NumEntity,
                                         &Current.x, &Current.y, &Current.z) ;
 		break ;
+	      case VOLUMESOF :
+		Current.NumEntity = abs(Num_Entity[i_Entity]) ;
+                break;
 	      case EDGESOF :
 		Current.NumEntity = abs(Num_Entity[i_Entity]) ;
 		Current.NumEntityInElement = i_Entity ;
@@ -252,7 +256,7 @@ void  Get_ValueForConstraint(struct ConstraintInFS * Constraint_P, double Value[
 	 List_Pointer(Problem_S.Expression,
 		      Constraint_P->ConstraintPerRegion->TimeFunctionIndex),
 	 NULL, 0., 0., 0., &Val_TimeFunction) ;
-      Cal_ProductValue(&Val_Modulus, &Val_TimeFunction,  &Val_Modulus) ;
+      Cal_ProductValue(&Val_Modulus, &Val_TimeFunction, &Val_Modulus) ;
     }
     for (k = 0 ; k < Current.NbrHar ; k++){
       Value[k] = Val_Modulus.Val[MAX_DIM*k] ;
@@ -329,12 +333,13 @@ void  Treatment_ConstraintForRegion(struct GlobalQuantity   * GlobalQuantity_P,
 	      ConstraintPerRegion_P->Type ;
 
 	    if (ConstraintPerRegion_P->Type == ASSIGN ||
-		ConstraintPerRegion_P->Type == INIT)
+		ConstraintPerRegion_P->Type == INIT) {
 	      Get_ValueForConstraint
 		(Constraint_P,
                  QuantityStorage_P->BasisFunction[0].Value,
                  QuantityStorage_P->BasisFunction[0].Value2,
 		 &QuantityStorage_P->BasisFunction[0].TimeFunctionIndex) ;
+            }
 	    else if (ConstraintPerRegion_P->Type == CST_LINK ||
 		     ConstraintPerRegion_P->Type == CST_LINKCPLX) {
 	      Get_LinkForConstraint
@@ -451,6 +456,7 @@ void  Generate_LinkNodes(struct ConstraintInFS * Constraint_P,
 			 List_T * ExtendedList_L, List_T * ExtendedSuppList_L,
 			 struct Group * RegionRef_P, struct Group * SubRegionRef_P,
 			 int Index_Filter, int Index_Function, int Index_Coef,
+                         double ToleranceFactor,
 			 List_T * Couples_L) ;
 void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
 			 struct Group * Group_P,
@@ -529,6 +535,7 @@ void  Generate_Link(struct ConstraintInFS * Constraint_P, int Flag_New)
 		       Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FunctionIndex,
 		       Constraint_P->ConstraintPerRegion->Case.Link.CoefIndex,
+		       Constraint_P->ConstraintPerRegion->Case.Link.ToleranceFactor,
 		       Active->Case.Link.Couples) ;
     break ;
   case EDGESOF :
@@ -560,16 +567,20 @@ void  Generate_LinkNodes(struct ConstraintInFS * Constraint_P,
 			 List_T * ExtendedList_L, List_T * ExtendedSuppList_L,
 			 struct Group * RegionRef_P, struct Group * SubRegionRef_P,
 			 int Index_Filter, int Index_Function, int Index_Coef,
+                         double ToleranceFactor,
 			 List_T * Couples_L)
 {
   int  Nbr_Entity, i, Nbr_EntityRef, Flag_Filter ;
-  double TOL = Current.GeoData->CharacteristicLength * 1.e-8;
+  double TOL ;
   struct TwoIntOneDouble  TwoIntOneDouble ;
   struct NodeXYZ  NodeXYZ, NodeXYZRef ;
   List_T  * NodeXYZ_L, * NodeXYZRef_L ;
   List_T  * ExtendedListRef_L, * ExtendedSuppListRef_L ;
   struct Value  Value ;
 
+  TOL = Current.GeoData->CharacteristicLength * ToleranceFactor ;
+  // by default, ToleranceFactor is 1.e-8 (to be defined with ToleranceFactor value; in the Link constraint
+
   /* Nodes with Constraint */
 
   Nbr_Entity = List_Nbr(ExtendedList_L) ;
@@ -733,6 +744,7 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FunctionIndex,
 		       Constraint_P->ConstraintPerRegion->Case.Link.CoefIndex,
+                       Constraint_P->ConstraintPerRegion->Case.Link.ToleranceFactor,
 		       CouplesOfNodes_L) ;
   }
   else {
@@ -743,11 +755,13 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FunctionIndex,
 		       Constraint_P->ConstraintPerRegion->Case.Link.CoefIndex,
+                       Constraint_P->ConstraintPerRegion->Case.Link.ToleranceFactor,
 		       CouplesOfNodes_L) ;
     Generate_LinkNodes(Constraint_P, ExtendedListNodes_L, NULL, RegionRef_P, NULL,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex2,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FunctionIndex2,
 		       Constraint_P->ConstraintPerRegion->Case.Link.CoefIndex2,
+                       Constraint_P->ConstraintPerRegion->Case.Link.ToleranceFactor,
 		       CouplesOfNodes2_L) ;
   }
 
@@ -996,6 +1010,7 @@ void  Generate_LinkFacets(struct ConstraintInFS * Constraint_P,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FunctionIndex,
 		       Constraint_P->ConstraintPerRegion->Case.Link.CoefIndex,
+                       Constraint_P->ConstraintPerRegion->Case.Link.ToleranceFactor,
 		       CouplesOfNodes_L) ;
   }
   else {
@@ -1006,11 +1021,13 @@ void  Generate_LinkFacets(struct ConstraintInFS * Constraint_P,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FunctionIndex,
 		       Constraint_P->ConstraintPerRegion->Case.Link.CoefIndex,
+                       Constraint_P->ConstraintPerRegion->Case.Link.ToleranceFactor,
 		       CouplesOfNodes_L) ;
     Generate_LinkNodes(Constraint_P, ExtendedListNodes_L, NULL, RegionRef_P, NULL,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex2,
 		       Constraint_P->ConstraintPerRegion->Case.Link.FunctionIndex2,
 		       Constraint_P->ConstraintPerRegion->Case.Link.CoefIndex2,
+                       Constraint_P->ConstraintPerRegion->Case.Link.ToleranceFactor,
 		       CouplesOfNodes2_L) ;
   }
 
