@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -200,6 +200,14 @@ void  Get_DofOfElement(struct Element          * Element,
 	   StartingIndex, i_BFunction, REGION, Element->NumSubFunction[1]) ;
 	break ;
 
+      case GROUPOFREGIONSOF :
+        Get_GroupOfRegionsForElement(Element, &StartingIndex, BasisFunction_P) ;
+        Get_CodesOfElement
+          (FunctionSpace_P, QuantityStorage_P,
+           Element->NbrGroupsOfEntities, Element->NumGroupsOfEntities,
+           StartingIndex, i_BFunction, GROUPOFREGIONSOF, Element->NumSubFunction[1]) ;
+        break ;
+
       case GLOBAL :
 	Get_GlobalForElement(Element, &StartingIndex, BasisFunction_P) ;
 	Get_CodesOfElement
@@ -215,6 +223,8 @@ void  Get_DofOfElement(struct Element          * Element,
 
   QuantityStorage_P->NbrElementaryBasisFunction = Nbr_ElementaryBF ;
 
+  // FIXME: just for testing - this should be called in a separate loop with
+  // appropriate initializations in Treatment_Formulation
   if (TreatmentStatus == _PRE)
     Treatment_ConstraintByLocalProjection(Element, FunctionSpace_P,
                                           QuantityStorage_P);
@@ -383,6 +393,42 @@ void  Get_RegionForElement(struct Element * Element, int * StartingIndex,
   }
 }
 
+/* ------------------------------------------------------------------------ */
+/*  G e t _ G r o u p O f R e g i o n s F o r E l e m e n t                 */
+/* ------------------------------------------------------------------------ */
+
+void  Get_GroupOfRegionsForElement(struct Element * Element, int * StartingIndex,
+                                   struct BasisFunction * BasisFunction_P)
+{
+  int Nbr_SubFunction, i_SF ;
+
+  if (Element->NumLastElementForGroupsOfEntities != Element->Num) {
+    Element->NumLastElementForGroupsOfEntities = Element->Num ;
+    Element->NbrGroupsOfEntities = 0 ;
+  }
+
+  *StartingIndex = Element->NbrGroupsOfEntities ;
+
+  if (!BasisFunction_P->SubFunction) {
+    Element->NumSubFunction[1][Element->NbrGroupsOfEntities] = 0 ;
+    Element->NumSubFunction[0][Element->NbrGroupsOfEntities] = -1 ;
+    Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = GroupEntity_P->Num ;
+  }
+  else { /* For SubFunctions (basis functions for a global function) */
+    Nbr_SubFunction = List_Nbr(BasisFunction_P->SubFunction) ;
+
+    for (i_SF = 0 ; i_SF < Nbr_SubFunction ; i_SF++) {
+      Element->NumSubFunction[1][Element->NbrGroupsOfEntities] = i_SF ; /* Index SF */
+      Element->NumSubFunction[0][Element->NbrGroupsOfEntities] =
+        *((int *)List_Pointer(BasisFunction_P->SubFunction, i_SF)) ; /* Index Expression */
+      if (BasisFunction_P->SubdFunction)
+        Element->NumSubFunction[2][Element->NbrGroupsOfEntities] =
+          *((int *)List_Pointer(BasisFunction_P->SubdFunction, i_SF)) ; /* Index Expression */
+      Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = GroupEntity_P->Num ;
+    }
+  }
+}
+
 /* ------------------------------------------------------------------------ */
 /*  G e t _ G l o b a l F o r E l e m e n t                                 */
 /* ------------------------------------------------------------------------ */
@@ -402,16 +448,24 @@ void  Get_GlobalForElement(struct Element * Element, int * StartingIndex,
   Nbr_Global = List_Nbr(GroupEntity_P->InitialList) ;
   Num_Global = (Nbr_Global)? (int*)List_Pointer(GroupEntity_P->InitialList, 0) : NULL ;
 
-  for (i = 0 ; i < Nbr_Global ; i++) {
-    Element->GlobalBasisFunction[Element->NbrGroupsOfEntities  ] =
-      (struct GlobalBasisFunction *)
-      List_Pointer(BasisFunction_P->GlobalBasisFunction, i) ;
-    /* Attention: correspondance i-i si liste triee ! fait dans yacc */
-    Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = Num_Global[i] ;
+  if (BasisFunction_P->GlobalBasisFunction) {
+    for (i = 0 ; i < Nbr_Global ; i++) {
+      Element->GlobalBasisFunction[Element->NbrGroupsOfEntities  ] =
+        (struct GlobalBasisFunction *)
+        List_Pointer(BasisFunction_P->GlobalBasisFunction, i) ;
+      /* Attention: correspondance i-i si liste triee ! fait dans yacc */
+      Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = Num_Global[i] ;
+    }
+
+    if (TreatmentStatus == _PRE)
+      Get_PreResolutionForGlobalBasisFunction(Nbr_Global, *StartingIndex, Element) ;
+  }
+  else {
+    for (i = 0 ; i < Nbr_Global ; i++) {
+      Element->NumGroupsOfEntities[Element->NbrGroupsOfEntities++] = Num_Global[i] ;
+    }
   }
 
-  if (TreatmentStatus == _PRE)
-    Get_PreResolutionForGlobalBasisFunction(Nbr_Global, *StartingIndex, Element) ;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -510,6 +564,7 @@ void  Get_DofOfRegion(int  Num_Region,
 		      struct QuantityStorage  * QuantityStorage_P)
 {
   int  CodeExist = 0, Num_BasisFunction, Num_AssociateBasisFunction ;
+  int  Num_Entity = -1;
   struct Dof  * Dof_P = NULL;
 
   Nbr_ElementaryBF = 0 ;
@@ -532,6 +587,11 @@ void  Get_DofOfRegion(int  Num_Region,
       Num_AssociateBasisFunction = BasisFunction_P->Num ;
     }
 
+    if (GroupEntity_P->FunctionType == GROUPOFREGIONSOF)
+      Num_Entity = GroupEntity_P->Num;
+    else
+      Num_Entity = Num_Region;
+
     switch (TreatmentStatus) {
     case _CAL :
     case _POS :
@@ -544,7 +604,7 @@ void  Get_DofOfRegion(int  Num_Region,
 
       CodeExist =
 	((Dof_P = Dof_GetDofStruct(FunctionSpace_P->DofData,
-				   Num_BasisFunction, Num_Region, 0)) != NULL) ;
+				   Num_BasisFunction, Num_Entity, 0)) != NULL) ;
       break ;
     case _PRE :
       CodeExist = 1 ;
@@ -556,7 +616,7 @@ void  Get_DofOfRegion(int  Num_Region,
       QuantityStorage_P->BasisFunction[0].Dof = Dof_P ;
 
       QuantityStorage_P->BasisFunction[0].CodeBasisFunction = Num_BasisFunction ;
-      QuantityStorage_P->BasisFunction[0].CodeEntity        = Num_Region ;
+      QuantityStorage_P->BasisFunction[0].CodeEntity        = Num_Entity ;
 
       QuantityStorage_P->BasisFunction[0].CodeAssociateBasisFunction =
 	Num_AssociateBasisFunction ;
