@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -138,6 +138,9 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
 
     switch (Type_Element) {
 
+    case LINE        : case LINE_2 :
+      *Type_Dimension = _1D ; return((void *)JacobianVolAxi1D) ;
+
     case TRIANGLE    : case TRIANGLE_2   :
     case QUADRANGLE  : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
       *Type_Dimension = _2D ; return((void *)JacobianVolAxi2D) ;
@@ -191,6 +194,9 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
 
     switch (Type_Element) {
 
+    case LINE        : case LINE_2 :
+      *Type_Dimension = _1D ; return((void *)JacobianVolAxiSqu1D) ;
+
     case TRIANGLE    : case TRIANGLE_2   :
     case QUADRANGLE  : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
       *Type_Dimension = _2D ; return((void *)JacobianVolAxiSqu2D) ;
@@ -264,6 +270,12 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
     case LINE        : case LINE_2 :
       *Type_Dimension = _2D ; return((void *)JacobianSurAxi2D) ;
 
+      // for integrals on surfaces in the study plane in axisymm. problems
+      // e.g. the computation of the area of a region
+    case TRIANGLE    : case TRIANGLE_2   :
+    case QUADRANGLE  : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
+      *Type_Dimension = _2D ; return((void *)JacobianVol2D) ;
+
     default :
       Message::Error("Unknown Jacobian SurAxi for Element Type (%s)",
                      Get_StringForDefine(Element_Type, Type_Element));
@@ -442,26 +454,28 @@ double  Transformation (int Dim, int Type, struct Element * Element, MATRIX3x3 *
   if(Type == JACOBIAN_RECT){
     if(Element->JacobianCase->NbrParameters >= 3)
       Axis = (int)Element->JacobianCase->Para[2] ;
-    if(Element->JacobianCase->NbrParameters >= 6){
+    if(Element->JacobianCase->NbrParameters >= 4)
       Cx = Element->JacobianCase->Para[3] ;
+    if(Element->JacobianCase->NbrParameters >= 5)
       Cy = Element->JacobianCase->Para[4] ;
+    if(Element->JacobianCase->NbrParameters >= 6)
       Cz = Element->JacobianCase->Para[5] ;
-    }
     if(Element->JacobianCase->NbrParameters >= 7)
       p = Element->JacobianCase->Para[6] ;
     if(Element->JacobianCase->NbrParameters >= 8)
       L = Element->JacobianCase->Para[7] ;
     if(Element->JacobianCase->NbrParameters >= 9){
       Message::Error("Too many parameters for rectangular transformation Jacobian. "
-                     "Vvalid parameters: Radius1 Radius2 Axis CenterX CenterY CenterZ Power 1/Infinity");
+                     "Valid parameters: Dist1 Dist2 Axis CenterX CenterY CenterZ Power 1/Infinity");
     }
   }
   else if(Type == JACOBIAN_SPH){
-    if(Element->JacobianCase->NbrParameters >= 5){
+    if(Element->JacobianCase->NbrParameters >= 3)
       Cx = Element->JacobianCase->Para[2] ;
+    if(Element->JacobianCase->NbrParameters >= 4)
       Cy = Element->JacobianCase->Para[3] ;
+    if(Element->JacobianCase->NbrParameters >= 5)
       Cz = Element->JacobianCase->Para[4] ;
-    }
     if(Element->JacobianCase->NbrParameters >= 6)
       p = Element->JacobianCase->Para[5] ;
     if(Element->JacobianCase->NbrParameters >= 7)
@@ -531,7 +545,6 @@ double  Transformation (int Dim, int Type, struct Element * Element, MATRIX3x3 *
     break;
   default :
     DetJac = f * f * f * (1.0 - theta);
-
     /*
     DetJac =  Jac->c11 * (Jac->c22 * Jac->c33 - Jac->c23*Jac->c32)
             - Jac->c12 * (Jac->c21 * Jac->c33 - Jac->c23*Jac->c31)
@@ -652,7 +665,29 @@ double  JacobianVolPlpdX2D (struct Element * Element, MATRIX3x3 * Jac)
   return(DetJac1 * DetJac2) ;
 }
 
-/* 2D Axi (Attention, l'axe doit etre x=z=0) */
+/* 1D & 2D Axi (Attention, l'axe doit etre x=z=0) */
+
+double  JacobianVolAxi1D (struct Element * Element, MATRIX3x3 * Jac)
+{
+  int  i ;
+  double s = 0., DetJac ;
+
+  DetJac = JacobianVol1D(Element, Jac) ;
+
+  for (i = 0 ; i < Element->GeoElement->NbrNodes ; i++)
+    s += Element->x[i] * Element->n[i] ;
+
+  /* Warning! For evaluations on the symmetry axis */
+  if (s==0.0) {
+    for (i = 0 ; i < Element->GeoElement->NbrNodes ; i++)
+      s += Element->x[i] ;
+    s /= (double)Element->GeoElement->NbrNodes  ;
+  }
+
+  Jac->c33 = s ;
+
+  return(DetJac * Jac->c33) ;
+}
 
 double  JacobianVolAxi2D (struct Element * Element, MATRIX3x3 * Jac)
 {
@@ -727,7 +762,38 @@ double  JacobianVolAxiPlpdX2D (struct Element * Element, MATRIX3x3 * Jac)
   return(DetJac1 * DetJac2) ;
 }
 
-/* 2D Axi avec transformation quadratique (Attention, l'axe doit etre x=z=0) */
+/* 1D & 2D Axi avec transformation quadratique (Attention, l'axe doit etre x=z=0) */
+double  JacobianVolAxiSqu1D (struct Element * Element, MATRIX3x3 * Jac)
+{
+  int    i ;
+  double s = 0., r, DetJac ;
+
+  Jac->c11 = 0. ;  Jac->c12 = 0. ;  Jac->c13 = 0. ;
+  Jac->c21 = 0. ;  Jac->c22 = 1. ;  Jac->c23 = 0. ;
+  Jac->c31 = 0. ;  Jac->c32 = 0. ;  Jac->c33 = 1. ;
+
+  for (i = 0 ; i < Element->GeoElement->NbrNodes ; i++)
+    s += SQU(Element->x[i]) * Element->n[i] ;
+
+
+  /* Warning! For evaluations on the symmetry axis */
+  if (s==0.0) {
+    for (i = 0 ; i < Element->GeoElement->NbrNodes ; i++)
+      s += Element->x[i] * Element->x[i] ;
+    s /= (double)Element->GeoElement->NbrNodes  ;
+  }
+
+  r = sqrt(s);
+
+  for ( i = 0 ; i < Element->GeoElement->NbrNodes ; i++ ) {
+    Jac->c11 += 0.5/r * SQU(Element->x[i]) * Element->dndu[i][0] ;
+  }
+  Jac->c33 = r ;
+
+  DetJac = Jac->c11 * Jac->c33 ;
+
+  return(DetJac) ;
+}
 
 double  JacobianVolAxiSqu2D (struct Element * Element, MATRIX3x3 * Jac)
 {
@@ -1247,7 +1313,7 @@ void  ChangeOfCoord_Form1P(struct Element * Element,
 void  ChangeOfCoord_Form2P(struct Element * Element,
 			   double vBFu[], double vBFx[])
 {
-  if(!Element->DetJac) Message::Error("Null determinant in 'ChangeOfCoord_Form2P'");
+  if(!Element->DetJac) Message::Error("Null determinant in 'ChangeOfCoord_Form2P' %d %d %d", Element->Num, Element->Type, Element->Region);
 
   vBFx[0] = (vBFu[0] * Element->Jac.c11 + vBFu[1] * Element->Jac.c21)
     / Element->DetJac ;
