@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -35,29 +35,20 @@ extern struct CurrentData Current ;
 // Options for PETSc can be provided on the command line, or in the file
 // ~/.petscrc.
 //
-// By default we try to use the MUMPS or UMFPACK direct solver (if
-// available, with PETSc 3). Otherwise we use a GMRES iterative solver
-// preconditionned with an ILU(6).
+// By default we use a direct solver (MUMPS, UMFPACK or the PETSc LU).
 //
 // All these options can be changed at runtime. For example you could
 // use
 //
 //   -pc_type ilu
 //   -pc_factor_levels 0
+//   -ksp_type gmres
 //   -ksp_rtol 1.e-6
 //   -ksp_gmres_restart 500
 //   -ksp_monitor
 //
 // for GMRES with ILU(0), with a restart of 500 and a stopping
-// criterion of 1e-6. Or you could set
-//
-//   -pc_type lu
-//   -pc_factor_mat_solver_package mumps
-//   -ksp_type gmres
-//
-// to use the MUMPS direct solver as a preconditioner to GMRES (useful
-// e.g if the matrix changed a bit, but you want to keep the same
-// factorization using "SolveAgain").
+// criterion of 1e-6.
 
 static MPI_Comm MyComm = MPI_COMM_SELF;
 static PetscViewer MyPetscViewer;
@@ -66,12 +57,14 @@ static void _try(int ierr)
 {
   CHKERRCONTINUE(ierr);
   if(PetscUnlikely(ierr)){
-    // Do not produce an error in case of a PETSc-crash
-    // when we are in TimeLoopAdaptive loop
+    const char *text;
+    PetscErrorMessage(ierr, &text, 0);
+    // Do not produce an error in case of a PETSc-crash when we are in
+    // TimeLoopAdaptive loop
     if (Message::GetOperatingInTimeLoopAdaptive())
-      Message::Warning("PETSc error %d", ierr);
+      Message::Warning("PETSc error: %s", text);
     else
-      Message::Error("PETSc error %d", ierr);
+      Message::Error("PETSc error: %s", text);
     Message::SetLastPETScError(ierr);
   }
 }
@@ -85,14 +78,17 @@ static int SolverInitialized = 0;
 
 void LinAlg_InitializeSolver(int* argc, char*** argv)
 {
+  if(SolverInitialized) return;
+  SolverInitialized = 1;
+
   // This function detects if MPI is initialized
   PetscInitialize(argc, argv, PETSC_NULL, PETSC_NULL);
+  PetscPopSignalHandler();
   MyPetscViewer = PETSC_VIEWER_STDOUT_SELF;
   MyComm = PETSC_COMM_WORLD;
 #if defined(HAVE_SLEPC)
   SlepcInitialize(argc, argv, PETSC_NULL, PETSC_NULL);
 #endif
-  SolverInitialized = 1;
 
   // get additional petsc options from specified file (useful e.g. on
   // Windows where we don't know where to search for ~/.petscrc)
@@ -109,6 +105,11 @@ void LinAlg_InitializeSolver(int* argc, char*** argv)
 
 void LinAlg_FinalizeSolver()
 {
+  // this causes random crashes when doing several initialize/finalize calls
+  // (when using getdp as a library). Until we figure out what's happening,
+  // let's simply initialize petsc/slepc once, and never finalize.
+  return;
+
   if(SolverInitialized){
 #if defined(HAVE_SLEPC)
     SlepcFinalize();
@@ -120,9 +121,9 @@ void LinAlg_FinalizeSolver()
 
 void LinAlg_SetCommSelf()
 {
+  Message::Info("Set communicator to SELF");
   MyComm = PETSC_COMM_SELF;
   Message::SetIsCommWorld(0);
-  Message::Info("Set communicator to SELF");
 }
 
 void LinAlg_SetCommWorld()
@@ -151,8 +152,13 @@ void LinAlg_CreateVector(gVector *V, gSolver *Solver, int n)
 
   // create sequential vector that will contain all the values on all
   // the procs
-  if(Message::GetCommSize() > 1)
+  if(Message::GetCommSize() > 1 && MyComm != PETSC_COMM_SELF){
     _try(VecCreateSeq(PETSC_COMM_SELF, n, &V->Vseq));
+    V->haveSeq = 1;
+  }
+  else{
+    V->haveSeq = 0;
+  }
 }
 
 void _fillseq(Vec &V, Vec &Vseq)
@@ -160,7 +166,7 @@ void _fillseq(Vec &V, Vec &Vseq)
   // collect all the values from the parallel petsc vector into a sequential
   // vector on each processor
   VecScatter ctx;
-  VecScatterCreateToAll(V, &ctx, &Vseq);
+  VecScatterCreateToAll(V, &ctx, NULL);
 #if (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR < 3)
   VecScatterBegin(V, Vseq, INSERT_VALUES, SCATTER_FORWARD, ctx);
   VecScatterEnd(V, Vseq, INSERT_VALUES, SCATTER_FORWARD, ctx);
@@ -169,7 +175,7 @@ void _fillseq(Vec &V, Vec &Vseq)
   VecScatterEnd(ctx, V, Vseq, INSERT_VALUES, SCATTER_FORWARD);
 #endif
 
-#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)) )
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
   VecScatterDestroy(&ctx);
 #else
   VecScatterDestroy(ctx);
@@ -178,8 +184,7 @@ void _fillseq(Vec &V, Vec &Vseq)
 
 static void _fillseq(gVector *V)
 {
-  if(Message::GetCommSize() == 1) return;
-  _fillseq(V->V, V->Vseq);
+  if(V->haveSeq) _fillseq(V->V, V->Vseq);
 }
 
 void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
@@ -212,16 +217,29 @@ void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
   for(int i = 0; i < nonloc; i++)
     nnz[Current.DofData->NonLocalEquations[i] - 1] = prealloc_full;
 
-  if(Message::GetCommSize() > 1) // FIXME: alloc full lines...
+  if(Message::GetCommSize() > 1){ // FIXME: alloc full lines...
 #if ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 3))
     _try(MatCreateAIJ(MyComm, PETSC_DECIDE, PETSC_DECIDE, n, m,
                       prealloc, PETSC_NULL, prealloc, PETSC_NULL, &M->M));
 #else
-  _try(MatCreateMPIAIJ(MyComm, PETSC_DECIDE, PETSC_DECIDE, n, m,
-                       prealloc, PETSC_NULL, prealloc, PETSC_NULL, &M->M));
+    _try(MatCreateMPIAIJ(MyComm, PETSC_DECIDE, PETSC_DECIDE, n, m,
+                         prealloc, PETSC_NULL, prealloc, PETSC_NULL, &M->M));
 #endif
-  else
+  }
+  else{
     _try(MatCreateSeqAIJ(PETSC_COMM_SELF, n, m, 0, &nnz[0], &M->M));
+    // PETSc (I)LU does not like matrices with empty (non assembled) diagonals
+
+    for(int i = 0; i < n; i++){
+      PetscScalar d = 0.;
+      _try(MatSetValues(M->M, 1, &i, 1, &i, &d, INSERT_VALUES));
+    }
+    _try(MatAssemblyBegin(M->M, MAT_FLUSH_ASSEMBLY));
+    _try(MatAssemblyEnd(M->M, MAT_FLUSH_ASSEMBLY));
+
+  }
+
+  //MatSetOption(M->M, MAT_NEW_NONZERO_ALLOCATION_ERR, PETSC_FALSE);
 
 #if ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 3))
   // Preallocation routines automatically set now MAT_NEW_NONZERO_ALLOCATION_ERR,
@@ -252,18 +270,16 @@ void LinAlg_DestroyVector(gVector *V)
 {
 #if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
   _try(VecDestroy(&V->V));
-  if(Message::GetCommSize() > 1)
-    _try(VecDestroy(&V->Vseq));
+  if(V->haveSeq) _try(VecDestroy(&V->Vseq));
 #else
   _try(VecDestroy(V->V));
-  if(Message::GetCommSize() > 1)
-    _try(VecDestroy(V->Vseq));
+  if(V->haveSeq) _try(VecDestroy(V->Vseq));
 #endif
 }
 
 void LinAlg_DestroyMatrix(gMatrix *M)
 {
-#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)) )
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
   _try(MatDestroy(&M->M));
 #else
   _try(MatDestroy(M->M));
@@ -278,8 +294,13 @@ void LinAlg_CopyScalar(gScalar *S1, gScalar *S2)
 void LinAlg_CopyVector(gVector *V1, gVector *V2)
 {
   _try(VecCopy(V1->V, V2->V));
-  if(Message::GetCommSize() > 1)
-    _try(VecCopy(V1->Vseq, V2->Vseq));
+  if(V1->haveSeq) _try(VecCopy(V1->Vseq, V2->Vseq));
+}
+
+void LinAlg_SwapVector(gVector *V1, gVector *V2)
+{
+  _try(VecSwap(V1->V, V2->V));
+  if(V1->haveSeq) _try(VecSwap(V1->Vseq, V2->Vseq));
 }
 
 void LinAlg_CopyMatrix(gMatrix *M1, gMatrix *M2)
@@ -297,8 +318,7 @@ void LinAlg_ZeroVector(gVector *V)
   PetscScalar zero = 0.0;
 
   _try(VecSet(V->V, zero));
-  if(Message::GetCommSize() > 1)
-    _try(VecSet(V->Vseq, zero));
+  if(V->haveSeq) _try(VecSet(V->Vseq, zero));
 }
 
 void LinAlg_ZeroMatrix(gMatrix *M)
@@ -379,7 +399,7 @@ void LinAlg_PrintVector(FILE *file, gVector *V, bool matlab,
   if(!matlab){
     PetscInt n;
     _try(VecGetSize(V->V, &n));
-    Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
+    Vec VV = V->haveSeq ? V->Vseq : V->V;
     PetscScalar *tmp;
     _try(VecGetArray(VV, &tmp));
     for (int i = 0; i < n; i++){
@@ -461,7 +481,7 @@ void LinAlg_WriteVector(FILE *file, gVector *V)
 {
   PetscInt n;
   _try(VecGetSize(V->V, &n));
-  Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
+  Vec VV = V->haveSeq ? V->Vseq : V->V;
   PetscScalar *tmp;
   _try(VecGetArray(VV, &tmp));
   fwrite(tmp, sizeof(PetscScalar), n, file);
@@ -546,7 +566,7 @@ void LinAlg_GetComplexInScalar(double *d1, double *d2, gScalar *S)
 
 void LinAlg_GetScalarInVector(gScalar *S, gVector *V, int i)
 {
-  Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
+  Vec VV = V->haveSeq ? V->Vseq : V->V;
   PetscScalar *tmp;
   _try(VecGetArray(VV, &tmp));
   S->s = tmp[i];
@@ -555,7 +575,7 @@ void LinAlg_GetScalarInVector(gScalar *S, gVector *V, int i)
 
 void LinAlg_GetDoubleInVector(double *d, gVector *V, int i)
 {
-  Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
+  Vec VV = V->haveSeq ? V->Vseq : V->V;
   PetscScalar *tmp;
   _try(VecGetArray(VV, &tmp));
 #if defined(PETSC_USE_COMPLEX)
@@ -568,7 +588,7 @@ void LinAlg_GetDoubleInVector(double *d, gVector *V, int i)
 
 void LinAlg_GetAbsDoubleInVector(double *d, gVector *V, int i)
 {
-  Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
+  Vec VV = V->haveSeq ? V->Vseq : V->V;
   PetscScalar *tmp;
   _try(VecGetArray(VV, &tmp));
 #if defined(PETSC_USE_COMPLEX)
@@ -581,7 +601,7 @@ void LinAlg_GetAbsDoubleInVector(double *d, gVector *V, int i)
 
 void LinAlg_GetComplexInVector(double *d1, double *d2, gVector *V, int i, int j)
 {
-  Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
+  Vec VV = V->haveSeq ? V->Vseq : V->V;
   PetscScalar *tmp;
   _try(VecGetArray(VV, &tmp));
 #if defined(PETSC_USE_COMPLEX)
@@ -646,8 +666,7 @@ void LinAlg_SetVector(gVector *V, double *v)
 {
   PetscScalar tmp = *v;
   _try(VecSet(V->V, tmp));
-  if(Message::GetCommSize() > 1)
-    _try(VecSet(V->Vseq, tmp));
+  if(V->haveSeq) _try(VecSet(V->Vseq, tmp));
 }
 
 void LinAlg_SetScalarInVector(gScalar *S, gVector *V, int i)
@@ -740,14 +759,24 @@ void LinAlg_AddScalarScalar(gScalar *S1, gScalar *S2, gScalar *S3)
 
 void LinAlg_DummyVector(gVector *V)
 {
-  if(Current.DofData->DummyDof)
-    Message::Error("DummyVector not yet implemented");
-  return;
+  PetscInt n;
+  PetscScalar zero = 0.0;
+
+  if (Current.DofData->DummyDof == NULL) return ;
+
+  _try(VecGetSize(V->V, &n));
+  for(PetscInt i = 0; i < n; i++)
+    if (Current.DofData->DummyDof[i]==1)
+      _try(VecSetValues(V->V, 1, &i, &zero, INSERT_VALUES));
 }
 
 void LinAlg_AddScalarInVector(gScalar *S, gVector *V, int i)
 {
   if(!_isInLocalRange(V, i)) return;
+
+  if(Current.DofData->DummyDof)
+    if(Current.DofData->DummyDof[i]==1) return ;
+
   PetscInt ti = i;
   _try(VecSetValues(V->V, 1, &ti, &S->s, ADD_VALUES));
 }
@@ -755,6 +784,10 @@ void LinAlg_AddScalarInVector(gScalar *S, gVector *V, int i)
 void LinAlg_AddDoubleInVector(double d, gVector *V, int i)
 {
   if(!_isInLocalRange(V, i)) return;
+
+  if(Current.DofData->DummyDof)
+    if(Current.DofData->DummyDof[i]==1) return ;
+
   PetscScalar tmp = d;
   PetscInt ti = i;
   _try(VecSetValues(V->V, 1, &ti, &tmp, ADD_VALUES));
@@ -763,19 +796,26 @@ void LinAlg_AddDoubleInVector(double d, gVector *V, int i)
 void LinAlg_AddComplexInVector(double d1, double d2, gVector *V, int i, int j)
 {
   PetscScalar tmp;
+  int iok=1, jok=1;
+
+  if(Current.DofData->DummyDof){
+    if(Current.DofData->DummyDof[i]==1) iok=0;
+    if(Current.DofData->DummyDof[j]==1) jok=0;
+  }
+
 #if defined(PETSC_USE_COMPLEX)
-  if(_isInLocalRange(V, i)){
+  if(_isInLocalRange(V, i) && iok && jok){
     PetscInt ti = i;
     tmp = d1 + PETSC_i * d2;
     _try(VecSetValues(V->V, 1, &ti, &tmp, ADD_VALUES));
   }
 #else
   PetscInt ti = i, tj = j;
-  if(_isInLocalRange(V, i)){
+  if(_isInLocalRange(V, i) && iok){
     tmp = d1;
     _try(VecSetValues(V->V, 1, &ti, &tmp, ADD_VALUES));
   }
-  if(_isInLocalRange(V, j)){
+  if(_isInLocalRange(V, j) && jok){
     tmp = d2;
     _try(VecSetValues(V->V, 1, &tj, &tmp, ADD_VALUES));
   }
@@ -785,6 +825,11 @@ void LinAlg_AddComplexInVector(double d1, double d2, gVector *V, int i, int j)
 void LinAlg_AddScalarInMatrix(gScalar *S, gMatrix *M, int i, int j)
 {
   if(!_isInLocalRange(M, i)) return;
+
+  if (Current.DofData->DummyDof)
+    if ( (Current.DofData->DummyDof[i]==1 || Current.DofData->DummyDof[j]==1) && (i!=j) )
+      return;
+
   PetscInt ti = i, tj = j;
   _try(MatSetValues(M->M, 1, &ti, 1, &tj, &S->s, ADD_VALUES));
 }
@@ -792,6 +837,11 @@ void LinAlg_AddScalarInMatrix(gScalar *S, gMatrix *M, int i, int j)
 void LinAlg_AddDoubleInMatrix(double d, gMatrix *M, int i, int j)
 {
   if(!_isInLocalRange(M, i)) return;
+
+  if (Current.DofData->DummyDof)
+    if ( (Current.DofData->DummyDof[i]==1 || Current.DofData->DummyDof[j]==1) && (i!=j) )
+      return;
+
   PetscScalar tmp = d;
   PetscInt ti = i, tj = j;
   _try(MatSetValues(M->M, 1, &ti, 1, &tj, &tmp, ADD_VALUES));
@@ -1152,6 +1202,8 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
   else
     Message::Info("Converged in %d iterations", its);
 
+  Current.KSPIts = its;
+
   for(PetscInt i = 0; i < n; i++){
     PetscScalar d;
 #if defined(PETSC_USE_COMPLEX)
@@ -1173,6 +1225,7 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
 static PetscErrorCode _myKspMonitor(KSP ksp, PetscInt it, PetscReal rnorm, void *mctx)
 {
   Message::Info("%3ld KSP Residual norm %14.12e", (long)it, rnorm);
+  // Current.KSPResidual = rnorm;
   return 0;
 }
 
@@ -1185,7 +1238,6 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
   PetscOptionsGetTruth(PETSC_NULL, "-zitsol", &zitsol, &set);
   if(zitsol){ _zitsol(A, B, X); return; }
 #endif
-//printf("INDEX SOLVER : %d\n", kspIndex);
   if(kspIndex < 0 || kspIndex > 9){
     Message::Error("Linear Solver index out of range (%d)", kspIndex);
     return;
@@ -1203,40 +1255,33 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
   if(kspIndex != 0)
     Message::Info("Using solver index %d", kspIndex);
 
-//printf("kspIndex = %d\n",kspIndex);
-
   if(!Solver->ksp[kspIndex]) {
     _try(KSPCreate(MyComm, &Solver->ksp[kspIndex]));
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 5)))
+    _try(KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M));
+#else
     _try(KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M, DIFFERENT_NONZERO_PATTERN));
+#endif
     _try(KSPMonitorSet(Solver->ksp[kspIndex], _myKspMonitor, PETSC_NULL, PETSC_NULL));
     PC pc;
     _try(KSPGetPC(Solver->ksp[kspIndex], &pc));
 
-    // set some default options
-    _try(KSPSetTolerances(Solver->ksp[kspIndex], 1.e-12, PETSC_DEFAULT, PETSC_DEFAULT,
-                          PETSC_DEFAULT));
+    // set some default options: use direct solver (PARDISO, MUMPS, UMFPACK, or
+    // native PETSc LU)
+    _try(KSPSetType(Solver->ksp[kspIndex], "preonly"));
+    _try(PCSetType(pc, PCLU));
 #if (PETSC_VERSION_MAJOR > 2) && defined(PETSC_HAVE_MUMPS)
-    // use MUMPS by default if available
-    _try(PCSetType(pc, PCLU));
     _try(PCFactorSetMatSolverPackage(pc, "mumps"));
-    _try(KSPSetType(Solver->ksp[kspIndex], "preonly"));
-#elif (PETSC_VERSION_MAJOR > 2) && defined(PETSC_HAVE_UMFPACK)
-    // otherwise use UMFPACK if available
-    _try(PCSetType(pc, PCLU));
+#elif (PETSC_VERSION_MAJOR > 2) && defined(PETSC_HAVE_MKL_PARDISO)
+    _try(PCFactorSetMatSolverPackage(pc, "mkl_pardiso"));
+#elif (PETSC_VERSION_MAJOR > 2) && (defined(PETSC_HAVE_UMFPACK) || defined(PETSC_HAVE_SUITESPARSE))
     _try(PCFactorSetMatSolverPackage(pc, "umfpack"));
-    _try(KSPSetType(Solver->ksp[kspIndex], "preonly"));
 #else
-    // otherwise use ILU(6) + GMRES
-    _try(PCSetType(pc, PCILU));
-#if (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR == 0)
-    _try(PCILUSetMatOrdering(pc, MATORDERING_RCM));
-    _try(PCILUSetLevels(pc, 6));
-#elif (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR < 3)
+    _try(PetscOptionsSetValue("-pc_factor_nonzeros_along_diagonal", "1e-12"));
+#if (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR < 3)
     _try(PCFactorSetMatOrdering(pc, MATORDERING_RCM));
-    _try(PCFactorSetLevels(pc, 6));
 #else
     _try(PCFactorSetMatOrderingType(pc, MATORDERINGRCM));
-    _try(PCFactorSetLevels(pc, 6));
 #endif
 #endif
 
@@ -1248,13 +1293,13 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
       // either we are on parallel (!GetIsCommWorld) or in sequential with rank
       // = 0 (GetIsCommWorld)
 
-#if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR == 4)
+#if (PETSC_VERSION_RELEASE == 0) || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 4))
       const char *ksptype = "";
       _try(KSPGetType(Solver->ksp[kspIndex], &ksptype));
       const char *pctype = "";
       _try(PCGetType(pc, &pctype));
 #else
- const KSPType ksptype;
+      const KSPType ksptype;
       _try(KSPGetType(Solver->ksp[kspIndex], &ksptype));
       const PCType pctype;
       _try(PCGetType(pc, &pctype));
@@ -1270,8 +1315,16 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
     }
   }
   else if(precond){
-    _try(KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M,
-                         DIFFERENT_NONZERO_PATTERN));
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 5)))
+    _try(KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M));
+#else
+    _try(KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M, DIFFERENT_NONZERO_PATTERN));
+#endif
+  }
+  else{
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 5)))
+    _try(KSPSetReusePreconditioner(Solver->ksp[kspIndex], PETSC_TRUE));
+#endif
   }
 
   _try(KSPSolve(Solver->ksp[kspIndex], B->V, X->V));
@@ -1282,17 +1335,17 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
   if(view && Message::GetVerbosity() > 5)
     _try(KSPView(Solver->ksp[kspIndex], MyPetscViewer));
 
+  PetscInt its;
+  _try(KSPGetIterationNumber(Solver->ksp[kspIndex], &its));
   if(!Message::GetCommRank() || !Message::GetIsCommWorld()){
-    PetscInt its;
-    _try(KSPGetIterationNumber(Solver->ksp[kspIndex], &its));
     if(its > 1) Message::Info("%d iterations", its);
   }
+  Current.KSPIts = its;
 }
 
 void LinAlg_Solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
                   int solverIndex)
 {
-//	printf("solverIndex = %d\n", solverIndex);
   _solve(A, B, Solver, X, 1, solverIndex);
 }
 
@@ -1302,73 +1355,71 @@ void LinAlg_SolveAgain(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
   _solve(A, B, Solver, X, 0, solverIndex);
 }
 
+void LinAlg_SetGlobalSolverOptions(const std::string &opt)
+{
+  _try(PetscOptionsInsertString(opt.c_str()));
+}
+
+
 extern void Generate_Residual (gVector *x, gVector *f) ;
 extern void Generate_FullJacobian (gVector *x, gMatrix *Jac) ;
 
 static PetscErrorCode _NLFormFunction(SNES snes, Vec x, Vec f, void *mctx)
 {
-  /*
-    snes - the SNES context
-    x 	 - input vector (solution at each NL iteration)
-    f    - vector to store function value (residual)
-    mctx - [optional] user-defined Jacobian context
-  */
-
   gVector gx, gf ;
   gx.V = x ;
+  gx.haveSeq = 0;
   gf.V = f ;
-
+  gf.haveSeq = 0;
   Generate_Residual(&gx, &gf) ;
 
   PetscScalar *ff ;
   _try(VecGetArray(gf.V, &ff)) ;
-
   PetscInt n;
   _try(VecGetSize(f, &n)) ;
   for(PetscInt i = 0; i < n; i++)
     _try(VecSetValues(f, 1, &i, &ff[i], INSERT_VALUES));
-
   _try(VecGetArray(f, &ff));
-
   return 0;
 }
 
+#if (PETSC_VERSION_MAJOR == 2) || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR < 5))
 static PetscErrorCode _NLFormJacobian(SNES snes, Vec x, Mat *J, Mat *PC,
-                                      MatStructure *flag, void *mctx)
+                                      MatStructure *flag,
+                                      void *mctx)
 {
-  /*
-    snes - the SNES context
-    x 	 - input vector
-    J    - Jacobian matrix
-    PC 	 - preconditioner matrix, usually the same as Jac
-    flag - flag indicating information about the preconditioner matrix structure
-           (same as flag in KSPSetOperators()), one of
-           SAME_NONZERO_PATTERN,DIFFERENT_NONZERO_PATTERN,SAME_PRECONDITIONER
-    mctx - [optional] user-defined Jacobian context
-  */
-
   gVector gx ;
   gx.V = x ;
-
+  gx.haveSeq = 0;
   gMatrix gJ ;
   gJ.M = *J ;
-
   Generate_FullJacobian(&gx, &gJ);
-
   *J = gJ.M ;
   *flag = DIFFERENT_NONZERO_PATTERN ;
-
   Message::Barrier();
   _try(MatAssemblyBegin(*J, MAT_FINAL_ASSEMBLY));
   _try(MatAssemblyEnd(*J, MAT_FINAL_ASSEMBLY));
-
   if (*PC != *J){
     _try(MatAssemblyBegin(*PC, MAT_FINAL_ASSEMBLY));
     _try(MatAssemblyEnd(*PC, MAT_FINAL_ASSEMBLY));
   }
-
   return 0;
 }
+#else
+static PetscErrorCode _NLFormJacobian(SNES snes, Vec x, Mat J, Mat PC,
+                                      void *mctx)
+{
+  gVector gx ;
+  gx.V = x ;
+  gx.haveSeq = 0;
+  gMatrix gJ ;
+  Generate_FullJacobian(&gx, &gJ);
+  //J = gJ.M;
+  MatCopy(gJ.M, J, SAME_NONZERO_PATTERN);
+  //Message::Barrier();
+  return 0;
+}
+#endif
 
 static PetscErrorCode _mySnesMonitor(SNES snes, PetscInt it, PetscReal rnorm, void *mctx)
 {
@@ -1376,7 +1427,6 @@ static PetscErrorCode _mySnesMonitor(SNES snes, PetscInt it, PetscReal rnorm, vo
   return 0;
 }
 
-// SNES - PETSC nonlinear solvers
 static void _solveNL(gMatrix *A, gVector *B, gMatrix *J, gVector *R, gSolver *Solver,
                      gVector *X, int precond, int solverIndex)
 {
@@ -1402,27 +1452,23 @@ static void _solveNL(gMatrix *A, gVector *B, gMatrix *J, gVector *R, gSolver *So
   if(solverIndex != 0)
     Message::Info("Using nonlinear solver index %d", solverIndex);
 
-  PetscTruth fd_jacobian = PETSC_FALSE, snes_fd = PETSC_FALSE ;
-
   // Setting nonlinear solver defaults
   if(!Solver->snes[solverIndex]) {
     _try(SNESCreate(MyComm, &Solver->snes[solverIndex]));
-    if(Message::UseSocket())
-      _try(SNESMonitorSet(Solver->snes[solverIndex], _mySnesMonitor,
-                          PETSC_NULL, PETSC_NULL));
+    _try(SNESMonitorSet(Solver->snes[solverIndex], _mySnesMonitor,
+                        PETSC_NULL, PETSC_NULL));
     _try(SNESSetTolerances(Solver->snes[solverIndex], 1.e-12, PETSC_DEFAULT,
-                           PETSC_DEFAULT, PETSC_DEFAULT,
-                           PETSC_DEFAULT));
+                           PETSC_DEFAULT, PETSC_DEFAULT, PETSC_DEFAULT));
 
     // override default options with those from database (if any)
     _try(SNESSetFromOptions(Solver->snes[solverIndex]));
-
-    PetscOptionsGetTruth(PETSC_NULL,"-fd_jacobian",&fd_jacobian,0);
-    PetscOptionsGetTruth(PETSC_NULL,"-snes_fd",&snes_fd,0);
+    PetscTruth fd_jacobian = PETSC_FALSE, snes_fd = PETSC_FALSE ;
+    PetscOptionsGetTruth(PETSC_NULL, "-fd_jacobian", &fd_jacobian, 0);
+    PetscOptionsGetTruth(PETSC_NULL, "-snes_fd", &snes_fd, 0);
     if (fd_jacobian || snes_fd) {
-      Message::Error("Finite Difference Jacobian not yet implemented");
+      //  Message::Error("Finite Difference Jacobian not yet implemented");
 
-#if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR == 4)
+#if (PETSC_VERSION_RELEASE == 0) || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 4))
       _try(SNESSetJacobian(Solver->snes[solverIndex], J->M, J->M,
                            SNESComputeJacobianDefault, PETSC_NULL));
 #else
@@ -1439,6 +1485,16 @@ static void _solveNL(gMatrix *A, gVector *B, gMatrix *J, gVector *R, gSolver *So
                          PETSC_NULL)); // R(x) = A(x)*x-b
   }
 
+
+  KSP ksp;
+  SNESGetKSP(Solver->snes[solverIndex], &ksp);
+  PC pc;
+  _try(KSPGetPC(ksp, &pc));
+  _try(KSPSetType(ksp, "preonly"));
+  _try(PCSetType(pc, PCLU));
+#if (PETSC_VERSION_MAJOR > 2) && defined(PETSC_HAVE_MUMPS)
+  _try(PCFactorSetMatSolverPackage(pc, "mumps"));
+#endif
   _try(SNESSolve(Solver->snes[solverIndex], PETSC_NULL, X->V));
 
   // copy result on all procs
