@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -26,7 +26,7 @@
 
 int     Flag_PRE = 0, Flag_CAL = 0, Flag_POS = 0, Flag_RESTART = 0;
 int     Flag_XDATA = 0, Flag_BIN = 0, Flag_SPLIT = 0, Flag_GMSH_VERSION = 1;
-int     Flag_NETWORK_CACHE = 0;
+int     Flag_NETWORK_CACHE = 0, Flag_CALLED_WITH_ONELAB_SERVER = 0;
 double  Flag_ORDER = -1.;
 char   *Name_Generic = 0, *Name_Path = 0;
 char   *Name_Resolution = 0;
@@ -41,7 +41,7 @@ static void Info(int level, char *arg0)
   case 0 :
     fprintf(stderr,
 	    "GetDP, a General environment for the treatment of Discrete Problems\n"
-	    "Copyright (C) 1997-2013 P. Dular, C. Geuzaine\n"
+	    "Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege\n"
 	    "Usage: %s [file] [options]\n"
 	    "Processing options:\n"
 	    "  -pre 'Resolution'         pre-processing\n"
@@ -62,6 +62,9 @@ static void Info(int level, char *arg0)
 	    "  -solver file              specify parameter file (default: .petscrc)\n"
 	    "  [PETsc options]           PETSc options (must be listed after [file])\n"
 #endif
+#if defined(HAVE_SLEPC)
+	    "  -slepc                    use SLEPc instead of Arpack as eigensolver\n"
+#endif
 #if defined(HAVE_SPARSKIT)
 	    "  -solver file              specify parameter file (default: solver.par)\n"
 	    "  -'Parameter' num          override value of solver parameter 'Parameter'\n"
@@ -73,7 +76,7 @@ static void Info(int level, char *arg0)
 	    "  -check                    interactive check of problem structure\n"
 	    "  -v num                    set verbosity level (default: 5)\n"
 	    "  -p num                    set progress indicator update (default: 10)\n"
-	    "  -onelab name [address]    communicate with OneLab (file or server address)\n"
+	    "  -onelab name [address]    communicate with ONELAB (file or server address)\n"
             "  -setnumber name value     set constant number name=value\n"
             "  -setstring name value     set constant string name=value\n"
 	    "  -version                  show version number\n"
@@ -85,29 +88,32 @@ static void Info(int level, char *arg0)
     break;
   case 2:
     fprintf(stderr, "Version          : %s\n", GETDP_VERSION);
-#if defined(HAVE_PETSC)
-#if defined(PETSC_USE_COMPLEX)
-    fprintf(stderr, "PETSc arithmetic : Complex\n");
-#else
-    fprintf(stderr, "PETSc arithmetic : Real\n");
-#endif
-#endif
     fprintf(stderr, "License          : %s\n", GETDP_SHORT_LICENSE);
     fprintf(stderr, "Build OS         : %s\n", GETDP_OS);
-    fprintf(stderr, "Build options    :%s\n", GETDP_CONFIG_OPTIONS);
     fprintf(stderr, "Build date       : %s\n", GETDP_DATE);
     fprintf(stderr, "Build host       : %s\n", GETDP_HOST);
-#if defined(HAVE_GMSH)
-    fprintf(stderr, "Gmsh version     : %s%s (%s)\n",
-            GMSH_VERSION, GMSH_EXTRA_VERSION, GMSH_DATE);
-    fprintf(stderr, "Gmsh options     :%s\n", GMSH_CONFIG_OPTIONS);
+    fprintf(stderr, "Build options    :%s\n", GETDP_CONFIG_OPTIONS);
+#if defined(HAVE_PETSC)
+    fprintf(stderr, "PETSc version    : %d.%d.%d (%s arithmetic)\n", PETSC_VERSION_MAJOR,
+            PETSC_VERSION_MINOR, PETSC_VERSION_SUBMINOR,
+#if defined(PETSC_USE_COMPLEX)
+            "complex"
+#else
+            "real"
 #endif
-    fprintf(stderr, "Packager         : %s\n", GETDP_PACKAGER);
+            );
+#endif
+#if defined(HAVE_GMSH)
+    fprintf(stderr, "Gmsh lib version : %s%s (%s)\n",
+            GMSH_VERSION, GMSH_EXTRA_VERSION, GMSH_DATE);
+    fprintf(stderr, "Gmsh lib options :%s\n", GMSH_CONFIG_OPTIONS);
+#endif
+    fprintf(stderr, "Packaged by      : %s\n", GETDP_PACKAGER);
     fprintf(stderr, "Web site         : http://www.geuz.org/getdp/\n");
     fprintf(stderr, "Mailing list     : getdp@geuz.org\n");
     break;
   }
-  exit(0);
+  Message::Exit(0);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -135,38 +141,39 @@ static void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *
       else if (!strcmp(argv[i]+1, "split"))  { Flag_SPLIT   = 1; i++; }
 
       else if (!strcmp(argv[i]+1, "socket")) {
-	i++;
-	if (i < argc && argv[i][0] != '-') {
-	  Message::InitializeSocket(argv[i]); i++;
-	}
-	else {
-	  Message::Error("Missing socket name");
-	}
+        i++;
+        if (i < argc && argv[i][0] != '-') {
+          Message::InitializeSocket(argv[i]); i++;
+        }
+        else {
+          Message::Error("Missing socket name");
+        }
       }
 
       else if (!strcmp(argv[i]+1, "onelab")) {
         i++;
-	if (i + 1 < argc && argv[i][0] != '-' && argv[i + 1][0] != '-') {
-	  Message::InitializeOnelab(argv[i], argv[i + 1]);
+        if (i + 1 < argc && argv[i][0] != '-' && argv[i + 1][0] != '-') {
+          Message::InitializeOnelab(argv[i], argv[i + 1]);
           i += 2;
-	}
-	else if (i < argc && argv[i][0] != '-') {
-	  Message::InitializeOnelab(argv[i], "");
+        }
+        else if (i < argc && argv[i][0] != '-') {
+          Message::InitializeOnelab(argv[i], "");
           i += 1;
-	}
-	else {
-	  Message::Error("Missing client name and/or address of OneLab server");
-	}
+        }
+        else {
+          Message::Error("Missing client name and/or address of ONELAB server");
+        }
       }
 
       else if (!strcmp(argv[i]+1, "setnumber")) {
         i++;
-	if (i + 1 < argc && argv[i][0] != '-' && argv[i + 1][0] != '-') {
-          CommandLineNumbers[argv[i]] = atof(argv[i + 1]);
+	if (i + 1 < argc && argv[i][0] != '-') {
+          CommandLineNumbers[argv[i]] = std::vector<double>(1, atof(argv[i + 1]));
           i += 2;
 	}
-        else
+        else{
           Message::Error("Missing name and/or value for number definition");
+        }
       }
 
       else if (!strcmp(argv[i]+1, "setstring")) {
@@ -175,8 +182,30 @@ static void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *
           CommandLineStrings[argv[i]] = argv[i + 1];
           i += 2;
 	}
-        else
+        else{
           Message::Error("Missing name and/or value for string definition");
+        }
+      }
+
+      else if (!strcmp(argv[i]+1, "setlist") ||
+               !strcmp(argv[i]+1, "setlistofnumbers")) {
+        i++;
+	if (i + 1 < argc && argv[i][0] != '-') {
+          std::string n(argv[i]);
+          std::vector<double> v;
+          int s = atoi(argv[i + 1]), j = 0;
+          i += 2;
+          while(j < s && i < argc){
+            v.push_back(atof(argv[i]));
+            i++; j++;
+          }
+          if(j < s)
+            Message::Error("Missing values in list (got %d instead of %d)", j, s);
+          CommandLineNumbers[n] = v;
+	}
+        else{
+          Message::Error("Missing name and/or value for definition of list of numbers");
+        }
       }
 
       else if (!strcmp(argv[i]+1, "restart")){
@@ -353,8 +382,7 @@ static void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *
       else if (!strcmp(argv[i]+1, "name")) {
 	i++;
 	if (i < argc && argv[i][0] != '-') {
-	  Name_Generic = (char*)Malloc((strlen(argv[i]) + 1) * sizeof(char));
-	  strcpy(Name_Generic, argv[i]); i++;
+	  Name_Generic = strSave(argv[i]); i++;
 	}
 	else {
 	  Message::Error("Missing string");
@@ -386,26 +414,28 @@ static void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *
 
   if(!strlen(pro)){
     Message::Error("Missing input file name");
-    Name_Generic = (char*)Malloc(sizeof(char));
-    strcpy(Name_Generic, "");
+    Name_Generic = strSave("");
     *sargc = 0;
   }
   else{
     if(!Name_Generic){
-      Name_Generic = (char*)Malloc((strlen(pro) + 1) * sizeof(char));
-      strcpy(Name_Generic, pro);
+      Name_Generic = strSave(pro);
       if(strcmp(pro+(strlen(pro)-4), ".pro") &&
 	 strcmp(pro+(strlen(pro)-4), ".PRO"))
 	strcat(pro,".pro");
       else
 	Name_Generic[strlen(pro)-4] = '\0';
     }
-    else if(strcmp(pro+(strlen(pro)-4), ".pro") &&
-	    strcmp(pro+(strlen(pro)-4), ".PRO"))
-      strcat(pro,".pro");
+    else{
+      std::string fix = Fix_RelativePath(Name_Generic, pro);
+      Free(Name_Generic);
+      Name_Generic = strSave(fix.c_str());
+      if(strcmp(pro+(strlen(pro)-4), ".pro") &&
+         strcmp(pro+(strlen(pro)-4), ".PRO"))
+        strcat(pro,".pro");
+    }
 
-    Name_Path = (char*)Malloc((strlen(Name_Generic) + 1) * sizeof(char));
-    strcpy(Name_Path, Name_Generic);
+    Name_Path = strSave(Name_Generic);
     i = strlen(Name_Path)-1;
     while(i >= 0 && Name_Path[i] != '/' && Name_Path[i] != '\\') i--;
     Name_Path[i+1] = '\0';
@@ -477,11 +507,10 @@ int MainLegacy(int argc, char *argv[])
     Message::Finalize();
     return Message::GetErrorCount();
   }
-
   Message::Info("Running '%s' [GetDP %s, %d node%s]", cmdline.c_str(),
                 GETDP_VERSION, Message::GetCommSize(),
                 Message::GetCommSize() > 1 ? "s" : "");
-  Message::Cpu(3, "Started");
+  Message::Cpu(3, true, true, true, true, "Started");
 
   if(sargc > 1){
     std::string solveropt("");
@@ -513,7 +542,7 @@ int MainLegacy(int argc, char *argv[])
 #if defined(HAVE_GMSH)
   Message::Info("Initializing Gmsh");
   GmshInitialize();
-  if(!GmshGetMessageHandler()){
+  if(!GmshGetMessageHandler() && !Flag_CALLED_WITH_ONELAB_SERVER){
     // do not set msg handler if one is provided (e.g. on Android/iOS)
     GmshMsg c;
     GmshSetMessageHandler(&c);
@@ -553,10 +582,10 @@ int MainLegacy(int argc, char *argv[])
   LinAlg_FinalizeSolver();
 
   Message::PrintErrorCounter("Run");
-  Message::Cpu(3, "Stopped");
+  Message::Cpu(3, true, true, true, true, "Stopped");
 
 #if defined(HAVE_GMSH)
-  GmshFinalize();
+  if(!Flag_CALLED_WITH_ONELAB_SERVER) GmshFinalize();
 #endif
 
   Free_GlobalVariables();
@@ -569,7 +598,10 @@ int MainLegacy(int argc, char *argv[])
 int GetDP(std::vector<std::string> &args, void *ptr)
 {
   onelab::server *onelabServer = (onelab::server*) ptr;
-  if(onelabServer != NULL) onelab::server::setInstance(onelabServer);
+  if(onelabServer != NULL){
+    onelab::server::setInstance(onelabServer);
+    Flag_CALLED_WITH_ONELAB_SERVER = 1;
+  }
   int argc = args.size();
   std::vector<char*> argv(argc + 1, (char*)0);
   for(int i = 0; i < argc; i++) argv[i] = (char*)args[i].c_str();
