@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -22,8 +22,15 @@ void  Format_PostElement(struct PostSubOperation * PSO_P, int Contour, int Store
 			 double *Dummy, struct PostElement *PE);
 
 void  Format_PostValue(int Format, int Flag_Comma, int Group_FunctionType,
-		       double Time, int iRegion, int numRegion, int NbrRegion,
-		       int NbrHarmonics, int HarmonicToTime, int Flag_NoNewLine,
+		       int iTime, double Time, int NbrTimeStep,
+                       int iRegion, int numRegion, int NbrRegion,
+		       int NbrHarmonics, int HarmonicToTime, int FourierTransform,
+                       int Flag_NoNewLine,
 		       struct Value * Value) ;
 
+void Pos_FourierTransform(int NbrTimeStep, int NbrRegion,
+                          double *Times, struct Value *TmpValues, int Size,
+                          int TypeOutput,
+                          int *NbrFreq, double **Frequencies, struct Value **OutValues);
+
 #endif
