@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -9,6 +9,7 @@
 #include "DofData.h"
 #include "GeoData.h"
 #include "Get_DofOfElement.h"
+#include "Cal_Quantity.h"
 #include "Pos_Print.h"
 #include "Pos_Format.h"
 #include "ListUtils.h"
@@ -32,12 +33,11 @@
 
 extern struct Problem Problem_S ;
 extern struct CurrentData Current ;
-
 extern int    Flag_BIN, Flag_GMSH_VERSION ;
-
 extern char   *Name_Path ;
 
 FILE *PostStream = stdout;
+char PostFileName[256];
 
 /* ------------------------------------------------------------------------ */
 /*  P o s _ F e m F o r m u l a t i o n                                     */
@@ -64,7 +64,6 @@ void  Pos_FemFormulation(struct Formulation       *Formulation_P,
 				  sizeof (struct QuantityStorage) ) ;
 
   for(i = 0 ; i < List_Nbr(Formulation_P->DefineQuantity) ; i++) {
-
     QuantityStorage.DefineQuantity = DefineQuantity_P0 + i ;
 
     if(QuantityStorage.DefineQuantity->Type == INTEGRALQUANTITY &&
@@ -144,13 +143,42 @@ int Pos_InitTimeSteps(struct PostSubOperation *PostSubOperation_P)
 {
   int iTime, NbTimeStep;
 
-  if(PostSubOperation_P->LastTimeStepOnly || PostSubOperation_P->AppendTimeStepToFileName){
+  // last time step only
+  if(PostSubOperation_P->LastTimeStepOnly ||
+     PostSubOperation_P->AppendTimeStepToFileName){
     iTime = List_Nbr(Current.DofData->Solutions) - 1;
     List_Reset(PostSubOperation_P->TimeStep_L);
     List_Add(PostSubOperation_P->TimeStep_L, &iTime);
     return 1;
   }
 
+  // specific time values
+  if(List_Nbr(PostSubOperation_P->TimeValue_L) ||
+     List_Nbr(PostSubOperation_P->TimeImagValue_L)){
+    List_Reset(PostSubOperation_P->TimeStep_L);
+    for(int i = 0; i < List_Nbr(Current.DofData->Solutions); i++){
+      Solution *s = (struct Solution*)List_Pointer(Current.DofData->Solutions, i);
+      int step = s->TimeStep;
+      double time = s->Time, timeImag = s->TimeImag;
+      for(int j = 0; j < List_Nbr(PostSubOperation_P->TimeValue_L); j++){
+        double t;
+        List_Read(PostSubOperation_P->TimeValue_L, j, &t);
+        if(fabs(t - time) < 1.e-15){
+          List_Insert(PostSubOperation_P->TimeStep_L, &step, fcmp_double);
+        }
+      }
+      for(int j = 0; j < List_Nbr(PostSubOperation_P->TimeImagValue_L); j++){
+        double t;
+        List_Read(PostSubOperation_P->TimeImagValue_L, j, &t);
+        if(fabs(t - timeImag) < 1.e-15)
+          List_Insert(PostSubOperation_P->TimeStep_L, &step, fcmp_double);
+      }
+    }
+    NbTimeStep = List_Nbr(PostSubOperation_P->TimeStep_L);
+    if(NbTimeStep) return NbTimeStep;
+  }
+
+  // specific time steps
   NbTimeStep = List_Nbr(PostSubOperation_P->TimeStep_L);
 
   if(!NbTimeStep || !PostSubOperation_P->FrozenTimeStepList){
@@ -179,14 +207,20 @@ void Pos_InitAllSolutions(List_T * TimeStep_L, int Index_TimeStep)
       (Current.DofData_P0+k)->CurrentSolution = (struct Solution*)
 	List_Pointer((Current.DofData_P0+k)->Solutions, Num_Solution) ;
 
-  Current.Time = Current.DofData->CurrentSolution->Time ;
-  Current.TimeImag = Current.DofData->CurrentSolution->TimeImag ;
-
-  if(List_Nbr(Current.DofData->Solutions) > TimeStepIndex)
-    Current.TimeStep = ((struct Solution*)List_Pointer
-                        (Current.DofData->Solutions, TimeStepIndex))->TimeStep ;
-  else // Warning: this can be wrong
+  if(TimeStepIndex >= 0 && TimeStepIndex < List_Nbr(Current.DofData->Solutions)){
+    Solution *Solution_P = ((struct Solution*)List_Pointer
+                            (Current.DofData->Solutions, TimeStepIndex));
+    Current.TimeStep = Solution_P->TimeStep ;
+    Current.Time = Solution_P->Time ;
+    Current.TimeImag = Solution_P->TimeImag ;
+  }
+  else{ // Warning: this can be wrong
     Current.TimeStep = TimeStepIndex;
+    if(Current.DofData->CurrentSolution){
+      Current.Time = Current.DofData->CurrentSolution->Time;
+      Current.TimeImag = Current.DofData->CurrentSolution->TimeImag;
+    }
+  }
 }
 
 /* ------------------------------------------------------------------------ */
@@ -320,45 +354,65 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
 		      struct PostProcessing    *PostProcessing_P,
 		      struct PostSubOperation  *PostSubOperation_P)
 {
-  // currently cannot postprocess in parallel!
-  if(Message::GetIsCommWorld() && Message::GetCommRank()) return;
-
   struct PostQuantity   *NCPQ_P = NULL, *CPQ_P = NULL ;
   double                 Pulsation ;
   int                    i, Order = 0 ;
-  char                   FileName[256], AddExt[100] ;
+
+  if(PostSubOperation_P->Type == POP_MERGE){
+    Message::SendMergeFileRequest(PostSubOperation_P->FileOut);
+    return;
+  }
 
   if(PostSubOperation_P->FileOut){
     if(PostSubOperation_P->FileOut[0] == '/' ||
        PostSubOperation_P->FileOut[0] == '\\'){
-      strcpy(FileName, PostSubOperation_P->FileOut);
+      strcpy(PostFileName, PostSubOperation_P->FileOut);
     }
     else{
-      strcpy(FileName, Name_Path);
-      strcat(FileName, PostSubOperation_P->FileOut);
+      strcpy(PostFileName, Name_Path);
+      strcat(PostFileName, PostSubOperation_P->FileOut);
+    }
+
+    if(PostSubOperation_P->AppendExpressionToFileName >= 0) {
+      struct Value Value ;
+      Get_ValueOfExpressionByIndex(PostSubOperation_P->AppendExpressionToFileName,
+                                   NULL, 0., 0., 0., &Value) ;
+      char AddExt[100];
+      if(PostSubOperation_P->AppendExpressionFormat)
+        sprintf(AddExt, PostSubOperation_P->AppendExpressionFormat, Value.Val[0]);
+      else
+        sprintf(AddExt, "%.16g", Value.Val[0]);
+      strcat(PostFileName, AddExt);
     }
 
     if(PostSubOperation_P->AppendTimeStepToFileName) {
-      /* We should implement something more general, like strings with
-	 tags (e.g., "file_%TimeStep.pos") */
+      char AddExt[100] ;
       sprintf(AddExt, "_%03d", (PostSubOperation_P->OverrideTimeStepValue >= 0) ?
               PostSubOperation_P->OverrideTimeStepValue : (int)Current.TimeStep) ;
-      strcat(FileName, AddExt);
+      strcat(PostFileName, AddExt);
     }
 
-    if(!PostSubOperation_P->CatFile) {
-      if((PostStream = FOpen(FileName, Flag_BIN ? "wb" : "w")))
-	Message::Direct(4, "          > '%s'", FileName) ;
+    if(PostSubOperation_P->AppendStringToFileName) {
+      strcat(PostFileName, PostSubOperation_P->AppendStringToFileName);
+    }
+
+    if(Message::GetIsCommWorld() && Message::GetCommRank()){
+      // in parallel mode (SetCommWorld), only rank 0 prints output
+      PostStream = NULL ;
+    }
+    else if(!PostSubOperation_P->CatFile) {
+      if((PostStream = FOpen(PostFileName, Flag_BIN ? "wb" : "w")))
+	Message::Direct(4, "          > '%s'", PostFileName) ;
       else{
-	Message::Error("Unable to open file '%s'", FileName) ;
+	Message::Error("Unable to open file '%s'", PostFileName) ;
         PostStream = stdout ;
       }
     }
     else {
-      if((PostStream = FOpen(FileName, Flag_BIN ? "ab" : "a")))
-	Message::Direct(4, "          >> '%s'", FileName) ;
+      if((PostStream = FOpen(PostFileName, Flag_BIN ? "ab" : "a")))
+	Message::Direct(4, "          >> '%s'", PostFileName) ;
       else{
-	Message::Error("Unable to open file '%s'", FileName) ;
+	Message::Error("Unable to open file '%s'", PostFileName) ;
         PostStream = stdout ;
       }
     }
@@ -379,7 +433,14 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
      PostSubOperation_P->Format != FORMAT_GMSH)
     Message::Warning("StoreInField only available with Gmsh output format");
 
-  if(PostSubOperation_P->CatFile == 2)  fprintf(PostStream, "\n\n") ;
+  if(PostSubOperation_P->StoreInMeshBasedField >= 0){
+    Flag_GMSH_VERSION = 2;
+    if(PostSubOperation_P->SubType != PRINT_ONELEMENTSOF ||
+       PostSubOperation_P->Depth != 1)
+      Message::Error("StoreInMeshBasedField not compatible with selected options");
+  }
+
+  if(PostStream && PostSubOperation_P->CatFile == 2)  fprintf(PostStream, "\n\n") ;
   /*  two blanks lines for -index in gnuplot  */
 
   Format_PostFormat(PostSubOperation_P) ;
@@ -437,7 +498,7 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
 
   Flag_GMSH_VERSION = oldVersion;
 
-  if(PostSubOperation_P->FileOut){
+  if(PostStream && PostSubOperation_P->FileOut){
     fclose(PostStream) ;
 
     if(PostSubOperation_P->SendToServer == NULL ||
@@ -445,11 +506,11 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
       if(PostSubOperation_P->Format == FORMAT_GMSH_PARSED ||
          PostSubOperation_P->Format == FORMAT_GMSH){
         // send merge request
-        Message::SendMergeFileRequest(FileName);
+        Message::SendMergeFileRequest(PostFileName);
       }
       // Add link to file
       Message::AddOnelabStringChoice(Message::GetOnelabClientName() + "/9Output files",
-                                     "file", FileName);
+                                     "file", PostFileName);
     }
 
     /* NewCoordinates print option: write a new mesh */
@@ -457,7 +518,7 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
 
 #if defined(HAVE_GMSH)
 
-      GmshMergeFile(std::string(FileName));
+      GmshMergeFile(std::string(PostFileName));
       int iview = PView::list.size() - 1;
       PViewData *data = PView::list[iview]->getData();
 
@@ -473,7 +534,7 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
           std::vector<double> xyz(3);
           if(!data->searchVector(v->x(), v->y(), v->z(), &xyz[0]))
             Message::Error("Did not find new coordinate Vector at point (%g,%g,%g) "
-                           "from file %s", v->x(), v->y(), v->z(), FileName);
+                           "from file %s", v->x(), v->y(), v->z(), PostFileName);
           newcoords[v] = xyz;
         }
       }
