@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -8,6 +8,7 @@
 #include <math.h>
 #include "ProData.h"
 #include "GeoData.h"
+#include "DofData.h"
 #include "Cal_PostQuantity.h"
 #include "Get_Geometry.h"
 #include "Get_DofOfElement.h"
@@ -79,13 +80,84 @@ struct CutEdge {
   struct  Value  *Value ;
 } ;
 
-struct xyzv{
+struct xyzv {
   double x,y,z;
   struct Value v;
   /*int nbvals; for time domain -> malloc Value *v... */
   int nboccurences;
 };
 
+struct ValMinMax {
+  struct Value Val, ValX, ValY, ValZ;
+};
+
+int CompareValue(const Value * valA_P, const Value * valB_P)
+{
+  double cmp=0, VecLengthSquA, VecLengthSquB;
+
+  //if (Current.NbrHar != 1)
+  //  Message::Error("Cannot compare multi-harmonic values");
+  // -> we compare the real part in this case
+
+  switch (valA_P->Type) {
+  case SCALAR:
+    cmp = valA_P->Val[0] - valB_P->Val[0];
+    break;
+  case VECTOR:
+    VecLengthSquA = valA_P->Val[0] * valA_P->Val[0] +
+                    valA_P->Val[1] * valA_P->Val[1] +
+                    valA_P->Val[2] * valA_P->Val[2];
+    VecLengthSquB = valB_P->Val[0] * valB_P->Val[0] +
+                    valB_P->Val[1] * valB_P->Val[1] +
+                    valB_P->Val[2] * valB_P->Val[2];
+    cmp = VecLengthSquA - VecLengthSquB;
+    break;
+  default:
+    Message::Error("Cannot compare values other than SCALAR and VECTOR");
+  }
+  if(cmp > 1.e-16)
+    return 1;
+  else if(cmp < -1.e-16)
+    return -1;
+  else
+    return 0;
+}
+
+void SetValMinMax(struct PostElement *PE_P,
+                  int    iNode,
+                  struct ValMinMax   *ValueMinMax_P)
+{
+  Cal_CopyValue(&PE_P->Value[iNode], &ValueMinMax_P->Val);
+  ValueMinMax_P->ValX.Val[0] = PE_P->x[iNode];
+  ValueMinMax_P->ValY.Val[0] = PE_P->y[iNode];
+  ValueMinMax_P->ValZ.Val[0] = PE_P->z[iNode];
+}
+
+void InitValMinMax(struct ValMinMax   *ValueMinMax_P,
+                   struct PostElement *PE_P)
+{
+  // Init ValueMin and ValueMax
+  ValueMinMax_P->ValX.Type = SCALAR;
+  ValueMinMax_P->ValY.Type = SCALAR;
+  ValueMinMax_P->ValZ.Type = SCALAR;
+  Cal_ZeroValue(&ValueMinMax_P->ValX);
+  Cal_ZeroValue(&ValueMinMax_P->ValY);
+  Cal_ZeroValue(&ValueMinMax_P->ValZ);
+  SetValMinMax(PE_P, 0, ValueMinMax_P);
+}
+
+void EvalMinMax(struct PostElement *PE_P,
+                struct ValMinMax   *ValueMin_P,
+                struct ValMinMax   *ValueMax_P)
+{
+  for(int iNode = 0 ; iNode < PE_P->NbrNodes ; iNode++) {
+    if (CompareValue(&PE_P->Value[iNode], &ValueMin_P->Val) < 0)
+      SetValMinMax(PE_P, iNode, ValueMin_P);
+    if (CompareValue(&PE_P->Value[iNode], &ValueMax_P->Val) > 0)
+      SetValMinMax(PE_P, iNode, ValueMax_P);
+  }
+}
+
 static int fcmp_xyzv(const void * a, const void * b)
 {
   struct xyzv *p1, *p2;
@@ -151,11 +223,23 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
   struct PostElement  * PE ;
   struct Value        * CumulativeValues ;
   struct xyzv           xyzv, *xyzv_P ;
+  struct ValMinMax      ValueMin, ValueMax ;
   Tree_T              * xyzv_T ;
   double  * Error=NULL, Dummy[5], d, x1, x2 ;
   int       jj, NbrGeo, iGeo, incGeo, NbrPost=0, iPost ;
   int       NbrTimeStep, iTime, iNode ;
   int       Store = 0, DecomposeInSimplex = 0, Depth ;
+  bool      StoreMinMax, ValueMinMaxInitialized;
+
+  /* Do we have to store min. and max. values? */
+  if (PSO_P->StoreMinInRegister  >= 0 || PSO_P->StoreMaxInRegister >= 0 ||
+      PSO_P->StoreMinXinRegister >= 0 || PSO_P->StoreMaxXinRegister >= 0 ||
+      PSO_P->StoreMinYinRegister >= 0 || PSO_P->StoreMaxYinRegister >= 0 ||
+      PSO_P->StoreMinZinRegister >= 0 || PSO_P->StoreMaxZinRegister >= 0) {
+    StoreMinMax = true;
+  }
+  else
+    StoreMinMax = false;
 
   /* Select the TimeSteps */
 
@@ -188,11 +272,6 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
 			       QuantityStorage_P0, &CumulativeValues);
   }
 
-  /* Init a search grid if we plot a NonCumulative quantity with OnGrid */
-
-  if(NCPQ_P && PSO_P->SubType == PRINT_ONGRID)
-    Init_SearchGrid(&Current.GeoData->Grid) ;
-
   /* If we compute a skin, apply smoothing, sort the results, or
      perform adaption, we'll need to store all the PostElements */
 
@@ -305,6 +384,7 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
   /* Loop on GeoElements */
 
   Message::ResetProgressMeter();
+  ValueMinMaxInitialized = false;
   for(iGeo = 0 ; iGeo < NbrGeo ; iGeo += incGeo) {
 
     if(Store){
@@ -346,7 +426,7 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
 	  for (iTime = 0 ; iTime < NbrTimeStep ; iTime++) {
 	    Pos_InitAllSolutions(PSO_P->TimeStep_L, iTime) ;
 	    for(iNode = 0 ; iNode < PE->NbrNodes ; iNode++){
-	      InWhichElement(Current.GeoData->Grid, Region_L, &Element,
+	      InWhichElement(&Current.GeoData->Grid, Region_L, &Element,
 			     PSO_P->Dimension,
 			     PE->x[iNode], PE->y[iNode], PE->z[iNode],
 			     &PE->u[iNode], &PE->v[iNode], &PE->w[iNode]) ;
@@ -361,6 +441,16 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
 		Combine_PostQuantity(PSO_P->CombinationType, Order,
 				     &PE->Value[iNode], &CumulativeValues[iNode]) ;
 	    }
+	    if (StoreMinMax) {
+	      if (!ValueMinMaxInitialized){
+	        // Init ValueMin and ValueMax
+                InitValMinMax(&ValueMin, PE);
+                InitValMinMax(&ValueMax, PE);
+                ValueMinMaxInitialized = true;
+	      }
+	      EvalMinMax(PE, &ValueMin, &ValueMax);
+	    }
+
 	    if(!Store)
 	      Format_PostElement(PSO_P, PSO_P->Iso, 0,
 				 Current.Time, iTime, NbrTimeStep,
@@ -386,10 +476,19 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
 	      Cal_PostQuantity(NCPQ_P, DefineQuantity_P0, QuantityStorage_P0,
 			       NULL, &Element,
 			       PE->u[iNode], PE->v[iNode], PE->w[iNode], &PE->Value[iNode]);
-	    if(CPQ_P)
-	      Combine_PostQuantity(PSO_P->CombinationType, Order,
-				   &PE->Value[iNode], &CumulativeValues[iTime]) ;
+              if(CPQ_P)
+                Combine_PostQuantity(PSO_P->CombinationType, Order,
+                                     &PE->Value[iNode], &CumulativeValues[iTime]) ;
 	    }
+	    if (StoreMinMax) {
+              if (!ValueMinMaxInitialized){
+                // Init ValueMin and ValueMax
+                InitValMinMax(&ValueMin, PE);
+                InitValMinMax(&ValueMax, PE);
+                ValueMinMaxInitialized = true;
+              }
+              EvalMinMax(PE, &ValueMin, &ValueMax);
+            }
 
 	    if(!Store)
 	      Format_PostElement(PSO_P, PSO_P->Iso, 0,
@@ -407,6 +506,26 @@ void  Pos_PrintOnElementsOf(struct PostQuantity     *NCPQ_P,
     if(Message::GetErrorCount()) break;
   } /* for iGeo */
 
+  /* Store minimum or maximum value in register */
+  if (StoreMinMax) {
+    if (PSO_P->StoreMinInRegister >= 0)
+      Cal_StoreInRegister(&ValueMin.Val,  PSO_P->StoreMinInRegister) ;
+    if (PSO_P->StoreMinXinRegister >= 0)
+      Cal_StoreInRegister(&ValueMin.ValX, PSO_P->StoreMinXinRegister) ;
+    if (PSO_P->StoreMinYinRegister >= 0)
+      Cal_StoreInRegister(&ValueMin.ValY, PSO_P->StoreMinYinRegister) ;
+    if (PSO_P->StoreMinZinRegister >= 0)
+      Cal_StoreInRegister(&ValueMin.ValZ, PSO_P->StoreMinZinRegister) ;
+    if (PSO_P->StoreMaxInRegister >= 0)
+      Cal_StoreInRegister(&ValueMax.Val, PSO_P->StoreMaxInRegister) ;
+    if (PSO_P->StoreMaxXinRegister >= 0)
+      Cal_StoreInRegister(&ValueMax.Val, PSO_P->StoreMaxXinRegister) ;
+    if (PSO_P->StoreMaxYinRegister >= 0)
+      Cal_StoreInRegister(&ValueMax.Val, PSO_P->StoreMaxYinRegister) ;
+    if (PSO_P->StoreMaxZinRegister >= 0)
+      Cal_StoreInRegister(&ValueMax.Val, PSO_P->StoreMaxZinRegister) ;
+  }
+
   /* Perform Smoothing */
 
   if(PSO_P->Smoothing){
@@ -881,7 +1000,7 @@ void  Pos_PrintOnSection(struct PostQuantity     *NCPQ_P,
    }                                                                    \
  }                                                                      \
  else{                                                                  \
-   InWhichElement(Current.GeoData->Grid, NULL, &Element, PSO_P->Dimension, \
+   InWhichElement(&Current.GeoData->Grid, NULL, &Element, PSO_P->Dimension, \
                   Current.x, Current.y, Current.z, &u, &v, &w) ;        \
    Current.Region = Element.Region ;                                    \
    if(Element.Num != NO_ELEMENT)                                        \
@@ -918,7 +1037,7 @@ void  Pos_PrintOnSection(struct PostQuantity     *NCPQ_P,
          ARRAY(i1,i2,k,ts) = (float)CumulativeValues[ts].Val[MAX_DIM*k] ; \
  }                                                                      \
  else{                                                                  \
-   InWhichElement(Current.GeoData->Grid, NULL, &Element, PSO_P->Dimension, \
+   InWhichElement(&Current.GeoData->Grid, NULL, &Element, PSO_P->Dimension, \
                   Current.x, Current.y, Current.z, &u, &v, &w) ;        \
    Current.Region = Element.Region ;                                    \
    for (ts = 0 ; ts < NbTimeStep ; ts++) {                              \
@@ -962,8 +1081,6 @@ void  Pos_PrintOnGrid(struct PostQuantity     *NCPQ_P,
 			       QuantityStorage_P0, &CumulativeValues);
   }
 
-  Init_SearchGrid(&Current.GeoData->Grid) ;
-
   Format_PostHeader(PSO_P, NbTimeStep, Order,
                     PSO_P->Label ? PSO_P->Label :
 		    (NCPQ_P ? NCPQ_P->Name : NULL),
@@ -980,6 +1097,11 @@ void  Pos_PrintOnGrid(struct PostQuantity     *NCPQ_P,
     Current.z = PSO_P->Case.OnGrid.z[0] ;
     Normal[0] = Normal[1] = Normal[2] = 0.0 ;
     LETS_PRINT_THE_RESULT ;
+
+    if (PSO_P->StoreInRegister >= 0)
+      Cal_StoreInRegister(&PE->Value[0], PSO_P->StoreInRegister) ;
+    if (PSO_P->StoreInVariable)
+      Cal_StoreInVariable(&PE->Value[0], PSO_P->StoreInVariable) ;
     break;
 
   case PRINT_ONGRID_1D :
@@ -1037,7 +1159,7 @@ void  Pos_PrintOnGrid(struct PostQuantity     *NCPQ_P,
 	  LETS_PRINT_THE_RESULT ;
 	}
       }
-      if(PSO_P->Depth < 2 && !Flag_BIN) fprintf(PostStream, "\n");
+      if(PostStream && PSO_P->Depth < 2 && !Flag_BIN) fprintf(PostStream, "\n");
     }
 
     if(PSO_P->Depth > 1){
@@ -1115,9 +1237,9 @@ void  Pos_PrintOnGrid(struct PostQuantity     *NCPQ_P,
 	  Current.z = Z[0] + (Z[1]-Z[0])*S[0] + (Z[2]-Z[0])*S[1] + (Z[3]-Z[0])*S[2] ;
 	  LETS_PRINT_THE_RESULT ;
 	}
-	if(!Flag_BIN) fprintf(PostStream, "\n");
+	if(PostStream && !Flag_BIN) fprintf(PostStream, "\n");
       }
-      if(!Flag_BIN) fprintf(PostStream, "\n\n");
+      if(PostStream && !Flag_BIN) fprintf(PostStream, "\n\n");
       /*  two blanks lines for -index in gnuplot  */
     }
     break;
@@ -1143,10 +1265,10 @@ void  Pos_PrintOnGrid(struct PostQuantity     *NCPQ_P,
 	  Normal[2] = Current.c ;
 	  LETS_PRINT_THE_RESULT ;
 	}
-	if(List_Nbr(PSO_P->Case.OnParamGrid.ParameterValue[2])>1 && !Flag_BIN)
+	if(PostStream && List_Nbr(PSO_P->Case.OnParamGrid.ParameterValue[2])>1 && !Flag_BIN)
 	  fprintf(PostStream, "\n");
       }
-      if(List_Nbr(PSO_P->Case.OnParamGrid.ParameterValue[1])>1 && !Flag_BIN)
+      if(PostStream && List_Nbr(PSO_P->Case.OnParamGrid.ParameterValue[1])>1 && !Flag_BIN)
 	fprintf(PostStream, "\n\n");
       /*  two blanks lines for -index in gnuplot  */
     }
@@ -1257,7 +1379,7 @@ void  Pos_PrintOnRegion(struct PostQuantity      *NCPQ_P,
 	}
         if(PostStream == stdout || PostStream == stderr)
           Message::Direct(sstream.str().c_str());
-        else
+        else if(PostStream)
           fprintf(PostStream, "%s\n", sstream.str().c_str()) ;
       }
     }
@@ -1275,9 +1397,6 @@ void  Pos_PrintOnRegion(struct PostQuantity      *NCPQ_P,
   else
     Nbr_Region = 1 ;
 
-  if (Type_Evaluation == LOCAL)
-    Init_SearchGrid(&Current.GeoData->Grid) ;
-
   for (iTime = 0 ; iTime < NbrTimeStep ; iTime++) {
 
     Pos_InitAllSolutions(PSO_P->TimeStep_L, iTime) ;
@@ -1313,7 +1432,7 @@ void  Pos_PrintOnRegion(struct PostQuantity      *NCPQ_P,
 	if (Group_FunctionType == NODESOF)
 	  Geo_GetNodesCoordinates(1, &Num_Region,
 				  &Current.x, &Current.y, &Current.z) ;
-	InWhichElement(Current.GeoData->Grid, NULL, &Element,
+	InWhichElement(&Current.GeoData->Grid, NULL, &Element,
 		       PSO_P->Dimension,
 		       Current.x, Current.y, Current.z, &u, &v, &w) ;
 
@@ -1324,8 +1443,9 @@ void  Pos_PrintOnRegion(struct PostQuantity      *NCPQ_P,
       if (PSO_P->Format != FORMAT_REGION_VALUE) {
         if (PSO_P->StoreInRegister >= 0)
           Cal_StoreInRegister(&Value, PSO_P->StoreInRegister) ;
-        if (PSO_P->SendToServer &&
-            strcmp(PSO_P->SendToServer, "No")){
+        if (PSO_P->StoreInVariable)
+          Cal_StoreInVariable(&Value, PSO_P->StoreInVariable) ;
+        if (PSO_P->SendToServer && strcmp(PSO_P->SendToServer, "No")){
           if(Value.Type == SCALAR)
             Message::AddOnelabNumberChoice(PSO_P->SendToServer,
                                            Value.Val[0], PSO_P->Color);
@@ -1336,8 +1456,10 @@ void  Pos_PrintOnRegion(struct PostQuantity      *NCPQ_P,
 
       Format_PostValue(PSO_P->Format, PSO_P->Comma,
 		       Group_FunctionType,
-		       Current.Time, i, Current.NumEntity, Nbr_Region,
+		       iTime, Current.Time, NbrTimeStep,
+                       i, Current.NumEntity, Nbr_Region,
 		       Current.NbrHar, PSO_P->HarmonicToTime,
+                       PSO_P->FourierTransform,
 		       PSO_P->NoNewLine,
 		       &Value) ;
 
@@ -1350,12 +1472,13 @@ void  Pos_PrintOnRegion(struct PostQuantity      *NCPQ_P,
         Message::ProgressMeter(i + 1, Nbr_Region, "Post-processing (OnRegion)");
     }
 
-    if (PSO_P->Format == FORMAT_REGION_VALUE) {
+    if (PostStream && PSO_P->Format == FORMAT_REGION_VALUE) {
       fprintf(PostStream, "%s", Print_Value_ToString(&ValueSummed).c_str());
       if (PSO_P->StoreInRegister >= 0)
         Cal_StoreInRegister(&ValueSummed, PSO_P->StoreInRegister) ;
-      if (PSO_P->SendToServer &&
-          strcmp(PSO_P->SendToServer, "No")){
+      if (PSO_P->StoreInVariable)
+        Cal_StoreInVariable(&ValueSummed, PSO_P->StoreInVariable) ;
+      if (PSO_P->SendToServer && strcmp(PSO_P->SendToServer, "No")){
         if(Value.Type == SCALAR)
           Message::AddOnelabNumberChoice(PSO_P->SendToServer,
                                          ValueSummed.Val[0], PSO_P->Color);
@@ -1426,8 +1549,9 @@ void  Pos_PrintWithArgument(struct PostQuantity      *NCPQ_P,
 
     Format_PostValue(PSO_P->Format, PSO_P->Comma,
                      REGION,
-                     x, 0, 0, 1,
+                     0, x, 1,  0, 0, 1,
                      Current.NbrHar, PSO_P->HarmonicToTime,
+                     PSO_P->FourierTransform,
                      PSO_P->NoNewLine,
                      &Value) ;
   }
@@ -1445,6 +1569,11 @@ void  Pos_PrintExpression(struct PostSubOperation *PSO_P)
   char *str2 = PSO_P->Case.Expression.String2;
   int expr = PSO_P->Case.Expression.ExpressionIndex;
 
+  if((!str || !strlen(str)) && (!str2 || !strlen(str2)) && expr < 0)
+    return; // nothing to print; useful to request merging an existing file
+
+  if(!PostStream) return;
+
   NbrTimeStep = Pos_InitTimeSteps(PSO_P);
 
   for(iTime = 0; iTime < NbrTimeStep; iTime++){
@@ -1463,7 +1592,8 @@ void  Pos_PrintExpression(struct PostSubOperation *PSO_P)
     }
     if(PSO_P->NoNewLine)
       fprintf(PostStream, " ") ;
-    else fprintf(PostStream, "\n") ;
+    else
+      fprintf(PostStream, "\n") ;
   }
 }
 
@@ -1482,6 +1612,10 @@ void  Pos_PrintGroup(struct PostSubOperation *PSO_P)
   double               y [NBR_MAX_NODES_IN_ELEMENT] ;
   double               z [NBR_MAX_NODES_IN_ELEMENT] ;
 
+  int                  numDofData, Code_BasisFunction, CodeExist = 0, k;
+  struct Dof  * Dof_P = NULL;
+  double               sizeEdge, Val_Dof, Val_Dof_i ;
+
   NbrGeo = Geo_GetNbrGeoElements() ;
 
   Format_PostHeader(PSO_P, 1, 0, PSO_P->Label, NULL);
@@ -1528,6 +1662,64 @@ void  Pos_PrintGroup(struct PostSubOperation *PSO_P)
 	}
 	break ;
 
+      case EDGESOF :
+	if(!GeoElement->NbrEdges) Geo_CreateEdgesOfElement(GeoElement) ;
+	for(i=0 ; i<GeoElement->NbrEdges ; i++){
+          NumNodes = Geo_GetNodesOfEdgeInElement(GeoElement, i) ;
+          SL->Index = iGeo;
+          SL->x[0] = x[abs(NumNodes[0])-1]; SL->x[1] = x[abs(NumNodes[1])-1];
+          SL->y[0] = y[abs(NumNodes[0])-1]; SL->y[1] = y[abs(NumNodes[1])-1];
+          SL->z[0] = z[abs(NumNodes[0])-1]; SL->z[1] = z[abs(NumNodes[1])-1];
+          SL->Value[0].Type = SL->Value[1].Type = SCALAR ;
+
+          //          SL->Value[0].Val[0] = SL->Value[1].Val[0] = fabs(GeoElement->NumEdges[i]);
+
+          // Dof : type, num, 0
+          if (List_Nbr(PSO_P->Value_L)<2)
+            Message::Error("Number of Values needed: 2");
+
+          numDofData         = int(*(double*)List_Pointer(PSO_P->Value_L, 0));
+          Code_BasisFunction = int(*(double*)List_Pointer(PSO_P->Value_L, 1));
+
+          CodeExist =
+            ((Dof_P =
+              Dof_GetDofStruct(Current.DofData_P0+ numDofData,
+                               Code_BasisFunction, abs(GeoElement->NumEdges[i]), 0))
+             != NULL) ;
+
+          if (CodeExist) {
+            sizeEdge = sqrt( SQU(SL->x[1]-SL->x[0]) + SQU(SL->y[1]-SL->y[0]) + SQU(SL->z[1]-SL->z[0]) );
+            if(Current.NbrHar==1){
+              Dof_GetRealDofValue(Current.DofData_P0+ numDofData, Dof_P, &Val_Dof) ;
+
+              Val_Dof = Val_Dof / sizeEdge ;
+
+              SL->Value[0].Val[0] = SL->Value[1].Val[0] = Val_Dof;
+            }
+            else{
+              for (k = 0 ; k < Current.NbrHar ; k+=2) {
+		Dof_GetComplexDofValue
+		  (Current.DofData_P0+ numDofData,
+                   Dof_P + k/2*gCOMPLEX_INCREMENT,
+		   &Val_Dof, &Val_Dof_i) ;
+
+              Val_Dof   = Val_Dof / sizeEdge ;
+              Val_Dof_i = Val_Dof_i / sizeEdge ;
+
+              SL->Value[0].Val[MAX_DIM*k    ] = SL->Value[1].Val[MAX_DIM*k    ] = Val_Dof;
+              SL->Value[0].Val[MAX_DIM*(k+1)] = SL->Value[1].Val[MAX_DIM*(k+1)] = Val_Dof_i;
+              }
+            }
+
+            Format_PostElement(PSO_P, PSO_P->Iso, 0,
+                               0, 0, 1,
+                               Current.NbrHar, PSO_P->HarmonicToTime,
+                               NULL, SL);
+          }
+
+	}
+	break ;
+
       case FACETSOFTREEIN :
 	if(!GeoElement->NbrFacets) Geo_CreateFacetsOfElement(GeoElement) ;
 	for(i=0 ; i<GeoElement->NbrFacets ; i++){
