@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2013 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2015 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@geuz.org>.
@@ -349,7 +349,12 @@ void Pre_GlobalTermOfFemEquation(int  Num_Region,
 
       switch(QuantityStorageDof_P->BasisFunction[0].Constraint) {
       case NONE:
-	if (QuantityStorageDof_P->BasisFunction[0].CodeAssociateBasisFunction > 0)
+	if (!QuantityStorageDof_P->BasisFunction[0].CodeAssociateBasisFunction)
+          Dof_DefineUnknownDof
+            (QuantityStorageDof_P->BasisFunction[0].CodeBasisFunction,
+             QuantityStorageDof_P->BasisFunction[0].CodeEntity, Current.NbrHar,
+             true) ;
+        else
 	  Dof_DefineAssociateDof
 	    (QuantityStorageDof_P->BasisFunction[0].CodeAssociateBasisFunction,
 	     QuantityStorageDof_P->BasisFunction[0].CodeEntity,
@@ -365,20 +370,20 @@ void Pre_GlobalTermOfFemEquation(int  Num_Region,
 	   QuantityStorageDof_P->BasisFunction[0].TimeFunctionIndex) ;
 	break ;
       case INIT:
+        /* This was used instead of what follows... Why!!!
 	  Dof_DefineAssociateDof
 	    (QuantityStorageDof_P->BasisFunction[0].CodeAssociateBasisFunction,
 	     QuantityStorageDof_P->BasisFunction[0].CodeEntity,
 	     QuantityStorageDof_P->BasisFunction[0].CodeBasisFunction,
 	     QuantityStorageDof_P->BasisFunction[0].CodeEntity, Current.NbrHar,
              1, QuantityStorageDof_P->BasisFunction[0].Value) ;
-          /*
+        */
 	Dof_DefineInitFixedDof
 	  (QuantityStorageDof_P->BasisFunction[0].CodeBasisFunction,
 	   QuantityStorageDof_P->BasisFunction[0].CodeEntity, Current.NbrHar,
 	   QuantityStorageDof_P->BasisFunction[0].Value,
 	   QuantityStorageDof_P->BasisFunction[0].Value2,
            true);
-          */
 	break;
       case ASSIGNFROMRESOLUTION:
 	Dof_DefineAssignSolveDof
@@ -433,13 +438,15 @@ void  Cst_GlobalTermOfFemEquation(int  Num_Region,
     switch(QuantityStorageEqu_P->BasisFunction[0].Constraint) {
     case ASSIGN:
       Dof_UpdateAssignFixedDof
-	(QuantityStorageEqu_P->BasisFunction[0].Dof, Current.NbrHar,
+	(QuantityStorageEqu_P->BasisFunction[0].CodeBasisFunction,
+         QuantityStorageEqu_P->BasisFunction[0].CodeEntity, Current.NbrHar,
 	 QuantityStorageEqu_P->BasisFunction[0].Value) ;
       break ;
     case CST_LINK:
     case CST_LINKCPLX:
       Dof_UpdateLinkDof
-	(QuantityStorageEqu_P->BasisFunction[0].Dof, Current.NbrHar,
+	(QuantityStorageEqu_P->BasisFunction[0].CodeBasisFunction,
+         QuantityStorageEqu_P->BasisFunction[0].CodeEntity, Current.NbrHar,
 	 QuantityStorageEqu_P->BasisFunction[0].Value,
 	 QuantityStorageEqu_P->BasisFunction[0].CodeEntity_Link) ;
       break;
@@ -452,13 +459,15 @@ void  Cst_GlobalTermOfFemEquation(int  Num_Region,
       switch(QuantityStorageDof_P->BasisFunction[0].Constraint) {
       case ASSIGN:
 	Dof_UpdateAssignFixedDof
-	  (QuantityStorageDof_P->BasisFunction[0].Dof, Current.NbrHar,
+	  (QuantityStorageDof_P->BasisFunction[0].CodeBasisFunction,
+           QuantityStorageDof_P->BasisFunction[0].CodeEntity, Current.NbrHar,
 	   QuantityStorageDof_P->BasisFunction[0].Value) ;
 	break ;
       case CST_LINK:
       case CST_LINKCPLX:
 	Dof_UpdateLinkDof
-	  (QuantityStorageDof_P->BasisFunction[0].Dof, Current.NbrHar,
+	  (QuantityStorageDof_P->BasisFunction[0].CodeBasisFunction,
+           QuantityStorageDof_P->BasisFunction[0].CodeEntity, Current.NbrHar,
 	   QuantityStorageDof_P->BasisFunction[0].Value,
 	   QuantityStorageDof_P->BasisFunction[0].CodeEntity_Link) ;
 	break;
@@ -669,7 +678,8 @@ void Cst_TermOfFemEquation(struct Element          * Element,
 	*/
       case ASSIGN:
 	Dof_UpdateAssignFixedDof
-	  (QuantityStorageEqu_P->BasisFunction[i].Dof, Current.NbrHar,
+	  (QuantityStorageEqu_P->BasisFunction[i].CodeBasisFunction,
+	   QuantityStorageEqu_P->BasisFunction[i].CodeEntity, Current.NbrHar,
 	   QuantityStorageEqu_P->BasisFunction[i].Value) ;
 	break;
 	/*
@@ -696,7 +706,8 @@ void Cst_TermOfFemEquation(struct Element          * Element,
       case CST_LINK:
       case CST_LINKCPLX:
 	Dof_UpdateLinkDof
-	  (QuantityStorageEqu_P->BasisFunction[i].Dof, Current.NbrHar,
+	  (QuantityStorageEqu_P->BasisFunction[i].CodeBasisFunction,
+	   QuantityStorageEqu_P->BasisFunction[i].CodeEntity, Current.NbrHar,
 	   QuantityStorageEqu_P->BasisFunction[i].Value,
 	   QuantityStorageEqu_P->BasisFunction[i].CodeEntity_Link) ;
 	break;
@@ -713,7 +724,8 @@ void Cst_TermOfFemEquation(struct Element          * Element,
       switch(QuantityStorageDof_P->BasisFunction[i].Constraint){
       case ASSIGN:
 	Dof_UpdateAssignFixedDof
-	  (QuantityStorageDof_P->BasisFunction[i].Dof, Current.NbrHar,
+	  (QuantityStorageDof_P->BasisFunction[i].CodeBasisFunction,
+	   QuantityStorageDof_P->BasisFunction[i].CodeEntity, Current.NbrHar,
 	   QuantityStorageDof_P->BasisFunction[i].Value) ;
 	break;
 	/*
@@ -739,7 +751,8 @@ void Cst_TermOfFemEquation(struct Element          * Element,
       case CST_LINK:
       case CST_LINKCPLX:
 	Dof_UpdateLinkDof
-	  (QuantityStorageDof_P->BasisFunction[i].Dof, Current.NbrHar,
+	  (QuantityStorageDof_P->BasisFunction[i].CodeBasisFunction,
+	   QuantityStorageDof_P->BasisFunction[i].CodeEntity, Current.NbrHar,
 	   QuantityStorageDof_P->BasisFunction[i].Value,
 	   QuantityStorageDof_P->BasisFunction[i].CodeEntity_Link) ;
 	break;
