@@ -202,7 +202,7 @@ void  Cal_FemGlobalEquation(struct EquationTerm    * EquationTerm_P,
 	    (double)(MCPR_P->Active->Case.Network.MatNode[i_Node][j_Branch]) ;
 	  if (Current.NbrHar > 1) {
 	    Val[1] = 0. ;
-	    for (k = 2 ; k < Current.NbrHar ; k += 2) {
+            for (k = 2 ; k < std::min(NBR_MAX_HARMONIC, Current.NbrHar) ; k += 2) {
 	      Val[k] = Val[0] ;  Val[k+1] = 0. ;
 	    }
 	  }
@@ -241,7 +241,7 @@ void  Cal_FemGlobalEquation(struct EquationTerm    * EquationTerm_P,
 	    (double)(MCPR_P->Active->Case.Network.MatLoop[i_Loop][j_Branch]) ;
 	  if (Current.NbrHar > 1) {
 	    Val[1] = 0. ;
-	    for (k = 2 ; k < Current.NbrHar ; k += 2) {
+            for (k = 2 ; k < std::min(NBR_MAX_HARMONIC, Current.NbrHar) ; k += 2) {
 	      Val[k] = Val[0] ;  Val[k+1] = 0. ;
 	    }
 	  }
@@ -485,7 +485,7 @@ void  Treatment_FemFormulation(struct Formulation * Formulation_P)
 	     * dont on prend la trace n'intervient qu'une seule fois dans le terme.
 	     * du a - manque de generalite du code au niveau de la gestion des
 	     *        espaces fonctionnels des fcts tests  pour 'Trace de Dof'
-	     *      - et Christophe fatigu
+	     *      - et Christophe fatigue
 	     */
 
 	    if (QuantityStorage_P->NumLastElementForFunctionSpace != Element.Num ||
@@ -573,9 +573,9 @@ void  Treatment_FemFormulation(struct Formulation * Formulation_P)
 	    }/* Flag_Only */
 	    break ;
 
-	    case _CST :
-	      Cst_TermOfFemEquation(&Element, EquationTerm_P, QuantityStorage_P0) ;
-	      break ;
+          case _CST :
+            Cst_TermOfFemEquation(&Element, EquationTerm_P, QuantityStorage_P0) ;
+            break ;
 
 	  }
 
@@ -639,7 +639,6 @@ void  Treatment_FemFormulation(struct Formulation * Formulation_P)
 
 	  /* Only one Function space analysis */
 	  /* -------------------------------- */
-
 	  if (QuantityStorage_P->NumLastElementForFunctionSpace != Num_Region) {
 	    QuantityStorage_P->NumLastElementForFunctionSpace = Num_Region ;
 
@@ -715,6 +714,7 @@ void  Treatment_FemFormulation(struct Formulation * Formulation_P)
   /* -------------------------- */
 
   List_Delete(FemLocalTermActive_L) ;  List_Delete(QuantityStorage_L) ;
+  Cal_EndGalerkinTermOfFemEquation();
 }
 
 /* ------------------------------------------------------------------------ */
@@ -747,4 +747,3 @@ void  Treatment_Formulation(struct Formulation * Formulation_P)
     break ;
   }
 }
-
